/*****************************************************************************
**  File Name:          cfs.c
**  Generated with:     Solstice TMN Agent Toolkit 2.0
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
**      The current file contains the implementation of the methods required
**      for the cfs class.
**      This class is defined in the following file:
**           cfs.gdmo
**
**----------------------------------------------------------------------------
**
**
**      The cfs class is registered as 1.3.6.1.4.1.42.2.2.3.20.2.2.1
**
**      The cfs class contains the following attributes:
**            cfsId
**            cfsName
**            cfsAutoUpdate
**            objectClass
**            nameBinding
**            packages
**            allomorphs
**
**      The cfs class supports the following actions:
**            cfsUpdateSystem
**
**      The cfs class supports the following notifications:
**            attributeValueChange
**
**      This class supports the create operation
**      This class supports the delete operation
**
**
*****************************************************************************/


/* 
** NOTE : 
** To find the places where you need to implement your code, 
** search for the string 'ADD YOUR'.
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifdef WIN32
#include <xom.h>
#include <xmp.h>
#include <xmp_cmis.h>
#include <usr_caDefs.h>
#include <usr_caGo.h>
#include <usr_caDmi.h>
#include <usr_caXom.h>
#include <usr_caCtrl.h>
#include <usr_caTree.h>
#else /* WIN32 */
#include <cmip/xom.h>
#include <cmip/xmp.h>
#include <cmip/xmp_cmis.h>
#include <tmn_agent/usr_caDefs.h>
#include <tmn_agent/usr_caGo.h>
#include <tmn_agent/usr_caDmi.h>
#include <tmn_agent/usr_caXom.h>
#include <tmn_agent/usr_caCtrl.h>
#include <tmn_agent/usr_caTree.h>
#endif /* WIN32 */
#include "metadata.h"

#include "cfs.h"

/* 
** Headers included for the example
*/
#include "cfsFile.h"
#include "cfsDir.h"
#include <dirent.h> 	
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h> 
#include <signal.h>

/*
** Define the name used for binding the static cfs object to the system object
*/
#define CFSID "Cfs"

/*
** The time interval in seconds for each autoupdate
*/
#define UPDATETIME 30

/*
** Define home and parent directories
*/
#define HOME "."
#define PARENT ".." 

/*
** Object class OID (encoded)
*/


static OM_descriptor cfs_objectClass = {
	MP_ATTRIBUTE_VALUE,
	OM_S_ENCODING_STRING,
	{ 15, "\x80\x0d\x2b\x06\x01\x04\x01\x2a\x02\x02\x03\x14\x02\x02\x01" }
} ; /* 1.3.6.1.4.1.42.2.2.3.20.2.2.1 */

/*
** cfs_init
*/

OM_return_code cfs_init(
	ca_context_t	*context,
	void		*caSupRef,
	void		**caCurRef
)
{


/*
** This callback is used for initializing
** the cfs class.
**
** The following parameters are used:
**
**	caSupRef:	caRef to the superior moi (input)
**	caCurRef:	caRef to an moi created by the function (output)
**
** This interface is defined for convenience and parameters used
** may be changed.
*/
	OM_return_code	ret = OM_SUCCESS ;
	
	/*
	** Creates the Cfs object and set the object attributes
	*/
	ret = cfs_createCfs(
		context,
		caSupRef,
		caCurRef
	) ;

	return ret  ;
}

/*
** cfs_create
*/

OM_return_code cfs_create(
	ca_context_t	*context,
	void		*caSupRef,
	ca_binding_t	*binding,
	OM_descriptor	*localName,
	void		**caRef,
	void		**accessRef,
	OM_descriptor	*error
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t *newInstance = NULL;

	*caRef = NULL;
	*accessRef = NULL;

	/*
	** Trace
	*/
	if (context->trace & CA_L_CALLBACK) {
		printf("CALL CREATE METHOD FOR cfs CLASS\n");
		if (binding != NULL) {
			printf("\tBinding    : %s\n", (binding->id)->name);
			if (binding->superior != NULL)
				printf("\tSuperior   : %s\n", 
						(binding->superior)->name);
			else
				printf("\tSuperior   : root\n") ;
			printf("\tNamed With : %s\n", 
						((binding->namedBy)->id)->name);
		}
		else {
			printf("\tNo name binding specified\n");
		}
	}

	/*
	** Allocate a new instance.
	*/
	newInstance = (cfs_t *)calloc(1, sizeof(cfs_t)) ;
	if (newInstance == NULL) {
		printf("cfs_create: calloc failed\n") ;
		ret = OM_FUNCTION_DECLINED ;
	}

	/*
	** Initialise the new instance :
	**
	** (1) check and initialize the naming attribute value
	*/
	if (ret == OM_SUCCESS) {

		if (binding->namedBy == &cfs_cfsId__def) {
			ret = dec_CFSObject_ASN1module_CfsId(
				localName,
				&(newInstance->cfsId)
			);
		}
		else { /* Name binding is invalid */
			if (context->trace & CA_L_CALLBACK) {
				printf("cfs_create : invalid name binding");
			}
			ret = OM_FUNCTION_DECLINED;
		}
		if (ret == OM_ENCODING_INVALID) {
			/* The naming value has a wrong syntax */
			error->type = MP_ERROR_STATUS;
			error->syntax = OM_S_ENUMERATION;
			error->value.enumeration = MP_E_INVALID_ATTRIBUTE_VALUE;
			if (context->trace & CA_L_CALLBACK) {
				printf("cfs_create : naming attribute syntax is invalid");
			}
			ret = OM_SUCCESS;
		}
	}

	/*
	** Initialise the new instance :
	**
	** (2) set default values for all the fields.
	*/
	if (ret == OM_SUCCESS) {
		newInstance->packagesPresent = OM_FALSE ;
		newInstance->allomorphsPresent = OM_FALSE ;
		newInstance->nameBinding = binding;

		/*
		** ADD YOUR CODE HERE:
		** Add your code to initialize remaining attribute values.
		*/
	}

	/*
	** Add the new instance to the MIT.
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_addInstWithSupInTreeFast(
			context,
			&cfs__def,
			binding->namedBy->id,
			localName,
			NULL,
			caSupRef,
			newInstance,
			caRef
		) ;
		if (ret == OM_SUCCESS) {
			ret = ca_registerFuncForDel(
				context,
				*caRef,
				cfs_free_instance
			) ;
		}
		if (ret != OM_SUCCESS) {
			printf("cfs_create : failed to add instance in tree !\n") ;
		}
	}

	/*
	** ADD YOUR CODE HERE:
	** The new instance is now created and inserted in the MIT.
	** You can add code here to update the managed resources.
	** In case of error, you need to :
	**	- remove newInstance from the MIT
	**	- set ret to OM_FUNCTION_DECLINED
	*/
	if (ret == OM_SUCCESS) {



	}


	/*
	** Undo the create if an error has occurred
	*/
	if ((ret != OM_SUCCESS) || (error->type == MP_ERROR_STATUS)) {
		if (*caRef != NULL) {
			ret = ca_delFinalNodeInTree(
				context,
				NULL,
				NULL,
				NULL,
				*caRef
			) ;
			/*
			**  Note that cfs_free_instance() 
			** is called automatically
			*/
			newInstance = NULL ;
		}
		if (newInstance != NULL) {
			ret = cfs_free_instance(
				context, 
				*caRef,
				newInstance
			) ;
		}
		ret = OM_FUNCTION_DECLINED ;
	}


	/*
	** Trace
	*/
	if (context->trace & CA_L_PRINT_TREE) {
		ca_examinTree(context) ;
	}
	if (context->trace & CA_L_CALLBACK) {
		printf("END OF CREATE METHOD FOR cfs CLASS\n") ;
	}

	return ret ;
}

/*
** cfs_delete
*/

OM_return_code cfs_delete(
	ca_context_t	*context,
	OM_sint32	invokeId,
	OM_boolean	type,
	void		*usrRef,
	void		*caRef,
	void		*accessRef
)
{
	OM_return_code ret = OM_SUCCESS ;
	OM_descriptor error = OM_NULL_DESCRIPTOR ;
	cfs_t *instance ;
	ca_list_t *children ;
	OM_uint childrenNb ;


	instance = (cfs_t *) usrRef ;

	/*
	** Trace
	*/
	if (context->trace & CA_L_CALLBACK) {
		printf("CALL DELETE METHOD FOR cfs CLASS\n") ;
	}

	/*
	** First check the deletion is authorized by the name binding.
	*/
	if ((instance->nameBinding->oper & CA_N_DELETE) == 0) {
		ret = OM_FUNCTION_DECLINED;
		if (context->trace & CA_L_CALLBACK) {
			printf("cfs_delete: deletion not supported\n") ;
		}
	}
	else if (instance->nameBinding->oper & CA_N_DELETE_NO) {
		ret = ca_findChildOfInst(context, NULL, NULL, usrRef, NULL,
						&children, &childrenNb) ;
		if (ret == OM_SUCCESS) {
			ca_freeListOfInst(children);
			if (childrenNb >= 1) {
				ret = OM_FUNCTION_DECLINED;
				if (context->trace & CA_L_CALLBACK) {
					printf("cfs_delete: deletion not supported\n") ;
				}
			}
		}
	}

	/*
	** ADD YOUR CODE HERE:
	** The instance is going to be deleted soon.
	** You can add code here to update the managed resources.
	** If an error occurs, set ret to OM_FUNCTION_DECLINED.
	*/
	if (ret == OM_SUCCESS) {


	}

	/*
	** Remove the instance from the tree.
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_delAllSubNodesInTree(context, NULL, NULL, NULL, caRef);
		if (ret == OM_SUCCESS) 
			ret = ca_delFinalNodeInTree(context, NULL, NULL, NULL, caRef);
		if (ret != OM_SUCCESS) {
			printf("cfs_delete : failed to remove instance !\n");
		}
		/*
		** Note that cfs_free_instance() 
		** is called automatically
		*/
	}

	/*
	** Call ca_gom_delete_rsp().
	*/
	if (type == OM_TRUE) {
		if (ret != OM_SUCCESS) {
			error.type = MP_ERROR_STATUS;
			error.syntax = OM_S_ENUMERATION;
			error.value.enumeration = MP_E_PROCESSING_FAILURE;
		}
		ret = ca_gom_delete_rsp(
			context, 
			invokeId, 
			usrRef, 
			accessRef, 
			&error
		);
	}

	/*
	** Trace
	*/
	if (context->trace & CA_L_PRINT_TREE) {
		ca_examinTree(context) ;
	}
	if (context->trace & CA_L_CALLBACK) {
		printf("END OF DELETE METHOD FOR cfs CLASS\n") ;
	}

	return ret ;
}


/*
** cfs_free_instance
*/

OM_return_code cfs_free_instance(
	ca_context_t	*context,
	void		*caRef,
	void		*usrRef
)
{
	cfs_t *instance = (cfs_t *)usrRef;

	free_CFSObject_ASN1module_CfsId(&(instance->cfsId));
	free_CFSObject_ASN1module_CfsDirectory(&(instance->cfsName));
	free_CFSObject_ASN1module_AutoUpdate(&(instance->cfsAutoUpdate));
	free_Attribute_ASN1Module_Packages(&(instance->packages));
	free_Attribute_ASN1Module_Allomorphs(&(instance->allomorphs));
	if (instance->attrValChngLst != NULL)
		ca_dmi_freeAttrChngLst(context, instance->attrValChngLst) ;

	/*
	** ADD YOUR CODE HERE:
	** If you have added some fields to the instance structure, you
	** can add code here to free the memory allocated to these fields.
	*/



	free(instance) ;

	return OM_SUCCESS ;
}


/*
** cfs_check
*/

OM_return_code cfs_check(
	ca_context_t	*context,
	OM_sint		oper,
	void 		*caRef,
	void 		*usrRef
)
{
	OM_return_code	ret ;
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;


	/*
	** Notification management
	*/
	if (oper == MP_SET_IND) {
		/*
		** Emit attribute value change notification.
		*/
		if (instance->attrValChngLst != NULL) {
			ret= ca_dmi_attrValChngNotif(
				context,
				usrRef,
				CA_DMI_T_SOURCEINDICATOR_management,
				-1,
				instance->attrValChngLst,
				NULL
			) ;
			if (ret == OM_SUCCESS)
				instance->attrValChngLst = NULL ;
		}
	}

	/*
	** Clean remaining notifications.
	*/
	if (instance->attrValChngLst != NULL) {
		ca_dmi_freeAttrChngLst(context, instance->attrValChngLst) ;
		instance->attrValChngLst = NULL ;
	}

	return OM_SUCCESS ;
}


/*
** cfs_cfsId_get
*/

OM_return_code cfs_cfsId_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;
	ret = enc_CFSObject_ASN1module_CfsId(
		&(instance->cfsId),
		value
	);

	return ret ;
}


/*
** cfs_cfsId_set
*/

OM_return_code cfs_cfsId_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
)
{
	OM_return_code ret;
	cfs_t *instance;
	CFSObject_ASN1module_CfsId decodedValue;

	instance = (cfs_t *) usrRef ;

	/*
	** Decode the new value
	*/
	ret = dec_CFSObject_ASN1module_CfsId(valueForOperation, &decodedValue) ;
	if (ret == OM_SUCCESS) {
		ret = ca_copy_xom_value(resultingValue, valueForOperation) ;
		if (ret != OM_SUCCESS) {
			free_CFSObject_ASN1module_CfsId(&decodedValue) ;
		}
	}
	else if (ret == OM_ENCODING_INVALID) {
		resultingValue->type = MP_ERROR_STATUS;
		resultingValue->syntax = OM_S_ENUMERATION;
		resultingValue->value.enumeration =
					MP_E_INVALID_ATTRIBUTE_VALUE;
	}
	/*
	** Copy the decoded value in the managed object
	*/
	if (ret == OM_SUCCESS) {
		free_CFSObject_ASN1module_CfsId(&(instance->cfsId)) ;
		instance->cfsId = decodedValue ;
	}

	/*
	** Include the attribute in the attribute value change notification
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_dmi_bldAttrChngLst(
			context, 
			&(instance->attrValChngLst),
			&cfsId__id, 
			NULL, 
			valueForOperation
		) ;
	}

	/*
	** ADD YOUR CODE HERE:
	** Managed object is now updated.
	** You can code here to update the managed resource.
	*/
	if (ret == OM_SUCCESS) {


	}

	/*
	** When a CMIS error is reported, return code must be cleared.
	*/
	if (resultingValue->type == MP_ERROR_STATUS) {
		ret = OM_SUCCESS ;
	}

	return ret ;
}


/*
** cfs_cfsId_eval
*/

OM_return_code cfs_cfsId_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t *instance;
	CFSObject_ASN1module_CfsId decodedValue;

	instance = (cfs_t *) usrRef ;

	if (type == MP_PRESENT) {
		*result = OM_TRUE ;
		*error = OM_FALSE ;
	}
	else {
		/*
		** Decode value1
		*/
		ret = dec_CFSObject_ASN1module_CfsId(value1, &decodedValue) ;

		/*
		** Compare the decoded value to the value stored
		** in the instance structure.
		*/
		if (ret == OM_SUCCESS) {
			switch(type) {

			case MP_EQUALITY :
			case MP_GREATER_OR_EQUAL :
			case MP_LESS_OR_EQUAL :
			case MP_SUBSTRINGS :
				*result = cmp_CFSObject_ASN1module_CfsId(
					&decodedValue,
					&instance->cfsId,
					type
				) ;
				*error = OM_FALSE ;
				break ;

			default :
				*result = OM_FALSE ;
				*error = OM_TRUE ;
				break ;

			}
		}
		else if (ret == OM_ENCODING_INVALID) {
			*error = OM_TRUE ;
			ret = OM_SUCCESS ; 
		}

		/*
		** Cleaning
		*/
		if (ret == OM_SUCCESS)
			free_CFSObject_ASN1module_CfsId(&decodedValue) ;
	}

	return ret ;
}


/*
** cfs_cfsName_get
*/

OM_return_code cfs_cfsName_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;
	ret = enc_CFSObject_ASN1module_CfsDirectory(
		&(instance->cfsName),
		value
	);

	return ret ;
}


/*
** cfs_cfsName_set
*/

OM_return_code cfs_cfsName_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
)
{
	OM_return_code ret;
	cfs_t *instance;
	CFSObject_ASN1module_CfsDirectory decodedValue;

	instance = (cfs_t *) usrRef ;

	/*
	** Decode the new value
	*/
	ret = dec_CFSObject_ASN1module_CfsDirectory(valueForOperation, &decodedValue) ;
	if (ret == OM_SUCCESS) {
		ret = ca_copy_xom_value(resultingValue, valueForOperation) ;
		if (ret != OM_SUCCESS) {
			free_CFSObject_ASN1module_CfsDirectory(&decodedValue) ;
		}
	}
	else if (ret == OM_ENCODING_INVALID) {
		resultingValue->type = MP_ERROR_STATUS;
		resultingValue->syntax = OM_S_ENUMERATION;
		resultingValue->value.enumeration =
					MP_E_INVALID_ATTRIBUTE_VALUE;
	}
	/*
	** Copy the decoded value in the managed object
	*/
	if (ret == OM_SUCCESS) {
		free_CFSObject_ASN1module_CfsDirectory(&(instance->cfsName)) ;
		instance->cfsName = decodedValue ;
	}

	/*
	** Include the attribute in the attribute value change notification
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_dmi_bldAttrChngLst(
			context, 
			&(instance->attrValChngLst),
			&cfsName__id, 
			NULL, 
			valueForOperation
		) ;
	}

	/*
	** ADD YOUR CODE HERE:
	** Managed object is now updated.
	** You can code here to update the managed resource.
	*/
	if (ret == OM_SUCCESS) {


	}

	/*
	** When a CMIS error is reported, return code must be cleared.
	*/
	if (resultingValue->type == MP_ERROR_STATUS) {
		ret = OM_SUCCESS ;
	}

	return ret ;
}


/*
** cfs_cfsName_eval
*/

OM_return_code cfs_cfsName_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t *instance;
	CFSObject_ASN1module_CfsDirectory decodedValue;

	instance = (cfs_t *) usrRef ;

	if (type == MP_PRESENT) {
		*result = OM_TRUE ;
		*error = OM_FALSE ;
	}
	else {
		/*
		** Decode value1
		*/
		ret = dec_CFSObject_ASN1module_CfsDirectory(value1, &decodedValue) ;

		/*
		** Compare the decoded value to the value stored
		** in the instance structure.
		*/
		if (ret == OM_SUCCESS) {
			switch(type) {

			case MP_EQUALITY :
			case MP_GREATER_OR_EQUAL :
			case MP_LESS_OR_EQUAL :
			case MP_SUBSTRINGS :
				*result = cmp_CFSObject_ASN1module_CfsDirectory(
					&decodedValue,
					&instance->cfsName,
					type
				) ;
				*error = OM_FALSE ;
				break ;

			default :
				*result = OM_FALSE ;
				*error = OM_TRUE ;
				break ;

			}
		}
		else if (ret == OM_ENCODING_INVALID) {
			*error = OM_TRUE ;
			ret = OM_SUCCESS ; 
		}

		/*
		** Cleaning
		*/
		if (ret == OM_SUCCESS)
			free_CFSObject_ASN1module_CfsDirectory(&decodedValue) ;
	}

	return ret ;
}


/*
** cfs_cfsAutoUpdate_get
*/

OM_return_code cfs_cfsAutoUpdate_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;
	ret = enc_CFSObject_ASN1module_AutoUpdate(
		&(instance->cfsAutoUpdate),
		value
	);

	return ret ;
}


/*
** cfs_cfsAutoUpdate_set
*/

OM_return_code cfs_cfsAutoUpdate_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
)
{
	OM_return_code ret;
	cfs_t *instance;
	CFSObject_ASN1module_AutoUpdate decodedValue;

	instance = (cfs_t *) usrRef ;

	/*
	** Decode the new value
	*/
	ret = dec_CFSObject_ASN1module_AutoUpdate(valueForOperation, &decodedValue) ;
	if (ret == OM_SUCCESS) {
		ret = ca_copy_xom_value(resultingValue, valueForOperation) ;
		if (ret != OM_SUCCESS) {
			free_CFSObject_ASN1module_AutoUpdate(&decodedValue) ;
		}
	}
	else if (ret == OM_ENCODING_INVALID) {
		resultingValue->type = MP_ERROR_STATUS;
		resultingValue->syntax = OM_S_ENUMERATION;
		resultingValue->value.enumeration =
					MP_E_INVALID_ATTRIBUTE_VALUE;
	}
	/*
	** Copy the decoded value in the managed object
	*/
	if (ret == OM_SUCCESS) {
		free_CFSObject_ASN1module_AutoUpdate(&(instance->cfsAutoUpdate)) ;
		instance->cfsAutoUpdate = decodedValue ;
	}

	/*
	** Include the attribute in the attribute value change notification
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_dmi_bldAttrChngLst(
			context, 
			&(instance->attrValChngLst),
			&cfsAutoUpdate__id, 
			NULL, 
			valueForOperation
		) ;
	}

	/*
	** ADD YOUR CODE HERE:
	** Managed object is now updated.
	** You can code here to update the managed resource.
	*/
	if (ret == OM_SUCCESS) {
		/*
		** Activate the automatic update of the CFS object tree (MIT)
		*/
		if (instance->cfsAutoUpdate != OM_FALSE) {
			/*
			** Register a signal handler for the alarm signal
			*/
			if (signal(SIGALRM,cfs_timerproc) == SIG_ERR)
				ret = OM_FUNCTION_DECLINED ;

			if (ret == OM_SUCCESS) {	
				/*
				** Set the alarm clock to send a signal after 
				** an interval of UPDATETIME seconds
				*/
				alarm(UPDATETIME) ;
				updateUsrRef = usrRef ;
				updateContext = context ;
				printf("CFS will be automatically updated every %d seconds\n\n",UPDATETIME) ;
			}
		}
		else {
		/*
		** De-activate the automatic update of the CFS object tree (MIT)
		*/
			/*
			** Cancel the alarm clock 
			*/
			updateUsrRef = NULL ;
			updateContext = NULL ;
			alarm(0) ;
		}
	}

	/*
	** When a CMIS error is reported, return code must be cleared.
	*/
	if (resultingValue->type == MP_ERROR_STATUS) {
		ret = OM_SUCCESS ;
	}

	return ret ;
}


/*
** cfs_cfsAutoUpdate_eval
*/

OM_return_code cfs_cfsAutoUpdate_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t *instance;
	CFSObject_ASN1module_AutoUpdate decodedValue;

	instance = (cfs_t *) usrRef ;

	if (type == MP_PRESENT) {
		*result = OM_TRUE ;
		*error = OM_FALSE ;
	}
	else {
		/*
		** Decode value1
		*/
		ret = dec_CFSObject_ASN1module_AutoUpdate(value1, &decodedValue) ;

		/*
		** Compare the decoded value to the value stored
		** in the instance structure.
		*/
		if (ret == OM_SUCCESS) {
			switch(type) {

			case MP_EQUALITY :
				*result = cmp_CFSObject_ASN1module_AutoUpdate(
					&decodedValue,
					&instance->cfsAutoUpdate,
					type
				) ;
				*error = OM_FALSE ;
				break ;

			default :
				*result = OM_FALSE ;
				*error = OM_TRUE ;
				break ;

			}
		}
		else if (ret == OM_ENCODING_INVALID) {
			*error = OM_TRUE ;
			ret = OM_SUCCESS ; 
		}

		/*
		** Cleaning
		*/
		if (ret == OM_SUCCESS)
			free_CFSObject_ASN1module_AutoUpdate(&decodedValue) ;
	}

	return ret ;
}


/*
** cfs_objectClass_get
*/

OM_return_code cfs_objectClass_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	value->syntax = cfs_objectClass.syntax ;
	value->value  = cfs_objectClass.value ;

	return OM_SUCCESS;
}


/*
** cfs_nameBinding_get
*/

OM_return_code cfs_nameBinding_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;
	if (instance->nameBinding != NULL) {
		value->syntax = instance->nameBinding->id->priv.syntax ;
		value->value = instance->nameBinding->id->priv.value ;
	}
	else {
		value->syntax = OM_S_NULL ;
	}

	return OM_SUCCESS;
}


/*
** cfs_packages_get
*/

OM_return_code cfs_packages_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;
	if (instance->packagesPresent == OM_TRUE) {
		ret = enc_Attribute_ASN1Module_Packages(
			&(instance->packages), 
			value
		) ;
	}
	else {
		value->type = MP_ERROR_STATUS;
		value->syntax = OM_S_ENUMERATION;
		value->value.enumeration=MP_E_NO_SUCH_ATTRIBUTE;
		ret = OM_SUCCESS ;
	}
	return ret ;
}


/*
** cfs_packages_set
*/

OM_return_code cfs_packages_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
)
{
	OM_return_code ret;
	cfs_t *instance;
	Attribute_ASN1Module_Packages decodedValue;

	instance = (cfs_t *) usrRef ;

	/*
	** ADD YOUR CODE HERE:
	** The attribute is conditional.
	** By default the setting of the attribute is allowed.
	** You can add your code below to disallow setting of
	** the attribute for the current instance.
	*/

	/*
	** Decode the new value
	*/
	ret = dec_Attribute_ASN1Module_Packages(valueForOperation, &decodedValue) ;
	if (ret == OM_SUCCESS) {
		ret = ca_copy_xom_value(resultingValue, valueForOperation) ;
		if (ret != OM_SUCCESS) {
			free_Attribute_ASN1Module_Packages(&decodedValue) ;
		}
	}
	else if (ret == OM_ENCODING_INVALID) {
		resultingValue->type = MP_ERROR_STATUS;
		resultingValue->syntax = OM_S_ENUMERATION;
		resultingValue->value.enumeration =
					MP_E_INVALID_ATTRIBUTE_VALUE;
	}
	/*
	** Copy the decoded value in the managed object
	*/
	if (ret == OM_SUCCESS) {
		if (instance->packagesPresent)
			free_Attribute_ASN1Module_Packages(&(instance->packages)) ;
		instance->packages = decodedValue ;
		instance->packagesPresent = OM_TRUE ;
	}

	/*
	** Include the attribute in the attribute value change notification
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_dmi_bldAttrChngLst(
			context, 
			&(instance->attrValChngLst),
			&packages__id, 
			NULL, 
			valueForOperation
		) ;
	}

	/*
	** ADD YOUR CODE HERE:
	** Managed object is now updated.
	** You can code here to update the managed resource.
	*/
	if (ret == OM_SUCCESS) {


	}

	/*
	** When a CMIS error is reported, return code must be cleared.
	*/
	if (resultingValue->type == MP_ERROR_STATUS) {
		ret = OM_SUCCESS ;
	}

	return ret ;
}


/*
** cfs_packages_eval
*/

OM_return_code cfs_packages_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t *instance;
	Attribute_ASN1Module_Packages decodedValue;

	instance = (cfs_t *) usrRef ;

	if (type == MP_PRESENT) {
		*result = instance->packagesPresent ;
		*error = OM_FALSE ;
	}
	else {
		/*
		** Decode value1
		*/
		ret = dec_Attribute_ASN1Module_Packages(value1, &decodedValue) ;

		/*
		** Compare the decoded value to the value stored
		** in the instance structure.
		*/
		if (ret == OM_SUCCESS) {
			switch(type) {

			case MP_EQUALITY :
			case MP_SUBSET_OF :
			case MP_SUPERSET_OF :
			case MP_NON_NULL_SET_INTERSECTION :
				*result = cmp_Attribute_ASN1Module_Packages(
					&decodedValue,
					&instance->packages,
					type
				) ;
				*error = OM_FALSE ;
				break ;

			default :
				*result = OM_FALSE ;
				*error = OM_TRUE ;
				break ;

			}
		}
		else if (ret == OM_ENCODING_INVALID) {
			*error = OM_TRUE ;
			ret = OM_SUCCESS ; 
		}

		/*
		** Cleaning
		*/
		if (ret == OM_SUCCESS)
			free_Attribute_ASN1Module_Packages(&decodedValue) ;
	}

	return ret ;
}


/*
** cfs_allomorphs_get
*/

OM_return_code cfs_allomorphs_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t	*instance ;

	instance = (cfs_t *) usrRef ;
	if (instance->allomorphsPresent == OM_TRUE) {
		ret = enc_Attribute_ASN1Module_Allomorphs(
			&(instance->allomorphs), 
			value
		) ;
	}
	else {
		value->type = MP_ERROR_STATUS;
		value->syntax = OM_S_ENUMERATION;
		value->value.enumeration=MP_E_NO_SUCH_ATTRIBUTE;
		ret = OM_SUCCESS ;
	}
	return ret ;
}


/*
** cfs_allomorphs_set
*/

OM_return_code cfs_allomorphs_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
)
{
	OM_return_code ret;
	cfs_t *instance;
	Attribute_ASN1Module_Allomorphs decodedValue;

	instance = (cfs_t *) usrRef ;

	/*
	** ADD YOUR CODE HERE:
	** The attribute is conditional.
	** By default the setting of the attribute is allowed.
	** You can add your code below to disallow setting of
	** the attribute for the current instance.
	*/

	/*
	** Decode the new value
	*/
	ret = dec_Attribute_ASN1Module_Allomorphs(valueForOperation, &decodedValue) ;
	if (ret == OM_SUCCESS) {
		ret = ca_copy_xom_value(resultingValue, valueForOperation) ;
		if (ret != OM_SUCCESS) {
			free_Attribute_ASN1Module_Allomorphs(&decodedValue) ;
		}
	}
	else if (ret == OM_ENCODING_INVALID) {
		resultingValue->type = MP_ERROR_STATUS;
		resultingValue->syntax = OM_S_ENUMERATION;
		resultingValue->value.enumeration =
					MP_E_INVALID_ATTRIBUTE_VALUE;
	}
	/*
	** Copy the decoded value in the managed object
	*/
	if (ret == OM_SUCCESS) {
		if (instance->allomorphsPresent)
			free_Attribute_ASN1Module_Allomorphs(&(instance->allomorphs)) ;
		instance->allomorphs = decodedValue ;
		instance->allomorphsPresent = OM_TRUE ;
	}

	/*
	** Include the attribute in the attribute value change notification
	*/
	if (ret == OM_SUCCESS) {
		ret = ca_dmi_bldAttrChngLst(
			context, 
			&(instance->attrValChngLst),
			&allomorphs__id, 
			NULL, 
			valueForOperation
		) ;
	}

	/*
	** ADD YOUR CODE HERE:
	** Managed object is now updated.
	** You can code here to update the managed resource.
	*/
	if (ret == OM_SUCCESS) {


	}

	/*
	** When a CMIS error is reported, return code must be cleared.
	*/
	if (resultingValue->type == MP_ERROR_STATUS) {
		ret = OM_SUCCESS ;
	}

	return ret ;
}


/*
** cfs_allomorphs_eval
*/

OM_return_code cfs_allomorphs_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
)
{
	OM_return_code ret = OM_SUCCESS;
	cfs_t *instance;
	Attribute_ASN1Module_Allomorphs decodedValue;

	instance = (cfs_t *) usrRef ;

	if (type == MP_PRESENT) {
		*result = instance->allomorphsPresent ;
		*error = OM_FALSE ;
	}
	else {
		/*
		** Decode value1
		*/
		ret = dec_Attribute_ASN1Module_Allomorphs(value1, &decodedValue) ;

		/*
		** Compare the decoded value to the value stored
		** in the instance structure.
		*/
		if (ret == OM_SUCCESS) {
			switch(type) {

			case MP_EQUALITY :
			case MP_SUBSET_OF :
			case MP_SUPERSET_OF :
			case MP_NON_NULL_SET_INTERSECTION :
				*result = cmp_Attribute_ASN1Module_Allomorphs(
					&decodedValue,
					&instance->allomorphs,
					type
				) ;
				*error = OM_FALSE ;
				break ;

			default :
				*result = OM_FALSE ;
				*error = OM_TRUE ;
				break ;

			}
		}
		else if (ret == OM_ENCODING_INVALID) {
			*error = OM_TRUE ;
			ret = OM_SUCCESS ; 
		}

		/*
		** Cleaning
		*/
		if (ret == OM_SUCCESS)
			free_Attribute_ASN1Module_Allomorphs(&decodedValue) ;
	}

	return ret ;
}


/*
** cfs_cfsUpdateSystem
*/

OM_return_code cfs_cfsUpdateSystem(
	ca_context_t	*context,
	OM_sint32	invokeId,
	OM_uint		mode,
	OM_descriptor	*infoArg,
	void		*usrRef,
	void		*accessRef
)
{
	OM_return_code ret = OM_SUCCESS;
	OM_descriptor replyArg = OM_NULL_DESCRIPTOR;
	cfs_t * instance ;

	instance = (cfs_t *) usrRef ;

	/*
	** ADD YOUR CODE HERE:
	** Implement the action here.
	*/

	/*
	** Update the CFS agent object tree (MIT)
	*/
	ret = cfs_updateBranchOfCfs(context, usrRef) ;

	/*
	** Set the CMIS error to be reported
	*/
	if (ret != OM_SUCCESS) {
		replyArg.type = MP_ERROR_STATUS ;
		replyArg.syntax = OM_S_ENUMERATION ;
		replyArg.value.enumeration = MP_E_PROCESSING_FAILURE ;
	}

	/*
	** Call ca_gom_action_rsp()
	*/
	if ((ret == OM_SUCCESS) || (replyArg.type == MP_ERROR_STATUS)){
		if (mode == MP_T_CONFIRMED) {
			ret = ca_gom_action_rsp(
				context, 
				invokeId, 
				usrRef, 
				accessRef, 
				&replyArg, 
				OM_FALSE
			);
		}
	}

	/*
	** When a CMIS error is reported, return code must be cleared.
	*/
	if (replyArg.type == MP_ERROR_STATUS) {
		ret = OM_SUCCESS ;
	}

	return ret;
}


/*
** Functions added for the example
*/

/*****************************************************************************
**  Function Name: cfs_getFileName
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Find the filename from the file path pointed to by filepath
**
*****************************************************************************/

OM_return_code cfs_getFileName(
	char 		*filepath, 
	char 		**filename
)
{
	OM_return_code ret = OM_SUCCESS ;
 	char 	*ptr = NULL ;
	char	*name = NULL ;

	/* 
	** Initialize 
	*/
	(*filename) = NULL ;

	/* 
	** set the string pointer to the filename 
	*/
	ptr = strrchr(filepath, '/') ;
	ptr++ ;

	/*
	** Allocate memory for the filename
	*/
	name = (char*) calloc(strlen(ptr) + 1,1) ;
	if (name == NULL) {
	 	printf("cfs_getFileName : calloc failed\n") ;
	 	return OM_MEMORY_INSUFFICIENT ;
	} 

	/* 
	** Set the filename
	*/
	strcpy(name,ptr) ;
	(*filename) = name ;

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_findFilesOfDirectory
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Find all the files of the directory pointed to by parentDir
**
*****************************************************************************/

OM_return_code cfs_findFilesOfDirectory(
	char 		*parentDir, 
	ca_list_t	 **list , 
	OM_uint 	*nbItem
)
{
	OM_return_code ret = OM_SUCCESS ;	
	char *fileName = NULL ;
	ca_list_t *node = NULL ;
	DIR *openret = NULL ;
	struct dirent *dirInfo = NULL ;
	int 	accessret = 0 ;

	/* 
	** Initialize 
	*/
	(*list) = NULL ;
	(*nbItem) = 0 ;

	/* 
	** Test if the directory is searchable
	*/
	accessret = access(parentDir, X_OK) ;
	if (accessret == -1) 
		return OM_FUNCTION_INTERRUPTED ;

	/* 
	** Open the directory
	*/
	openret = opendir(parentDir) ;
	if (openret == NULL)
		return OM_FUNCTION_DECLINED ;		

	/* 
	** Retrieve the filenames from the file stream
	*/
	while ( ((dirInfo = readdir(openret)) != NULL) && (ret == OM_SUCCESS) ) {
		/* 
		** Ignore the home . and parent .. directories
		*/
		if ( (strcmp(dirInfo->d_name,PARENT) == 0) || (strcmp(dirInfo->d_name,HOME) == 0) )
			continue ;
		
		/*
		** Allocate memory for the filename
		*/
		fileName = (char*) calloc(strlen(dirInfo->d_name) + strlen(parentDir) + 2,1)  ;
		if (fileName == NULL)
			ret = OM_MEMORY_INSUFFICIENT ;

		/*
		** Allocate memory for a list node
		*/
		node = (ca_list_t*) calloc(1,sizeof(ca_list_t)) ;
		if (node == NULL) {
		        free(fileName) ;
			ret = OM_MEMORY_INSUFFICIENT ;
		}

		
		/* 
		** Add the file path to the list 
		*/
		if (ret == OM_SUCCESS) {
		 	strcpy(fileName, parentDir) ;
		 	strcat(fileName, "/") ;
		 	strcat(fileName, dirInfo->d_name) ;

			node->item = (void*) fileName ;
		 	node->next = (*list) ;
		 	(*list) = node ;
		 	(*nbItem) = (*nbItem) + 1 ;
		}
	}
	
	/*
	** Free the file stream
	*/
	if (openret != NULL)
		closedir(openret) ;

	/*
	** Problem encountered cleaning
	*/	
	if ( (ret != OM_SUCCESS) && ((*list) != NULL) )	{
		 ret = cfs_freeListOfFiles((*list)) ;
		 (*nbItem) = 0 ;
	}

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_freeListOfFiles
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Free the memory used for the list of file paths
**
*****************************************************************************/

OM_return_code cfs_freeListOfFiles(
	ca_list_t 	*list
)
{
	OM_return_code ret = OM_SUCCESS ;
	ca_list_t *templist = NULL ;

	if (list != NULL) {
		templist = list ;
	
		/* 
		** Free the memory allocated for the file paths 
		*/
		while (templist != NULL) {
			if (templist->item != NULL)		
			 	free(templist->item) ;

			 templist = templist->next ;
		}
	
		/* 
		** Free memory allocated for list nodes 
		*/
		ret = ca_freeListOfInst(list) ;
		list = NULL ;
	}

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_removeNodeFromList
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Remove a node from a list of caRef's and free the memory
**
*****************************************************************************/

OM_return_code cfs_removeNodeFromList(
	ca_list_t 	**list, 
	OM_uint 	position
)
{
	ca_list_t *prev ;
	ca_list_t *current ;
	int counter ;

	/* 
	** Initialize 
	*/
	current = (*list) ;
	prev = NULL ;
	counter = 1 ;

	/*
	** Find the node in the list 
	*/
	while ( (current->next != NULL) && (position != counter) ) {
		prev = current ;
		current = current->next ;

		counter++ ;
	}

	/* 
	** Remove the node from the list
	*/
	if (prev == NULL) {		/* The first node of the list */
		(*list) = current->next ;
		free(current) ;
	}
	else {				
		prev->next = current->next ;
		free(current) ;
	}

	return OM_SUCCESS ;
}


/*****************************************************************************
**  Function Name: cfs_isDirectory
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Test if the file pointed to by filename is a directory
**
*****************************************************************************/

OM_return_code cfs_isDirectory(
	char 		*filename, 
	OM_boolean 	*isDir
)
{
	OM_return_code ret = OM_SUCCESS ;
	struct stat fileinfo ;

	/*
	** Initialize
	*/ 
	(*isDir) = OM_FALSE ;	

	/*
	** Get information on the file
	*/
	if (stat(filename, &fileinfo) < 0) {
		printf("cfs_isDirectory : stat error\n") ;
		ret = OM_FUNCTION_DECLINED ;
	}
	else {
		/* 
		** Test if the file is a directory
		*/
	  	if (S_ISDIR(fileinfo.st_mode))
		 	(*isDir) = OM_TRUE ;
	}

	return 	ret ;
}


/*****************************************************************************
**  Function Name: cfs_isClasscfs
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Test if the class definition of the object pointed to by caRef
** 	is of the cfs class definition
**
*****************************************************************************/

OM_return_code cfs_isClasscfs(
	ca_context_t 	*context, 
	void 		*caRef, 
	OM_boolean 	*isCfs
)
{
	OM_return_code ret = OM_SUCCESS ;
	ca_class_t *class = NULL ;

	/*
	** Initialize
	*/ 
	(*isCfs) = OM_FALSE ;

	/*
	** Find the class definition of the object
	*/
	ret = ca_findClassDefForInst(
		context,
		caRef,
		&class
	) ;

	if (ret == OM_SUCCESS)	{
		/*
		** Compare the class definition of the object 
		** to the cfs definition 
		*/
		if (strcmp(class->name,cfs__def.name) == 0) 
		 		(*isCfs) = OM_TRUE ;
	}
	else {
		 printf("cfs_isClasscfs : ca_findClassDefForInst error\n") ;
	}

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_isClasscfsDir
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Test if the class definition of the object pointed to by caRef
** 	is of the cfsDir class definition
**
*****************************************************************************/

OM_return_code cfs_isClasscfsDir(
	ca_context_t 	*context, 
	void 		*caRef, 
	OM_boolean 	*iscfsDir
)
{
	OM_return_code ret = OM_SUCCESS ;
	ca_class_t *class = NULL;

	/*
	** Initialize
	*/ 
	(*iscfsDir) = OM_FALSE ;

	/*
	** Find the class definition of the object
	*/
	ret = ca_findClassDefForInst(
		context,
		caRef,
		&class
	) ;
 
	if (ret == OM_SUCCESS) {
		/*
		** Test if the class definition of the object 
		** is the same as the cfsDir definition
		*/
		if (strcmp(class->name,cfsDir__def.name) == 0) 
		 	(*iscfsDir) = OM_TRUE ;
	}
	else {
	 	printf("cfs_isClasscfsDir : ca_findClassDefForInst error\n") ;
	}

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_findcaRef
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Find the caRef of the object pointed to by the usrRef
**
*****************************************************************************/

OM_return_code cfs_findcaRef(
	ca_context_t 	*context, 
	void 		*usrRef, 
	void 		**caRef
)
{
	OM_return_code ret = OM_SUCCESS;	
	OM_private_object class = NULL ; 
	OM_private_object instance = NULL ;

	/* 
	** Initialize 
	*/
	(*caRef) = NULL ;

	/* 
	** Find the caRef 
	*/
	ret = ca_findRefOfInst(
		context,
		&class, 
		&instance, 
		&usrRef, 
		caRef
	) ;

	/*
	** Cleaning
	*/
	if (class != NULL) {
		ret = om_delete(class) ;
		if (ret != OM_SUCCESS)
			printf("cfs_findcaRef: OM_delete failed for class object\n") ;
	}
	if (instance != NULL) {
		ret = om_delete(instance) ;
		if (ret != OM_SUCCESS)
			printf("cfs_findcaRef: OM_delete failed for instance object\n") ;
	}

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_findusrRef
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Find the usrRef of the object pointed to by the caRef
**
*****************************************************************************/

OM_return_code cfs_findusrRef(
	ca_context_t 	*context, 
	void 		*caRef, 
	void 		**usrRef
)
{
	OM_return_code ret = OM_SUCCESS ;	
	OM_private_object class = NULL ; 
	OM_private_object instance = NULL ;

	/* 
	** Initialize 
	*/
	(*usrRef) = NULL ;

	/* 
	** Find the usrRef 
	*/
	ret = ca_findRefOfInst(
		context,
		&class, 
		&instance, 
		usrRef, 
		&caRef
	) ;

	/*
	** Cleaning
	*/
	if (class != NULL) {
		ret = om_delete(class) ;
		if (ret != OM_SUCCESS)
			printf("cfs_findusrRef: OM_delete failed for class object\n") ;
	}
	if (instance != NULL) {
		ret = om_delete(instance) ;
		if (ret != OM_SUCCESS)
			printf("cfs_findusrRef: OM_delete failed for instance object\n") ;
	}

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_createCfs
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Create the static cfs object and set cfsName attribute
**
*****************************************************************************/

OM_return_code cfs_createCfs(
	ca_context_t 	*context,
	void		*caSupRef,
	void		**current
) 
{
	OM_return_code	ret = OM_SUCCESS ;
	OM_descriptor localName  ; 
	OM_descriptor	resultingValue  ;	 /* Value passed back by set callback */ 
	void *usrRef = NULL ;	
	void *caRef = NULL ;
	void *accessRef = NULL ;
	OM_descriptor error  ;
	cfs_t *cfsInstance = NULL ;
	
	/*
	** Initialize
	*/
	(*current) = NULL;
	
	/*
	** Test if the starup directory is searchable
	*/
	if (access(cfsRootDirectory,X_OK) != 0) {
		printf("cfs_createCfs : The startup directory %s is not searchable\n", cfsRootDirectory) ;
		return OM_FUNCTION_DECLINED ;
	} ;

	/*
	** Set the reletive distinguished name (RDN) of the object
	*/
	localName.syntax = OM_S_PRINTABLE_STRING ;
	localName.value.string.elements =  CFSID ;
	localName.value.string.length = strlen(CFSID) ;

	/* 
	** Create the Cfs object 
	*/
	ret = cfs_create(
		context, 
		caSupRef, 
		&cfs__binding[0], 
		&localName, 
		&caRef, 
		&accessRef, 
		&error
	) ;

	if (ret == OM_SUCCESS) 	{
		/*
		** Set the caRef send back to the core 
		*/
		 (*current) = caRef ;

		/*
		** Get the usrRef for the Cfs object
		*/
		ret = cfs_findusrRef(
			context, 
			caRef, 
			&usrRef
		) ;

		/*
		** Set the attribute values of the Cfs object 
		*/
		if (ret == OM_SUCCESS) {
			/*
			** Set the descriptor with the value of attribute
			*/
			localName.syntax = OM_S_PRINTABLE_STRING ;
			localName.value.string.elements = cfsRootDirectory ;
			localName.value.string.length = strlen(cfsRootDirectory) ;
			
			/*
			** Set the cfsName value of the Cfs object to the starting directory
			*/
			ret = cfs_cfsName_set(
				context, 
				MP_T_CONFIRMED, 
				&localName, 
				usrRef, 
				&accessRef, 
				&resultingValue
			) ;

			/*
			** Check if the set operation was succesfull
			*/
			if (ret == OM_SUCCESS)
	 			ret = cfs_check(
					context,
					MP_SET_IND,
					caRef,
					usrRef
				) ;

			/*
			** Free memory used for set result
			*/
			ret = ca_free_copied_xom_value(&resultingValue) ;
		}

		/* 
		** Create the rest of the objects of the CMIP file system
		*/
		if (ret == OM_SUCCESS)
			ret = cfs_createBranchOfCfs(
				context, 
				cfsRootDirectory, 
				caRef, 
				accessRef
			) ;		
	}
	else {
		printf("cfs_createCfs: cfs_create failed \n") ;	
	}

	return ret  ;
}


/*****************************************************************************
**  Function Name: cfs_createBranchOfCfs
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Create a branch of objects in the MIT for the current directory 
**	pointed to by directory
**
*****************************************************************************/

OM_return_code cfs_createBranchOfCfs(
	ca_context_t	*context,
	char		*directory,
	void		*caSupRef,
	void		*accessRef
)
{
	OM_return_code ret = OM_SUCCESS ;
	ca_list_t *list = NULL ;
	ca_list_t *tempnode = NULL ;
	char *instanceName = NULL ;
	OM_uint nbItem = 0;

	/* 
	** Find the files of the current directory 
	*/
	ret =  cfs_findFilesOfDirectory(
		directory, 
		&list,
		&nbItem
	) ;

	if (ret == OM_FUNCTION_INTERRUPTED) 		 
		 return OM_SUCCESS ;	/* The directory was not searchable */

	/* 
	** Create an instance of each files in the current directory 
	*/
	tempnode = list ;
	while ( (nbItem > 0) && (ret == OM_SUCCESS) ) {
		/* 
		** Get the filename from the file path 
		*/
		ret = cfs_getFileName((char*) tempnode->item, &instanceName) ;

		/*
		** Create a new object and set the attributes of the object
		*/
		if (ret == OM_SUCCESS)
			ret = cfs_createInstance(
				context,
				caSupRef, 
				(char*) tempnode->item,
				instanceName
			) ;

		/*
		** Cleaning
		*/		
		if (instanceName != NULL)
			free(instanceName) ;

	    	nbItem-- ;
		tempnode = tempnode->next ;	
	} 

	/*
	** Cleaning
	*/
	if (list != NULL)
		ret = cfs_freeListOfFiles(list) ;

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_createInstance
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Create an object and set the object attributes
**
*****************************************************************************/

OM_return_code cfs_createInstance(
	ca_context_t 	*context,
	void 		*caSupRef, 
	char 		*filePath,
	char 		*instanceName
)
{
	OM_return_code ret = OM_SUCCESS ;
	void *caRef = NULL ;
	OM_boolean isDir ;

	/* 
	** Test if the file is a directory 
	*/
	ret = cfs_isDirectory(filePath, &isDir) ;

	/*
	** Create the object and set attribute values
	*/
	if (ret == OM_SUCCESS) {
	 	if (isDir == OM_TRUE) {
			/*
			** Create a cfsDir object
			*/
			ret = cfs_createcfsDir(
					context,
					caSupRef, 
					filePath,
					instanceName,
					&caRef
			) ;

			/*
			** Set the cfsFilePermissions attribute for cfsDir object 
			*/
			if (ret == OM_SUCCESS)
				ret = cfs_setcfsDir(
						context,
						caRef
				) ;
		}
		else {
			/*
			** Create a cfsFile object
			*/
			ret = cfs_createcfsFile(
					context,
					caSupRef, 
					filePath,
					instanceName,
					&caRef
			) ;

			/*
			** Set the cfsFilePermissions attribute for cfsFile object 
			*/
			if (ret == OM_SUCCESS)
				ret = cfs_setcfsFile(
						context,
						caRef
				) ;
		} 
	}

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_createcfsFile
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Create a cfsFile object
**
*****************************************************************************/

OM_return_code cfs_createcfsFile(
	ca_context_t 	*context,
	void 		*caSupRef, 
	char 		*filePath,
	char 		*instanceName,
	void		**caRef
)
{
	OM_return_code ret = OM_SUCCESS ;
	OM_descriptor localName ;
	OM_descriptor error ;
	OM_boolean isCfs ;
	void *accessRef = NULL ;
	void *usrRef = NULL ;
	ca_binding_t *binding = NULL ;

	/*
	** Initialize
	*/
	(*caRef) = NULL ;

	/*
	** Set the reletive distinguished name (RDN) of the object
	*/
	localName.syntax = OM_S_PRINTABLE_STRING ;
	localName.value.string.length = strlen(instanceName)  ;
	localName.value.string.elements = instanceName ;

	/*
	** Find if the parent object is the Cfs object
	*/
	ret = cfs_isClasscfs(
			context, 
			caSupRef, 
			&isCfs
	) ;

	/*
	** Set the binding 
	*/
	if (ret == OM_SUCCESS) {
		if (isCfs == OM_TRUE) {
			binding = &cfsFile__binding[1] ;	/* Bind to the cfs instance */
		}
		else {
			binding = &cfsFile__binding[0] ;	/* Bind to the cfsDir instance */
		}

		/*
		** Create a new cfsFile object
		*/
		ret = cfsFile_create(
				context,
				caSupRef,
				binding, 
				&localName, 
				caRef, 
				&accessRef, 
				&error
		)  ;

		if (ret == OM_SUCCESS) {
			/* 	
			** Find the usrRef of the object
			*/ 
			ret = cfs_findusrRef(
					context,
					(*caRef),
					&usrRef
			) ;

			/*
			** Check if the create operation was succesfully completed
			*/
			if (ret == OM_SUCCESS)
				ret = cfsFile_check(
						context,
						MP_CREATE_IND,
						(*caRef),
						usrRef
				) ;
		}
	}

	return ret ;
}

/*****************************************************************************
**  Function Name: cfs_createcfsDir
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Create a cfsDir object
**
*****************************************************************************/

OM_return_code cfs_createcfsDir(
	ca_context_t 	*context,
	void 		*caSupRef, 
	char 		*filePath,
	char 		*instanceName,
	void		**caRef
)
{
	OM_return_code ret = OM_SUCCESS ;
	OM_descriptor localName ;
	OM_descriptor error ;
	OM_boolean isCfs ;
	void *accessRef = NULL ;
	void *usrRef = NULL ;
	ca_binding_t *binding = NULL ;

	/*
	** Initialize
	*/
	(*caRef) = NULL ;

	/*
	** Set the reletive distinguished name (RDN) of the object
	*/
	localName.syntax = OM_S_PRINTABLE_STRING ;
	localName.value.string.length = strlen(instanceName)  ;
	localName.value.string.elements = instanceName ;

	/*
	** Find if the parent is the Cfs object
	*/
	ret = cfs_isClasscfs(
			context, 
			caSupRef, 
			&isCfs
	) ;

	/*
	** Set the binding 
	*/			
	if (ret == OM_SUCCESS) {
		if ( isCfs == OM_TRUE) {
			binding = &cfsDir__binding[1] ;	/* Bind to the cfs instance */
		}
		else {
			binding = &cfsDir__binding[0] ;	/* Bind to the cfsDir instance */
		}

		/*
		** Create a new cfsDir object
		*/
		ret = cfsDir_create(
				context,
				caSupRef,
				binding, 
				&localName, 
				caRef, 
				&accessRef, 
				&error
		) ;

		if (ret == OM_SUCCESS) {
			/* 	
			** Find the usrRef of the object
			*/
			ret = cfs_findusrRef(
					context,
					(*caRef),
					&usrRef
			) ;

			/*
			** Check if the create operation was succesfully completed
			*/
			if (ret == OM_SUCCESS)
				ret = cfsDir_check(
					context,
					MP_CREATE_IND,
					(*caRef),
					usrRef
				) ;
		}
	}

	return ret ;
}



/*****************************************************************************
**  Function Name: cfs_setcfsFile
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Set the cfsFilePermissions attribute for cfsFile object
**
*****************************************************************************/

OM_return_code cfs_setcfsFile(
	ca_context_t 	*context,
	void		*caRef
)
{
	OM_return_code ret = OM_SUCCESS ;
	OM_descriptor filePermissions ;
	OM_descriptor resultingValue ; 		/* Value passed back by set callback */
	void *accessRef = NULL ;
	void *usrRef = NULL ;

	/* 	
	** Find the usrRef of the object
	*/
	ret = cfs_findusrRef(
			context,
			caRef,
			&usrRef
	) ;

	/*
	** Get the access permissions from Unix
	*/
	if (ret == OM_SUCCESS)
		ret = cfs_getFilePermissions(
					context, 
					usrRef,
					&filePermissions
		)  ;

	/*
	** Set the cfsFilePermissions attribute
	*/
	if (ret == OM_SUCCESS) {
		ret = cfsFile_cfsFilePermissions_set(
					context,
					MP_T_ADD_VALUES,
					&filePermissions,
					usrRef,
					&accessRef,
					&resultingValue
		) ;

		/*
		** Check if the set operation was succesfull
		*/
		if (ret == OM_SUCCESS)
			ret = cfsFile_check(
					context,
					MP_SET_IND,
					caRef,
					usrRef
			) ;

		/*
		** Free memory used for set result
		*/
		ret = ca_free_copied_xom_value(&resultingValue) ;

		/*
		** Free memory used for encoding
		*/
		ret = ca_free_copied_xom_value(&filePermissions) ;
	}

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_setcfsDir
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Set the cfsFilePermissions attribute for cfsDir object
**
*****************************************************************************/

OM_return_code cfs_setcfsDir(
	ca_context_t 	*context,
	void		*caRef
)
{
	OM_return_code ret = OM_SUCCESS ;
	OM_descriptor filePermissions ;
	OM_descriptor resultingValue ; 		/* Value passed back by set callback */
	void *accessRef = NULL ;
	void *usrRef = NULL ;

	/* 	
	** Find the usrRef of the object
	*/
	ret = cfs_findusrRef(
			context,
			caRef,
			&usrRef
	) ;

	/*
	** Get the access permissions from Unix
	*/
	ret = cfs_getFilePermissions(
				context, 
				usrRef,
				&filePermissions
	)  ;

	/*
	** Set the cfsFilePermissions attribute
	*/
	if (ret == OM_SUCCESS) {
		ret = cfsDir_cfsFilePermissions_set(
						context,
						MP_T_ADD_VALUES,
						&filePermissions,
						usrRef,
						&accessRef,
						&resultingValue
			) ;

		/*
		** Check if the set operation was succesful
		*/
		if (ret == OM_SUCCESS)
			ret = cfsDir_check(
					context,
					MP_SET_IND,
					caRef,
					usrRef
			) ;

		/*
		** Free memory used for set result
		*/
		ret = ca_free_copied_xom_value(&resultingValue) ;

		/*
		** Free memory used for encoding
		*/
		ret = ca_free_copied_xom_value(&filePermissions) ;
	}

	return ret ;

}


/*****************************************************************************
**  Function Name: cfs_getFullPath
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Concatenate the full UNIX path of the calling object from the MIT tree
**
*****************************************************************************/

OM_return_code cfs_getFullPath(
	ca_context_t 	*context, 
	void 		*usrRef, 
	char 		*partialPath, 
	char 		**fullPath
)
{	

	OM_return_code ret = OM_SUCCESS ;
	OM_private_object class = NULL ;
	OM_private_object instance = NULL ;
	OM_boolean isCfs ;
	cfs_t	*tempInstance = NULL;	
	void	*usrSupRef = NULL ; 
	void 	*caSupRef = NULL  ;
	void 	*caRef	= NULL ;
	char 	*fileName = NULL ;

	/*
	** Initialize
	*/
	(*fullPath) = NULL ;

	/* 
	** Find the caRef of the object
	*/ 
	ret = cfs_findcaRef(
		context, 
		usrRef, 
		&caRef
	) ; 

	/* 
	** Test if the current object is the cfs object
	*/
	if (ret == OM_SUCCESS)
		ret = cfs_isClasscfs(
			context, 
			caRef, 
			&isCfs
		) ;

	/*
	** Concatenate the rest of the path
	*/
	if (ret == OM_SUCCESS) {
		tempInstance = (cfs_t*) usrRef ;

		/*
		** Concatenate the full path
		*/			
		if (isCfs == OM_TRUE) {
			if (partialPath == NULL) { 	/* First time the function is called */ 
				fileName = (char*) calloc((tempInstance->cfsName).length + 1, 1) ;

				/* 
				** Concatenate the cfs root directory to the path 
				*/
				strcpy(fileName,(tempInstance->cfsName).elements) ;

				(*fullPath) = fileName ;
			}
			else {
			 	fileName = (char*) calloc((tempInstance->cfsName).length + strlen(partialPath) + 1,1) ;
	
			 	if (fileName == NULL) {
				 	printf("cfs_getFullPath: calloc failed\n")  ;
				 	ret = OM_MEMORY_INSUFFICIENT  ;
			 	} 

				if (ret == OM_SUCCESS) {
					/* 
					** Concatenate the cfs directory to the path 
					*/
					strcat(fileName,(tempInstance->cfsName).elements) ;

					/* 
					** Concatenate the rest of the path to the full path 
					*/
					strcat(fileName,partialPath) ;
					free(partialPath) ;
				} 
				(*fullPath) = fileName ;
			} 
		}
		else { 					/* Not at the cfs object */
		/*
		** Concatenate the partial path
		*/
		 	if (partialPath == NULL) {	/* First time the function is called */
			 	fileName = (char*) calloc((tempInstance->cfsId).length + 3,1) ;
			}
			else {
			 	fileName = (char*) calloc((tempInstance->cfsId).length + strlen(partialPath)+ 2,1) ;
			} 

			if (fileName == NULL) {
				printf("cfs_getFullPath: calloc failed\n")  ;
				ret = OM_MEMORY_INSUFFICIENT  ;
			} 

			if (ret == OM_SUCCESS) {
				/* 
				** Concatenate the current directory name to the path
				*/
				strcat(fileName,"/") ;
				strcat(fileName,(tempInstance->cfsId).elements) ;

				/* 
				** Concatenate the rest of the path to give the full path 
				*/
				if (partialPath != NULL) {	/* Not the first time the function is called */
					strcat(fileName,partialPath) ;
					free(partialPath) ;
				} 
			
				/*
				** Find the parent object that contains the specified object
				*/
				ret = ca_findParentOfInst(
					context, 
					NULL, 
					NULL, 
					usrRef, 
					NULL, 
					&caSupRef
				) ;
				
				/* 
				** Find the usrRef of the parent object
				*/
				if (ret == OM_SUCCESS) 
					ret = cfs_findusrRef(
						context, 
						caSupRef, 
						&usrSupRef) ;

				/* 
				** Add the next directory to the full path 
				*/
				if (ret == OM_SUCCESS)
					ret = cfs_getFullPath(
						context,
						usrSupRef,
						fileName,
						fullPath
					) ;
			} 
		} 
	} 

	/*
	** Cleaning
	*/
	if (ret != OM_SUCCESS) {
		if (partialPath != NULL)
			free(partialPath) ;
		
		if (fileName != NULL)
			free(fileName) ;
	} 

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_updateBranchOfCfs
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Updates a branch of objects (MIT) from(not including) the object pointed 
**	to by the usrRef down
**
*****************************************************************************/

OM_return_code cfs_updateBranchOfCfs(
	ca_context_t	*context,
	void		*usrRef
)
{
	OM_return_code ret = OM_SUCCESS ;
	OM_boolean found = OM_FALSE ;
	ca_list_t *calist = NULL ;
	ca_list_t *captr = NULL ;
	ca_list_t *filelist = NULL ;
	ca_list_t *fileptr = NULL ;
	OM_uint nbca = 0 ;
	OM_uint nbfile = 0 ;
	char *filename = NULL ;
	char *dirname = NULL ;
	void *tempusrRef = NULL ;
	void *caRef = NULL ;
	OM_uint counter = 0 ;
	cfs_t *instance = NULL ;

	/* 
	** Find the child objects contained by the spesified object 
	*/
	ret = ca_findChildOfInst(
		context, 
		NULL, 
		NULL, 
		usrRef, 
		NULL, 
		&calist,
		&nbca
	) ;

	/* 
	** Get the caRef of the current object 
	*/
	if (ret == OM_SUCCESS) 
		ret = cfs_findcaRef(
			context,
			usrRef,
			&caRef
		) ;

	/* 
	** Find the files contained in the directory represented by the object
	*/
	if (ret == OM_SUCCESS) 
		ret = cfs_getFullPath(
			context, 
			usrRef, 
			NULL, 
			&dirname
		) ;

	if (ret == OM_SUCCESS) 
		ret = cfs_findFilesOfDirectory(
			dirname, 
			&filelist, 
			&nbfile
		) ;

	/*
	** Cleaning
	*/
	if (dirname != NULL) 
		free(dirname) ;

	/*
	** Compare the contained objects and the contained files 
	*/
	fileptr = filelist ;
	while ( (nbfile > 0) && (ret == OM_SUCCESS) ) {
		ret = cfs_getFileName((char*) fileptr->item,&filename) ;
		captr = calist ;
		found = OM_FALSE ;
		counter = 1 ;
		while ((counter <= nbca) && (ret == OM_SUCCESS) && (found == OM_FALSE)) {
			ret = cfs_findusrRef(
				context, 
				captr->item, 
				&tempusrRef
			) ;
			instance = (cfs_t*) tempusrRef ;

			if (ret == OM_SUCCESS) {
				if (strcmp(instance->cfsId.elements,filename) == 0) {
					found = OM_TRUE ;

					/*
					** Updates the object attributes
					*/
					ret = cfs_updateInstance(
						context, 
						tempusrRef, 
						captr->item
					) ;

					/*
					** Remove the object from list
					*/
					ret = cfs_removeNodeFromList(&calist, counter) ;
					nbca-- ;
				}
			}
			captr = captr->next ;
			counter++ ;
		}

		/*
		** Create an instance for the file not found
		*/
		if ( (found == OM_FALSE) && (ret == OM_SUCCESS) ) 
			ret = cfs_createInstance(
				context, 
				caRef, 
				(char*) fileptr->item,
				filename
			) ;

		/*
		** Cleaning
		*/
		if (filename != NULL)
			free(filename) ;

		fileptr = fileptr->next ;
		nbfile-- ;
	}

	/*
	** Remove the objects for which no corresponding files were found
	*/
	if (ret == OM_SUCCESS)
		ret = cfs_removeInstance(
				context, 
				calist 
		) ;

	/*
	** Cleaning
	*/
	if (filelist != NULL) 
		ret = cfs_freeListOfFiles(filelist) ;

	if (calist != NULL)
		ret = ca_freeListOfInst(calist) ;
	
	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_updateInstance
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Updates the attributes of the object pointed to by the usrRef
**
*****************************************************************************/

OM_return_code cfs_updateInstance(
	ca_context_t 	*context, 
	void 		*usrRef, 
	void 		*caRef
)
{
	OM_return_code ret = OM_SUCCESS ;
	OM_boolean iscfsDir ;

	/* 
	** Test if the object is of the cfsDir class
	*/
	if (ret == OM_SUCCESS)
		ret = cfs_isClasscfsDir(context, caRef, &iscfsDir) ;

	if (ret == OM_SUCCESS) {
		if (iscfsDir == OM_TRUE) {
			/*
			** Update a cfsDir object
			*/
			ret = cfs_updatecfsDir(
					context,
					usrRef,
					caRef
			) ;
			
			/*
			** Update the objects contained
			*/ 
			if (ret == OM_SUCCESS)
				ret = cfs_updateBranchOfCfs(
							context, 
							usrRef
				) ;
		}
		else {
			/*
			** Update a cfsFile object
			*/
			ret = cfs_updatecfsFile(
					context,
					usrRef,
					caRef
			) ;
		}
	}

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_updatecfsFile
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Update the attributes of the cfsFile object pointed to by the usrRef
**
*****************************************************************************/

OM_return_code cfs_updatecfsFile(
	ca_context_t 	*context, 
	void 		*usrRef,
	void		*caRef 
)
{
	OM_return_code ret = OM_SUCCESS ;
	OM_boolean result ;
	OM_boolean error ;
	OM_descriptor resultingValue ;
	OM_descriptor filePermissions ;
	cfsFile_t *cfsFileInstance = NULL ;

	/*
	** Get the file permissions
	*/	
	ret = cfs_getFilePermissions(
			context, 
			usrRef,
			&filePermissions
	)  ;

	/*
	** Evalute the file and object permissions for equality 
	*/
	if (ret == OM_SUCCESS) {
		ret = cfsFile_cfsFilePermissions_eval(
						context,
						usrRef,	
						NULL,
						&filePermissions,
						NULL,
						MP_EQUALITY,
						&result,
						&error
		) ;

		/*
		** Set the object cfsFilePermissions 
		*/	
		if ((ret == OM_SUCCESS) && (error != OM_ENCODING_INVALID)) {
			if (result == OM_FALSE) {
 				ret = cfsFile_cfsFilePermissions_set(
							context,
							MP_T_ADD_VALUES,
							&filePermissions,
							usrRef,
							NULL,
							&resultingValue
				) ;


				/*
				** Check if the set operation was succesfull
				*/
				if (ret == OM_SUCCESS)
					ret = cfsFile_check(
						context,
						MP_SET_IND,
						caRef,
						usrRef
					) ;

				/*
				** Free memory used for the set result
				*/
				ret = ca_free_copied_xom_value(&resultingValue) ;
			}
		}

		/*
		** Free memory used for encoding
		*/
		ret = ca_free_copied_xom_value(&filePermissions) ;	
	}

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_updatecfsDir
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Update the attributes of the cfsDir object pointed to by the usrRef
**
*****************************************************************************/

OM_return_code cfs_updatecfsDir(
	ca_context_t 	*context, 
	void 		*usrRef,
	void		*caRef 
)
{
	OM_return_code ret = OM_SUCCESS ;
	OM_boolean result ;
	OM_boolean error ;
	OM_descriptor resultingValue ;
	OM_descriptor filePermissions ;
	cfsDir_t *cfsDirInstance = NULL ;

	/*
	** Get the file permissions
	*/	
	ret = cfs_getFilePermissions(
			context, 
			usrRef,
			&filePermissions
	)  ;

	/*
	** Evalute the file and object permissions for equality 
	*/
	if (ret == OM_SUCCESS) {
		ret = cfsDir_cfsFilePermissions_eval(
					context,
					usrRef,	
					NULL,
					&filePermissions,
					NULL,
					MP_EQUALITY,
					&result,
					&error
		) ;

		/*
		** Set the object attribute cfsFilePermissions 
		*/
		if ((ret == OM_SUCCESS) && (error != OM_ENCODING_INVALID)) {
			if (result == OM_FALSE) {
				ret = cfsDir_cfsFilePermissions_set(
					context,
					MP_T_ADD_VALUES,
					&filePermissions,
					usrRef,
					NULL,
					&resultingValue
				) ;

				/*
				** Check if the set operation was succesfull
				*/
				if (ret == OM_SUCCESS)
					ret = cfsDir_check(
						context,
						MP_SET_IND,
						caRef,
						usrRef
					) ;

				/*
				** Free memory used for set result
				*/
				ret = ca_free_copied_xom_value(&resultingValue) ;
			}
		}
		/*
		** Free memory used for encoding
		*/
		ret = ca_free_copied_xom_value(&filePermissions) ;
	}

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_removeInstance
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Remove an object and contained objects
**
*****************************************************************************/

OM_return_code cfs_removeInstance(
	ca_context_t 	*context, 
	ca_list_t 	*list
)
{
	OM_return_code ret = OM_SUCCESS ;
	OM_boolean iscfsDir ;
	ca_list_t *calist = NULL ;
	void *usrRef = NULL ;

	calist = list ;
	while ((calist != NULL) && (ret == OM_SUCCESS)) {

		/*
		** Find the usrRef of the object
		*/
		ret = cfs_findusrRef(
				context, 
				calist->item, 
				&usrRef
				) ;

		/* 
		** Test if the object is of the cfsDir class
		*/
		if (ret == OM_SUCCESS)
			ret = cfs_isClasscfsDir(
				context, 
				calist->item, 
				&iscfsDir
			) ;

		if (ret == OM_SUCCESS) {
			if(iscfsDir == OM_TRUE)	{
				/*
				** Removes a cfsDir object
				*/			
				ret = cfs_removecfsDir(context, usrRef) ;
			}
			else {
				/*
				** Removes a cfsFile object
				*/
				ret = cfs_removecfsFile(context, usrRef) ;
			} 
		}
		calist = calist->next ;
	} 

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_removecfsFile
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Removes a cfsFile object
**
*****************************************************************************/

OM_return_code cfs_removecfsFile(
	ca_context_t 	*context, 
	void		*usrRef
) 
{
	OM_return_code ret = OM_SUCCESS ;

	/*
	** Emit an object deletion notification
	*/
	ret = ca_dmi_objDelNotif(
			context,
			usrRef,
			CA_DMI_T_SOURCEINDICATOR_management,
			OM_FALSE,
			-1,
			"delete cfsFile"
	);

	/*
	** Delete a terminal node in the MIT
	*/
	if (ret == OM_SUCCESS)
		ret = ca_delFinalNodeInTree(
				context, 
				NULL, 
				NULL, 
				usrRef,
				NULL
		) ;
	
	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_removecfsDir
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Removes a cfsDir object
**
*****************************************************************************/

OM_return_code cfs_removecfsDir(
	ca_context_t 	*context, 
	void		*usrRef
)
{
	OM_return_code ret = OM_SUCCESS ;

	/*
	** Emit an object deletion notification
	*/
	ret = ca_dmi_objDelNotif(
			context,
			usrRef,
			CA_DMI_T_SOURCEINDICATOR_management,
			OM_FALSE,
			-1,
			"delete cfsDir"
	);

	/*
	** Delete all subnodes in the MIT
	*/
	if (ret == OM_SUCCESS)
		ret = ca_delAllSubNodesInTree(
					context,
					NULL,
					NULL,
					usrRef,
					NULL
		) ;

	/*
	** Delete a terminal node in the MIT
	*/
	if (ret == OM_SUCCESS) 
		ret = ca_delFinalNodeInTree(
					context,
					NULL, 
					NULL,
					usrRef, 
					NULL
		) ;

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_getFilePermissions
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Get the permissions of the file pointed to by the usrRef from UNIX
**
*****************************************************************************/

OM_return_code cfs_getFilePermissions(
	ca_context_t 	*context, 
	void 		*usrRef,
	OM_descriptor  *FilePermissions
)
{
	OM_return_code 	ret = OM_SUCCESS ;
	CFSObject_ASN1module_FilePermissions PermissionBits ;
	char 		*path = NULL ;
	void 		*caRef = NULL ;
	struct 		stat fileInfo ;
	int		statret = 0 ;

	/* 
	** Get the full UNIX path of the file 
	*/
	ret = cfs_getFullPath(
		context,
		usrRef,
		NULL,
		&path
	) ;

	if (ret == OM_SUCCESS) {
		/*
		** Get information of the file from Unix
		*/
		statret = stat(path,&fileInfo) ;
		if (statret == 0) {
			/*
			** Set the permission bits
			*/
			PermissionBits.user.read = OM_FALSE ;
			if (fileInfo.st_mode & S_IRUSR) 
				PermissionBits.user.read = OM_TRUE ;
 
			PermissionBits.user.write = OM_FALSE ;
			if (fileInfo.st_mode & S_IWUSR) 
				PermissionBits.user.write = OM_TRUE ;

			PermissionBits.user.execute = OM_FALSE ;
			if (fileInfo.st_mode & S_IXUSR) 
				PermissionBits.user.execute = OM_TRUE ;
		
			PermissionBits.group.read = OM_FALSE ;
			if (fileInfo.st_mode & S_IRGRP) 
				PermissionBits.group.read = OM_TRUE ;
 
			PermissionBits.group.write = OM_FALSE ;
			if (fileInfo.st_mode & S_IWGRP) 
				PermissionBits.group.write = OM_TRUE ;	

			PermissionBits.group.execute = OM_FALSE ;
			if (fileInfo.st_mode & S_IXGRP) 
				PermissionBits.group.execute = OM_TRUE ;

			PermissionBits.other.read = OM_FALSE ;
			if (fileInfo.st_mode & S_IROTH) 
				PermissionBits.other.read = OM_TRUE ;
 
			PermissionBits.other.write = OM_FALSE ;
			if (fileInfo.st_mode & S_IWOTH) 
				PermissionBits.other.write = OM_TRUE ;

			PermissionBits.other.execute = OM_FALSE ;
			if (fileInfo.st_mode & S_IXOTH) 
				PermissionBits.other.execute = OM_TRUE ;
		}
		else {
			ret = OM_FUNCTION_DECLINED ;
		}
		free(path) ;
	}

	if (ret == OM_SUCCESS) 
		/*
		** Encode the attribute value
		*/
		ret = enc_CFSObject_ASN1module_FilePermissions(
				&PermissionBits,
				FilePermissions
			) ;

	return ret ;
}


/*****************************************************************************
**  Function Name: cfs_timerproc
**
**----------------------------------------------------------------------------
**
**  Description:
**  ------------
** 
**	Define a function to catch the alarm signal
**
*****************************************************************************/

static void cfs_timerproc(
	int 		sig
) 
{
	OM_return_code 	ret ;
	cfs_t *cfsInstance = NULL ;

	if ((updateContext != NULL) && (updateUsrRef != NULL)) {
		/*
		** Update the CFS from the Cfs object 
		*/
		printf("AUTOMATIC-UPDATE of the CMIP Filing System (CFS)\n\n") ;
		ret = cfs_updateBranchOfCfs(
			updateContext,
			updateUsrRef
		) ;
	} 

	if (ret == OM_SUCCESS) {
		/*
		** Register a signal handler for the alarm signal
		*/
		if (signal(SIGALRM,cfs_timerproc) == SIG_ERR)
			ret = OM_FUNCTION_DECLINED ;

		if (ret == OM_SUCCESS)
			/*
			** Set the alarm clock to send a signal after 
			** an interval of UPDATETIME seconds has passed
			*/
			alarm(UPDATETIME) ; 
	}
	
	/*
	** Problem encountered
	*/
	if (ret != OM_SUCCESS) {
		/*
		** De-activates the autoupdate if an error occures
		*/
		printf("cfs_timerproc : Update error, cancel AutoUpdate\n") ;
		cfsInstance = (cfs_t*) updateUsrRef ;
		cfsInstance->cfsAutoUpdate = OM_FALSE ;

		/*
		** Cancel the alarm clock 
		*/
		updateUsrRef = NULL ;
		updateContext = NULL ;
		alarm(0) ;
	}
}
