#ifndef __CFS_H__
#define __CFS_H__


#include "asn1.h"

/*
** Global variables for example purposes
*/
void *updateUsrRef ;
ca_context_t *updateContext ;
void *cfsRootDirectory ;

/*
** Instance structure
*/

typedef struct _cfs_t {

	CFSObject_ASN1module_CfsId cfsId ;
	CFSObject_ASN1module_CfsDirectory cfsName ;
	CFSObject_ASN1module_AutoUpdate cfsAutoUpdate ;
	ca_binding_t *nameBinding ;
	OM_boolean packagesPresent ;
	Attribute_ASN1Module_Packages packages ;
	OM_boolean allomorphsPresent ;
	Attribute_ASN1Module_Allomorphs allomorphs ;

	/* Notifications management */
	char * attrValChngLst ;

} cfs_t ;


extern OM_return_code cfs_init(
	ca_context_t	*context,
	void		*sup,
	void		**current
) ;

extern OM_return_code cfs_create(
	ca_context_t	*context,
	void		*caSupRef,
	ca_binding_t	*binding,
	OM_descriptor	*localName,
	void		**caRef,
	void		**accessRef,
	OM_descriptor	*error
) ;

extern OM_return_code cfs_delete(
	ca_context_t	*context,
	OM_sint32	invokeId,
	OM_boolean	type,
	void		*usrRef,
	void		*caRef,
	void		*accessRef
) ;

extern OM_return_code cfs_free_instance(
	ca_context_t *context,
	void *caRef,
	void *usrRef
) ;

extern OM_return_code cfs_check(
	ca_context_t	*context,
	OM_sint		oper,
	void		*caRef,
	void		*usrRef
) ;

extern OM_return_code cfs_cfsId_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
) ;

extern OM_return_code cfs_cfsId_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
) ;

extern OM_return_code cfs_cfsId_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
) ;

extern OM_return_code cfs_cfsName_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
) ;

extern OM_return_code cfs_cfsName_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
) ;

extern OM_return_code cfs_cfsName_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
) ;

extern OM_return_code cfs_cfsAutoUpdate_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
) ;

extern OM_return_code cfs_cfsAutoUpdate_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
) ;

extern OM_return_code cfs_cfsAutoUpdate_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
) ;

extern OM_return_code cfs_objectClass_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
) ;

extern OM_return_code cfs_nameBinding_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
) ;

extern OM_return_code cfs_packages_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
) ;

extern OM_return_code cfs_packages_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
) ;

extern OM_return_code cfs_packages_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
) ;

extern OM_return_code cfs_allomorphs_get(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value
) ;

extern OM_return_code cfs_allomorphs_set(
	ca_context_t	*context,
	OM_enumeration	modifyOperator,
	OM_descriptor	*valueForOperation,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*resultingValue
) ;

extern OM_return_code cfs_allomorphs_eval(
	ca_context_t	*context,
	void		*usrRef,
	void		*accessRef,
	OM_descriptor	*value1,
	OM_descriptor	*value2,
	OM_type		type,
	OM_boolean	*result,
	OM_boolean	*error
) ;

extern OM_return_code cfs_cfsUpdateSystem(
	ca_context_t	*context,
	OM_sint32	invokeId,
	OM_uint	mode,
	OM_descriptor	*actionInfoArg,
	void		*usrRef,
	void		*accessRef
) ;

/*
** Functions added for example purposes
*/

OM_return_code cfs_getFileName(
	char 		*filepath, 
	char 		**filename
) ;

OM_return_code cfs_findFilesOfDirectory(
	char 		*parentDir, 
	ca_list_t	 **list , 
	OM_uint 	*nbItem
) ;

OM_return_code cfs_freeListOfFiles(
	ca_list_t 	*list
) ;

OM_return_code cfs_removeNodeFromList(
	ca_list_t 	**list, 
	OM_uint 	position
) ;

OM_return_code cfs_isDirectory(
	char 		*filename, 
	OM_boolean 	*isDir
) ;

OM_return_code cfs_isClasscfs(
	ca_context_t 	*context, 
	void 		*caRef, 
	OM_boolean 	*isCfs
) ;

OM_return_code cfs_isClasscfsDir(
	ca_context_t 	*context, 
	void 		*caRef, 
	OM_boolean 	*iscfsDir
) ;

OM_return_code cfs_findcaRef(
	ca_context_t 	*context, 
	void 		*usrRef, 
	void 		**caRef
) ;

OM_return_code cfs_findusrRef(
	ca_context_t 	*context, 
	void 		*caRef, 
	void 		**usrRef
) ;

OM_return_code cfs_createCfs(
	ca_context_t 	*context,
	void		*caSupRef,
	void		**current
) ;

OM_return_code cfs_createBranchOfCfs(
	ca_context_t	*context,
	char		*directory,
	void		*caSupRef,
	void		*accessRef
) ;

OM_return_code cfs_createInstance(
	ca_context_t 	*context,
	void 		*caSupRef, 
	char 		*filePath,
	char 		*instanceName
) ;

OM_return_code cfs_createcfsFile(
	ca_context_t 	*context,
	void 		*caSupRef, 
	char 		*filePath,
	char 		*instanceName,
	void		**caRef
) ;

OM_return_code cfs_createcfsDir(
	ca_context_t 	*context,
	void 		*caSupRef, 
	char 		*filePath,
	char 		*instanceName,
	void		**caRef
) ;

OM_return_code cfs_setcfsFile(
	ca_context_t 	*context,
	void		*caRef
) ;

OM_return_code cfs_setcfsDir(
	ca_context_t 	*context,
	void		*caRef
) ;

OM_return_code cfs_getFullPath(
	ca_context_t 	*context, 
	void 		*usrRef, 
	char 		*new, 
	char 		**path
) ;


OM_return_code cfs_updateBranchOfCfs(
	ca_context_t	*context,
	void		*usrRef
) ;

OM_return_code cfs_updateInstance(
	ca_context_t 	*context, 
	void 		*usrRef, 
	void 		*caRef
) ;

OM_return_code cfs_updatecfsFile(
	ca_context_t 	*context, 
	void 		*usrRef,
	void		*caRef  
) ;

OM_return_code cfs_updatecfsDir(
	ca_context_t 	*context, 
	void 		*usrRef,
	void		*caRef 
) ;

OM_return_code cfs_removeInstance(
	ca_context_t 	*context, 
	ca_list_t 	*list
) ;

OM_return_code cfs_removecfsFile(
	ca_context_t 	*context, 
	void		*usrRef
) ;

OM_return_code cfs_removecfsDir(
	ca_context_t 	*context, 
	void		*usrRef
) ;

OM_return_code cfs_getFilePermissions(
	ca_context_t 	*context, 
	void 		*usrRef,
	OM_descriptor  *FilePermissions
) ;

static void cfs_timerproc(
	int 		sig
) ;

#endif /* __CFS_H__ */

