#include <stdio.h>
#ifdef WIN32
#include <xom.h>
#else /* WIN32 */
#include <cmip/xom.h>
#endif /* WIN32 */
#include "asn1.h"





/* =============== Type PrintableString =============== */

OM_return_code enc_PrintableString_content(PrintableString *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;

	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = ca_ber_enc_OctetString_content(value,
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Sum up the length of the subtype encoding */
		*contLen += subContLen;
	}

	/* Ignore the IMPLICIT tag of the subtype OctetString */


	/* Build leading tag encoding for OCTET_STRING : tag [UNIV 19] form is computed from previous tags */
	if (result == OM_SUCCESS) {
		*tagForm = subTagForm;
		result = ca_ber_encTag(tagBuffer, 19|TAG_UNIV_CLASS|*tagForm, tagLen);
	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type CMIP_1_ObjectClass =============== */

OM_return_code enc_CMIP_1_ObjectClass_content(CMIP_1_ObjectClass *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;
	switch(value->choice) {

		case CMIP_1_ObjectClass_globalForm : {
			ca_buff_setwrite(subTagBuffer);
			result = ca_buff_status(subTagBuffer);

			if (result == OM_SUCCESS) {
				result = ca_ber_enc_ObjectId_content(&(value->value.globalForm),
						contBuffer, &subContLen,
						subTagBuffer, &subTagLen, &subTagForm);
			}

			/* Sum up the length of the subtype encoding */
			*contLen += subContLen;

			/* Ignore the IMPLICIT tag of the subtype OM_object_identifier */


			/* Build leading tag encoding for OBJECT_IDENTIFIER : tag [CTXT 0] form is computed from previous tags */
			if (result == OM_SUCCESS) {
				*tagForm = subTagForm;
				result = ca_ber_encTag(tagBuffer, 0|TAG_CTXT_CLASS|*tagForm, tagLen);
			}
			break;
		}

		case CMIP_1_ObjectClass_localForm : {
			ca_buff_setwrite(subTagBuffer);
			result = ca_buff_status(subTagBuffer);

			if (result == OM_SUCCESS) {
				result = ca_ber_enc_Integer_content(&(value->value.localForm),
						contBuffer, &subContLen,
						subTagBuffer, &subTagLen, &subTagForm);
			}

			/* Sum up the length of the subtype encoding */
			*contLen += subContLen;

			/* Ignore the IMPLICIT tag of the subtype OM_integer */


			/* Build leading tag encoding for INTEGER : tag [CTXT 1] form is computed from previous tags */
			if (result == OM_SUCCESS) {
				*tagForm = subTagForm;
				result = ca_ber_encTag(tagBuffer, 1|TAG_CTXT_CLASS|*tagForm, tagLen);
			}
			break;
		}

	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type Attribute_ASN1Module_Packages =============== */

OM_return_code enc_Attribute_ASN1Module_Packages_content(Attribute_ASN1Module_Packages *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;
	Attribute_ASN1Module_PackagesNode *item = NULL;


	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;

	for (item = value->tail; (item != NULL) && (result == OM_SUCCESS); item = item ->prev) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = ca_ber_enc_ObjectId_content(&(item->elt),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) OM_object_identifier */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}
		*contLen += subContLen;
	}

	/* Encode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_encLength(*contLen, contBuffer, &lenLen);
		*contLen += lenLen;
	}

	/* Build leading tag encoding for SET_OF : tag is CONSTRUCTED */
	if (result == OM_SUCCESS) {
		result = ca_ber_encTag(tagBuffer, TAG_SET_OF, tagLen);
		*tagForm = TAG_CONS_FORM;
	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type Attribute_ASN1Module_Allomorphs =============== */

OM_return_code enc_Attribute_ASN1Module_Allomorphs_content(Attribute_ASN1Module_Allomorphs *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;
	Attribute_ASN1Module_AllomorphsNode *item = NULL;


	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;

	for (item = value->tail; (item != NULL) && (result == OM_SUCCESS); item = item ->prev) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_Attribute_ASN1Module_ObjectClass_content(&(item->elt),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) Attribute_ASN1Module_ObjectClass */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}
		*contLen += subContLen;
	}

	/* Encode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_encLength(*contLen, contBuffer, &lenLen);
		*contLen += lenLen;
	}

	/* Build leading tag encoding for SET_OF : tag is CONSTRUCTED */
	if (result == OM_SUCCESS) {
		result = ca_ber_encTag(tagBuffer, TAG_SET_OF, tagLen);
		*tagForm = TAG_CONS_FORM;
	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type CFSObject_ASN1module_PermissionBits =============== */

OM_return_code enc_CFSObject_ASN1module_PermissionBits_content(CFSObject_ASN1module_PermissionBits *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;
/*
** Member execute
*/


	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = ca_ber_enc_Boolean_content(&(value->execute),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) OM_boolean */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}

		/* Sum up the length of the subtype encoding */
		*contLen += subContLen;
	}
/*
** Member write
*/


	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = ca_ber_enc_Boolean_content(&(value->write),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) OM_boolean */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}

		/* Sum up the length of the subtype encoding */
		*contLen += subContLen;
	}
/*
** Member read
*/


	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = ca_ber_enc_Boolean_content(&(value->read),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) OM_boolean */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}

		/* Sum up the length of the subtype encoding */
		*contLen += subContLen;
	}

	/* Encode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_encLength(*contLen, contBuffer, &lenLen);
		*contLen += lenLen;
	}

	/* Build leading tag encoding for SEQUENCE : tag is CONSTRUCTED */
	if (result == OM_SUCCESS) {
		result = ca_ber_encTag(tagBuffer, TAG_SEQUENCE, tagLen);
		*tagForm = TAG_CONS_FORM;
	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type CFSObject_ASN1module_FilePermissions =============== */

OM_return_code enc_CFSObject_ASN1module_FilePermissions_content(CFSObject_ASN1module_FilePermissions *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;
/*
** Member other
*/


	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_CFSObject_ASN1module_PermissionBits_content(&(value->other),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) CFSObject_ASN1module_PermissionBits */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}

		/* Sum up the length of the subtype encoding */
		*contLen += subContLen;
	}
/*
** Member group
*/


	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_CFSObject_ASN1module_PermissionBits_content(&(value->group),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) CFSObject_ASN1module_PermissionBits */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}

		/* Sum up the length of the subtype encoding */
		*contLen += subContLen;
	}
/*
** Member user
*/


	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_CFSObject_ASN1module_PermissionBits_content(&(value->user),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) CFSObject_ASN1module_PermissionBits */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}

		/* Sum up the length of the subtype encoding */
		*contLen += subContLen;
	}

	/* Encode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_encLength(*contLen, contBuffer, &lenLen);
		*contLen += lenLen;
	}

	/* Build leading tag encoding for SEQUENCE : tag is CONSTRUCTED */
	if (result == OM_SUCCESS) {
		result = ca_ber_encTag(tagBuffer, TAG_SEQUENCE, tagLen);
		*tagForm = TAG_CONS_FORM;
	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type GraphicString =============== */

OM_return_code enc_GraphicString_content(GraphicString *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;

	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = ca_ber_enc_OctetString_content(value,
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Sum up the length of the subtype encoding */
		*contLen += subContLen;
	}

	/* Ignore the IMPLICIT tag of the subtype OctetString */


	/* Build leading tag encoding for OCTET_STRING : tag [UNIV 25] form is computed from previous tags */
	if (result == OM_SUCCESS) {
		*tagForm = subTagForm;
		result = ca_ber_encTag(tagBuffer, 25|TAG_UNIV_CLASS|*tagForm, tagLen);
	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type Attribute_ASN1Module_SystemId =============== */

OM_return_code enc_Attribute_ASN1Module_SystemId_content(Attribute_ASN1Module_SystemId *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;
	switch(value->choice) {

		case Attribute_ASN1Module_SystemId_name : {
			ca_buff_setwrite(subTagBuffer);
			result = ca_buff_status(subTagBuffer);

			if (result == OM_SUCCESS) {
				result = enc_GraphicString_content(&(value->value.name),
						contBuffer, &subContLen,
						tagBuffer, tagLen, tagForm);
			}

			/* Sum up the length of the subtype encoding */
			*contLen += subContLen;
			break;
		}

		case Attribute_ASN1Module_SystemId_number : {
			ca_buff_setwrite(subTagBuffer);
			result = ca_buff_status(subTagBuffer);

			if (result == OM_SUCCESS) {
				result = ca_ber_enc_Integer_content(&(value->value.number),
						contBuffer, &subContLen,
						tagBuffer, tagLen, tagForm);
			}

			/* Sum up the length of the subtype encoding */
			*contLen += subContLen;
			break;
		}

		case Attribute_ASN1Module_SystemId_nothing : {
			ca_buff_setwrite(subTagBuffer);
			result = ca_buff_status(subTagBuffer);

			if (result == OM_SUCCESS) {
				result = ca_ber_enc_Null_content(
						contBuffer, &subContLen,
						tagBuffer, tagLen, tagForm);
			}

			/* Sum up the length of the subtype encoding */
			*contLen += subContLen;
			break;
		}

	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type InformationFramework_AttributeValueAssertion =============== */

OM_return_code enc_InformationFramework_AttributeValueAssertion_content(InformationFramework_AttributeValueAssertion *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;
/*
** Member unnamedfield2
*/


	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_InformationFramework_AttributeValue_content(&(value->unnamedfield2),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) InformationFramework_AttributeValue */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}

		/* Sum up the length of the subtype encoding */
		*contLen += subContLen;
	}
/*
** Member unnamedfield1
*/


	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_InformationFramework_AttributeType_content(&(value->unnamedfield1),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) InformationFramework_AttributeType */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}

		/* Sum up the length of the subtype encoding */
		*contLen += subContLen;
	}

	/* Encode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_encLength(*contLen, contBuffer, &lenLen);
		*contLen += lenLen;
	}

	/* Build leading tag encoding for SEQUENCE : tag is CONSTRUCTED */
	if (result == OM_SUCCESS) {
		result = ca_ber_encTag(tagBuffer, TAG_SEQUENCE, tagLen);
		*tagForm = TAG_CONS_FORM;
	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type InformationFramework_RelativeDistinguishedName =============== */

OM_return_code enc_InformationFramework_RelativeDistinguishedName_content(InformationFramework_RelativeDistinguishedName *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;
	InformationFramework_RelativeDistinguishedNameNode *item = NULL;


	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;

	for (item = value->tail; (item != NULL) && (result == OM_SUCCESS); item = item ->prev) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_InformationFramework_AttributeValueAssertion_content(&(item->elt),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) InformationFramework_AttributeValueAssertion */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}
		*contLen += subContLen;
	}

	/* Encode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_encLength(*contLen, contBuffer, &lenLen);
		*contLen += lenLen;
	}

	/* Build leading tag encoding for SET_OF : tag is CONSTRUCTED */
	if (result == OM_SUCCESS) {
		result = ca_ber_encTag(tagBuffer, TAG_SET_OF, tagLen);
		*tagForm = TAG_CONS_FORM;
	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type InformationFramework_RDNSequence =============== */

OM_return_code enc_InformationFramework_RDNSequence_content(InformationFramework_RDNSequence *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;
	InformationFramework_RDNSequenceNode *item = NULL;


	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;

	for (item = value->tail; (item != NULL) && (result == OM_SUCCESS); item = item ->prev) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_InformationFramework_RelativeDistinguishedName_content(&(item->elt),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) InformationFramework_RelativeDistinguishedName */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}
		*contLen += subContLen;
	}

	/* Encode the length of the SEQUENCE_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_encLength(*contLen, contBuffer, &lenLen);
		*contLen += lenLen;
	}

	/* Build leading tag encoding for SEQUENCE_OF : tag is CONSTRUCTED */
	if (result == OM_SUCCESS) {
		result = ca_ber_encTag(tagBuffer, TAG_SEQUENCE_OF, tagLen);
		*tagForm = TAG_CONS_FORM;
	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type Attribute_ASN1Module_SystemTitle =============== */

OM_return_code enc_Attribute_ASN1Module_SystemTitle_content(Attribute_ASN1Module_SystemTitle *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;
	switch(value->choice) {

		case Attribute_ASN1Module_SystemTitle_distinguishedName : {
			ca_buff_setwrite(subTagBuffer);
			result = ca_buff_status(subTagBuffer);

			if (result == OM_SUCCESS) {
				result = enc_Attribute_ASN1Module_DistinguishedName_content(&(value->value.distinguishedName),
						contBuffer, &subContLen,
						tagBuffer, tagLen, tagForm);
			}

			/* Sum up the length of the subtype encoding */
			*contLen += subContLen;
			break;
		}

		case Attribute_ASN1Module_SystemTitle_oid : {
			ca_buff_setwrite(subTagBuffer);
			result = ca_buff_status(subTagBuffer);

			if (result == OM_SUCCESS) {
				result = ca_ber_enc_ObjectId_content(&(value->value.oid),
						contBuffer, &subContLen,
						tagBuffer, tagLen, tagForm);
			}

			/* Sum up the length of the subtype encoding */
			*contLen += subContLen;
			break;
		}

		case Attribute_ASN1Module_SystemTitle_nothing : {
			ca_buff_setwrite(subTagBuffer);
			result = ca_buff_status(subTagBuffer);

			if (result == OM_SUCCESS) {
				result = ca_ber_enc_Null_content(
						contBuffer, &subContLen,
						tagBuffer, tagLen, tagForm);
			}

			/* Sum up the length of the subtype encoding */
			*contLen += subContLen;
			break;
		}

	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type Attribute_ASN1Module_SupportedFeatures_SeqT =============== */

OM_return_code enc_Attribute_ASN1Module_SupportedFeatures_SeqT_content(Attribute_ASN1Module_SupportedFeatures_SeqT *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;
/*
** Member featureInfo
*/


	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = ca_ber_enc_Any_content(&(value->featureInfo),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) OM_descriptor */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}

		/* Sum up the length of the subtype encoding */
		*contLen += subContLen;
	}
/*
** Member featureIdentifier
*/


	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = ca_ber_enc_ObjectId_content(&(value->featureIdentifier),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) OM_object_identifier */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}

		/* Sum up the length of the subtype encoding */
		*contLen += subContLen;
	}

	/* Encode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_encLength(*contLen, contBuffer, &lenLen);
		*contLen += lenLen;
	}

	/* Build leading tag encoding for SEQUENCE : tag is CONSTRUCTED */
	if (result == OM_SUCCESS) {
		result = ca_ber_encTag(tagBuffer, TAG_SEQUENCE, tagLen);
		*tagForm = TAG_CONS_FORM;
	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type Attribute_ASN1Module_SupportedFeatures =============== */

OM_return_code enc_Attribute_ASN1Module_SupportedFeatures_content(Attribute_ASN1Module_SupportedFeatures *value,
			ca_buff_t * contBuffer, int * contLen,
			ca_buff_t * tagBuffer, int * tagLen, int *tagForm)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;
	Attribute_ASN1Module_SupportedFeaturesNode *item = NULL;


	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	/* Initialize overall length */
	*contLen = 0;

	for (item = value->tail; (item != NULL) && (result == OM_SUCCESS); item = item ->prev) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_Attribute_ASN1Module_SupportedFeatures_SeqT_content(&(item->elt),
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}

		/* Keep the tag of the subtype (EXPLICIT) Attribute_ASN1Module_SupportedFeatures_SeqT */
		if (result == OM_SUCCESS) {
			ca_buff_setread(subTagBuffer);
			ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
			result = ca_buff_status(contBuffer);
			subContLen += subTagLen;
		}
		*contLen += subContLen;
	}

	/* Encode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_encLength(*contLen, contBuffer, &lenLen);
		*contLen += lenLen;
	}

	/* Build leading tag encoding for SET_OF : tag is CONSTRUCTED */
	if (result == OM_SUCCESS) {
		result = ca_ber_encTag(tagBuffer, TAG_SET_OF, tagLen);
		*tagForm = TAG_CONS_FORM;
	}

	/* Release local tag buffer */
	ca_buff_release(subTagBuffer);

	return result;
}




/* =============== Type CFSObject_ASN1module_CfsId =============== */

OM_return_code enc_CFSObject_ASN1module_CfsId(CFSObject_ASN1module_CfsId *value,
			OM_descriptor * encoding)
{
	encoding->syntax = OM_S_PRINTABLE_STRING;
	return ca_xom_strcpy(&(encoding->value.string), value);

}



/* =============== Type CFSObject_ASN1module_CfsDirectory =============== */

OM_return_code enc_CFSObject_ASN1module_CfsDirectory(CFSObject_ASN1module_CfsDirectory *value,
			OM_descriptor * encoding)
{
	encoding->syntax = OM_S_PRINTABLE_STRING;
	return ca_xom_strcpy(&(encoding->value.string), value);

}



/* =============== Type CFSObject_ASN1module_AutoUpdate =============== */

OM_return_code enc_CFSObject_ASN1module_AutoUpdate(CFSObject_ASN1module_AutoUpdate *value,
			OM_descriptor * encoding)
{
	encoding->syntax = OM_S_BOOLEAN;
	encoding->value.boolean = *value;

	return OM_SUCCESS;
}



/* =============== Type CMIP_1_ObjectClass =============== */

OM_return_code enc_CMIP_1_ObjectClass(CMIP_1_ObjectClass *value,
			OM_descriptor * encoding)
{
	OM_return_code result = OM_SUCCESS;
	ca_buff_t cb, *contBuffer = &cb;
	int subContLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;


	/* Initialize content buffer */
	ca_buff_init(contBuffer);

	if (result == OM_SUCCESS) {
		ca_buff_setwrite(contBuffer);
		result = ca_buff_status(contBuffer);
	}

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_CMIP_1_ObjectClass_content(value,
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}
	}

	/* Keep the tag of the subtype (EXPLICIT) CMIP_1_ObjectClass */
	if (result == OM_SUCCESS) {
		ca_buff_setread(subTagBuffer);
		ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
		result = ca_buff_status(contBuffer);
	}

	/* Flush the buffer containing the encoding */
	if (result == OM_SUCCESS) {
		ca_buff_tom(contBuffer, encoding);
		result = ca_buff_status(contBuffer);
	}

	/* Release buffers */
	ca_buff_release(contBuffer);
	ca_buff_release(subTagBuffer);

	return result;
}



/* =============== Type Attribute_ASN1Module_NameBinding =============== */

OM_return_code enc_Attribute_ASN1Module_NameBinding(Attribute_ASN1Module_NameBinding *value,
			OM_descriptor * encoding)
{
	encoding->syntax = OM_S_OBJECT_IDENTIFIER_STRING;
	return ca_xom_strcpy(&(encoding->value.string), value);

}



/* =============== Type Attribute_ASN1Module_Packages =============== */

OM_return_code enc_Attribute_ASN1Module_Packages(Attribute_ASN1Module_Packages *value,
			OM_descriptor * encoding)
{
	OM_return_code result = OM_SUCCESS;
	ca_buff_t cb, *contBuffer = &cb;
	int subContLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;


	/* Initialize content buffer */
	ca_buff_init(contBuffer);

	if (result == OM_SUCCESS) {
		ca_buff_setwrite(contBuffer);
		result = ca_buff_status(contBuffer);
	}

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_Attribute_ASN1Module_Packages_content(value,
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}
	}

	/* Keep the tag of the subtype (EXPLICIT) Attribute_ASN1Module_Packages */
	if (result == OM_SUCCESS) {
		ca_buff_setread(subTagBuffer);
		ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
		result = ca_buff_status(contBuffer);
	}

	/* Flush the buffer containing the encoding */
	if (result == OM_SUCCESS) {
		ca_buff_tom(contBuffer, encoding);
		result = ca_buff_status(contBuffer);
	}

	/* Release buffers */
	ca_buff_release(contBuffer);
	ca_buff_release(subTagBuffer);

	return result;
}



/* =============== Type Attribute_ASN1Module_Allomorphs =============== */

OM_return_code enc_Attribute_ASN1Module_Allomorphs(Attribute_ASN1Module_Allomorphs *value,
			OM_descriptor * encoding)
{
	OM_return_code result = OM_SUCCESS;
	ca_buff_t cb, *contBuffer = &cb;
	int subContLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;


	/* Initialize content buffer */
	ca_buff_init(contBuffer);

	if (result == OM_SUCCESS) {
		ca_buff_setwrite(contBuffer);
		result = ca_buff_status(contBuffer);
	}

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_Attribute_ASN1Module_Allomorphs_content(value,
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}
	}

	/* Keep the tag of the subtype (EXPLICIT) Attribute_ASN1Module_Allomorphs */
	if (result == OM_SUCCESS) {
		ca_buff_setread(subTagBuffer);
		ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
		result = ca_buff_status(contBuffer);
	}

	/* Flush the buffer containing the encoding */
	if (result == OM_SUCCESS) {
		ca_buff_tom(contBuffer, encoding);
		result = ca_buff_status(contBuffer);
	}

	/* Release buffers */
	ca_buff_release(contBuffer);
	ca_buff_release(subTagBuffer);

	return result;
}



/* =============== Type CFSObject_ASN1module_FilePermissions =============== */

OM_return_code enc_CFSObject_ASN1module_FilePermissions(CFSObject_ASN1module_FilePermissions *value,
			OM_descriptor * encoding)
{
	OM_return_code result = OM_SUCCESS;
	ca_buff_t cb, *contBuffer = &cb;
	int subContLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;


	/* Initialize content buffer */
	ca_buff_init(contBuffer);

	if (result == OM_SUCCESS) {
		ca_buff_setwrite(contBuffer);
		result = ca_buff_status(contBuffer);
	}

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_CFSObject_ASN1module_FilePermissions_content(value,
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}
	}

	/* Keep the tag of the subtype (EXPLICIT) CFSObject_ASN1module_FilePermissions */
	if (result == OM_SUCCESS) {
		ca_buff_setread(subTagBuffer);
		ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
		result = ca_buff_status(contBuffer);
	}

	/* Flush the buffer containing the encoding */
	if (result == OM_SUCCESS) {
		ca_buff_tom(contBuffer, encoding);
		result = ca_buff_status(contBuffer);
	}

	/* Release buffers */
	ca_buff_release(contBuffer);
	ca_buff_release(subTagBuffer);

	return result;
}



/* =============== Type Attribute_ASN1Module_SystemId =============== */

OM_return_code enc_Attribute_ASN1Module_SystemId(Attribute_ASN1Module_SystemId *value,
			OM_descriptor * encoding)
{
	switch(value->choice) {

		case Attribute_ASN1Module_SystemId_name : {
			encoding->syntax = OM_S_GRAPHIC_STRING;
			return ca_xom_strcpy(&(encoding->value.string), &(value->value.name));

		}

		case Attribute_ASN1Module_SystemId_number : {
			encoding->syntax = OM_S_INTEGER;
			encoding->value.integer = value->value.number;

			return OM_SUCCESS;
		}

		case Attribute_ASN1Module_SystemId_nothing : {
			encoding->syntax = OM_S_NULL;
			return OM_SUCCESS;
		}

	}

}



/* =============== Type Attribute_ASN1Module_SystemTitle =============== */

OM_return_code enc_Attribute_ASN1Module_SystemTitle(Attribute_ASN1Module_SystemTitle *value,
			OM_descriptor * encoding)
{
	switch(value->choice) {

		case Attribute_ASN1Module_SystemTitle_oid : {
			encoding->syntax = OM_S_OBJECT_IDENTIFIER_STRING;
			return ca_xom_strcpy(&(encoding->value.string), &(value->value.oid));

		}

		case Attribute_ASN1Module_SystemTitle_nothing : {
			encoding->syntax = OM_S_NULL;
			return OM_SUCCESS;
		}

		case Attribute_ASN1Module_SystemTitle_distinguishedName : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subContLen;
			ca_buff_t stb, *subTagBuffer = &stb;
			int subTagLen, subTagForm;


			/* Initialize content buffer */
			ca_buff_init(contBuffer);

			if (result == OM_SUCCESS) {
				ca_buff_setwrite(contBuffer);
				result = ca_buff_status(contBuffer);
			}

			/* Initialize local tag buffer */
			ca_buff_init(subTagBuffer);

			if (result == OM_SUCCESS) {
				ca_buff_setwrite(subTagBuffer);
				result = ca_buff_status(subTagBuffer);

				if (result == OM_SUCCESS) {
					result = enc_Attribute_ASN1Module_SystemTitle_content(value,
							contBuffer, &subContLen,
							subTagBuffer, &subTagLen, &subTagForm);
				}
			}

			/* Keep the tag of the subtype (EXPLICIT) Attribute_ASN1Module_SystemTitle */
			if (result == OM_SUCCESS) {
				ca_buff_setread(subTagBuffer);
				ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
				result = ca_buff_status(contBuffer);
			}

			/* Flush the buffer containing the encoding */
			if (result == OM_SUCCESS) {
				ca_buff_tom(contBuffer, encoding);
				result = ca_buff_status(contBuffer);
			}

			/* Release buffers */
			ca_buff_release(contBuffer);
			ca_buff_release(subTagBuffer);

			return result;
		}

	}
}



/* =============== Type Attribute_ASN1Module_OperationalState =============== */

OM_return_code enc_Attribute_ASN1Module_OperationalState(Attribute_ASN1Module_OperationalState *value,
			OM_descriptor * encoding)
{
	encoding->syntax = OM_S_ENUMERATION;
	encoding->value.enumeration = *value;

	return OM_SUCCESS;
}



/* =============== Type Attribute_ASN1Module_UsageState =============== */

OM_return_code enc_Attribute_ASN1Module_UsageState(Attribute_ASN1Module_UsageState *value,
			OM_descriptor * encoding)
{
	encoding->syntax = OM_S_ENUMERATION;
	encoding->value.enumeration = *value;

	return OM_SUCCESS;
}



/* =============== Type Attribute_ASN1Module_AdministrativeState =============== */

OM_return_code enc_Attribute_ASN1Module_AdministrativeState(Attribute_ASN1Module_AdministrativeState *value,
			OM_descriptor * encoding)
{
	encoding->syntax = OM_S_ENUMERATION;
	encoding->value.enumeration = *value;

	return OM_SUCCESS;
}



/* =============== Type Attribute_ASN1Module_SupportedFeatures =============== */

OM_return_code enc_Attribute_ASN1Module_SupportedFeatures(Attribute_ASN1Module_SupportedFeatures *value,
			OM_descriptor * encoding)
{
	OM_return_code result = OM_SUCCESS;
	ca_buff_t cb, *contBuffer = &cb;
	int subContLen;
	ca_buff_t stb, *subTagBuffer = &stb;
	int subTagLen, subTagForm;


	/* Initialize content buffer */
	ca_buff_init(contBuffer);

	if (result == OM_SUCCESS) {
		ca_buff_setwrite(contBuffer);
		result = ca_buff_status(contBuffer);
	}

	/* Initialize local tag buffer */
	ca_buff_init(subTagBuffer);

	if (result == OM_SUCCESS) {
		ca_buff_setwrite(subTagBuffer);
		result = ca_buff_status(subTagBuffer);

		if (result == OM_SUCCESS) {
			result = enc_Attribute_ASN1Module_SupportedFeatures_content(value,
					contBuffer, &subContLen,
					subTagBuffer, &subTagLen, &subTagForm);
		}
	}

	/* Keep the tag of the subtype (EXPLICIT) Attribute_ASN1Module_SupportedFeatures */
	if (result == OM_SUCCESS) {
		ca_buff_setread(subTagBuffer);
		ca_buff_putb(contBuffer, subTagBuffer, subTagLen);
		result = ca_buff_status(contBuffer);
	}

	/* Flush the buffer containing the encoding */
	if (result == OM_SUCCESS) {
		ca_buff_tom(contBuffer, encoding);
		result = ca_buff_status(contBuffer);
	}

	/* Release buffers */
	ca_buff_release(contBuffer);
	ca_buff_release(subTagBuffer);

	return result;
}



/* =============== Type PrintableString =============== */

OM_return_code dec_PrintableString_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			PrintableString *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;

	*decLen = 0;

	/* Check leading tag [UNIV 19] */
	if (tagIsExplicit) {
		if (tagValue != 19
		    || tagClass != TAG_UNIV_CLASS)
		  result = OM_TEMPORARY_ERROR;
	}

	if (result == OM_SUCCESS) {
		subContLen = contLen;
	}

	if (result == OM_SUCCESS) {
		result = ca_ber_dec_OctetString_content(contBuffer,
						subContLen,
						0 /* implicit form */,
						tagValue,
						tagClass,
						tagForm,
						value,
						&subDecLen);
		*decLen += subDecLen;

		if (result == OM_SUCCESS) {
			contLen -= subDecLen;
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}
	return result;
}




/* =============== Type CMIP_1_ObjectClass =============== */

OM_return_code dec_CMIP_1_ObjectClass_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			CMIP_1_ObjectClass *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;

	*decLen = 0;

	if (result == OM_SUCCESS) {
		subContLen = contLen;
	}

	/* Try to detect presence of attribute globalForm in CMIP_1_ObjectClass */

	if (result == OM_SUCCESS) {
		result = OM_SUCCESS;

		/* Check leading tag [CTXT 0] */
		if (tagIsExplicit) {
			if (tagValue != 0
			    || tagClass != TAG_CTXT_CLASS)
			  result = OM_TEMPORARY_ERROR;
		}

		if (result == OM_SUCCESS) {
			result = ca_ber_dec_ObjectId_content(contBuffer,
							subContLen,
							0 /* implicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->value.globalForm),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS)
			value->choice = CMIP_1_ObjectClass_globalForm;
	}

	/* Try to detect presence of attribute localForm in CMIP_1_ObjectClass */

	if (result == OM_TEMPORARY_ERROR) {
		result = OM_SUCCESS;

		/* Check leading tag [CTXT 1] */
		if (tagIsExplicit) {
			if (tagValue != 1
			    || tagClass != TAG_CTXT_CLASS)
			  result = OM_TEMPORARY_ERROR;
		}

		if (result == OM_SUCCESS) {
			result = ca_ber_dec_Integer_content(contBuffer,
							subContLen,
							0 /* implicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->value.localForm),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS)
			value->choice = CMIP_1_ObjectClass_localForm;
	}
	return result;
}




/* =============== Type Attribute_ASN1Module_Packages =============== */

OM_return_code dec_Attribute_ASN1Module_Packages_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_Packages *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	Attribute_ASN1Module_PackagesNode *item = NULL;


	*decLen = 0;

	/* Check leading tag for SET_OF */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SET_OF)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

	value->head = value->tail = NULL;
	while (result == OM_SUCCESS) {

		if (contLen == 0)
			break;


		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the OBJECT_IDENTIFIER subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS) {
			if ((item = ca_ber_newItem(Attribute_ASN1Module_PackagesNode)) == NULL)
				result = OM_MEMORY_INSUFFICIENT;
			else {
				item->prev = value->tail;
				item->next = NULL;
				if (value->head == NULL)
					value->head = item;
				else
					value->tail->next = item;
				value->tail = item;
			}
		}

		if (result == OM_SUCCESS) {
			result = ca_ber_dec_ObjectId_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(item->elt),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type Attribute_ASN1Module_Allomorphs =============== */

OM_return_code dec_Attribute_ASN1Module_Allomorphs_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_Allomorphs *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	Attribute_ASN1Module_AllomorphsNode *item = NULL;


	*decLen = 0;

	/* Check leading tag for SET_OF */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SET_OF)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

	value->head = value->tail = NULL;
	while (result == OM_SUCCESS) {

		if (contLen == 0)
			break;


		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the Attribute_ASN1Module_ObjectClass subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS) {
			if ((item = ca_ber_newItem(Attribute_ASN1Module_AllomorphsNode)) == NULL)
				result = OM_MEMORY_INSUFFICIENT;
			else {
				item->prev = value->tail;
				item->next = NULL;
				if (value->head == NULL)
					value->head = item;
				else
					value->tail->next = item;
				value->tail = item;
			}
		}

		if (result == OM_SUCCESS) {
			result = dec_Attribute_ASN1Module_ObjectClass_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(item->elt),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type CMIP_1_AttributeId =============== */

OM_return_code dec_CMIP_1_AttributeId_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			CMIP_1_AttributeId *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;

	*decLen = 0;

	if (result == OM_SUCCESS) {
		subContLen = contLen;
	}

	/* Try to detect presence of attribute globalForm in CMIP_1_AttributeId */

	if (result == OM_SUCCESS) {
		result = OM_SUCCESS;

		/* Check leading tag [CTXT 0] */
		if (tagIsExplicit) {
			if (tagValue != 0
			    || tagClass != TAG_CTXT_CLASS)
			  result = OM_TEMPORARY_ERROR;
		}

		if (result == OM_SUCCESS) {
			result = ca_ber_dec_ObjectId_content(contBuffer,
							subContLen,
							0 /* implicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->value.globalForm),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS)
			value->choice = CMIP_1_AttributeId_globalForm;
	}

	/* Try to detect presence of attribute localForm in CMIP_1_AttributeId */

	if (result == OM_TEMPORARY_ERROR) {
		result = OM_SUCCESS;

		/* Check leading tag [CTXT 1] */
		if (tagIsExplicit) {
			if (tagValue != 1
			    || tagClass != TAG_CTXT_CLASS)
			  result = OM_TEMPORARY_ERROR;
		}

		if (result == OM_SUCCESS) {
			result = ca_ber_dec_Integer_content(contBuffer,
							subContLen,
							0 /* implicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->value.localForm),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS)
			value->choice = CMIP_1_AttributeId_localForm;
	}
	return result;
}




/* =============== Type Attribute_ASN1Module_AttributeIdentifierList =============== */

OM_return_code dec_Attribute_ASN1Module_AttributeIdentifierList_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_AttributeIdentifierList *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	Attribute_ASN1Module_AttributeIdentifierListNode *item = NULL;


	*decLen = 0;

	/* Check leading tag for SET_OF */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SET_OF)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

	value->head = value->tail = NULL;
	while (result == OM_SUCCESS) {

		if (contLen == 0)
			break;


		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the Attribute_ASN1Module_AttributeId subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS) {
			if ((item = ca_ber_newItem(Attribute_ASN1Module_AttributeIdentifierListNode)) == NULL)
				result = OM_MEMORY_INSUFFICIENT;
			else {
				item->prev = value->tail;
				item->next = NULL;
				if (value->head == NULL)
					value->head = item;
				else
					value->tail->next = item;
				value->tail = item;
			}
		}

		if (result == OM_SUCCESS) {
			result = dec_Attribute_ASN1Module_AttributeId_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(item->elt),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type Attribute_ASN1Module_AttributeValueChangeDefinition_SetOf_SeqT =============== */

OM_return_code dec_Attribute_ASN1Module_AttributeValueChangeDefinition_SetOf_SeqT_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_AttributeValueChangeDefinition_SetOf_SeqT *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	int decodeNextTag;

	*decLen = 0;

	/* Check leading tag for SEQUENCE */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SEQUENCE)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

/*
** Member attributeID
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the Attribute_ASN1Module_AttributeId subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = dec_Attribute_ASN1Module_AttributeId_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->attributeID),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

/*
** Member oldAttributeValue
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Extract tag [CTXT 1] */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		/* Check tag [CTXT 1] */
		if (result == OM_SUCCESS) {
			if (tagValue != 1
				|| tagClass != TAG_CTXT_CLASS
				|| tagForm != TAG_CONS_FORM)
			  	result = OM_TEMPORARY_ERROR;
		}

		/* Decode the length of the  contents */
		if (result == OM_SUCCESS) {
			result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
			*decLen += lenLen;
		}

		if (result == OM_SUCCESS) {
			contLen -= lenLen;
			if (contLen < 0)
				result = OM_ENCODING_INVALID;
		}

		if (result == OM_SUCCESS) {
			if (subContLen > contLen)
				result = OM_ENCODING_INVALID;
		}


		/* Decode the leading tag of the  subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = ca_ber_dec_Any_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->oldAttributeValue),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		/* Test if optional attribute oldAttributeValue is present (from tag presence) */
		if (result == OM_SUCCESS) {
			decodeNextTag = OM_TRUE;
			value->oldAttributeValueIsPresent = OM_TRUE;
		}
		else if (result == OM_TEMPORARY_ERROR) {
			decodeNextTag = OM_FALSE;
			value->oldAttributeValueIsPresent = OM_FALSE;
			result = OM_SUCCESS;
		}

	}

/*
** Member newAttributeValue
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Extract tag [CTXT 2] */
		if (decodeNextTag) {
			if (result == OM_SUCCESS) {
				result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
				*decLen += tagLen;

				if (result == OM_SUCCESS) {
					contLen -= tagLen;
					subContLen -= tagLen;
					if (contLen < 0)
						result = OM_ENCODING_INVALID;
				}
			}
		}


		/* Check tag [CTXT 2] */
		if (result == OM_SUCCESS) {
			if (tagValue != 2
				|| tagClass != TAG_CTXT_CLASS
				|| tagForm != TAG_CONS_FORM)
			  	result = OM_ENCODING_INVALID;
		}

		/* Decode the length of the  contents */
		if (result == OM_SUCCESS) {
			result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
			*decLen += lenLen;
		}

		if (result == OM_SUCCESS) {
			contLen -= lenLen;
			if (contLen < 0)
				result = OM_ENCODING_INVALID;
		}

		if (result == OM_SUCCESS) {
			if (subContLen > contLen)
				result = OM_ENCODING_INVALID;
		}


		/* Decode the leading tag of the  subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = ca_ber_dec_Any_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->newAttributeValue),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type Attribute_ASN1Module_AttributeValueChangeDefinition =============== */

OM_return_code dec_Attribute_ASN1Module_AttributeValueChangeDefinition_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_AttributeValueChangeDefinition *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	Attribute_ASN1Module_AttributeValueChangeDefinitionNode *item = NULL;


	*decLen = 0;

	/* Check leading tag for SET_OF */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SET_OF)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

	value->head = value->tail = NULL;
	while (result == OM_SUCCESS) {

		if (contLen == 0)
			break;


		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the SEQUENCE subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS) {
			if ((item = ca_ber_newItem(Attribute_ASN1Module_AttributeValueChangeDefinitionNode)) == NULL)
				result = OM_MEMORY_INSUFFICIENT;
			else {
				item->prev = value->tail;
				item->next = NULL;
				if (value->head == NULL)
					value->head = item;
				else
					value->tail->next = item;
				value->tail = item;
			}
		}

		if (result == OM_SUCCESS) {
			result = dec_Attribute_ASN1Module_AttributeValueChangeDefinition_SetOf_SeqT_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(item->elt),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfT =============== */

OM_return_code dec_Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfT_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfT *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfTNode *item = NULL;


	*decLen = 0;

	/* Check leading tag for SET_OF */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SET_OF)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

	value->head = value->tail = NULL;
	while (result == OM_SUCCESS) {

		if (contLen == 0)
			break;


		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the Attribute_ASN1Module_NotificationIdentifier subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS) {
			if ((item = ca_ber_newItem(Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfTNode)) == NULL)
				result = OM_MEMORY_INSUFFICIENT;
			else {
				item->prev = value->tail;
				item->next = NULL;
				if (value->head == NULL)
					value->head = item;
				else
					value->tail->next = item;
				value->tail = item;
			}
		}

		if (result == OM_SUCCESS) {
			result = dec_Attribute_ASN1Module_NotificationIdentifier_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(item->elt),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type InformationFramework_AttributeValueAssertion =============== */

OM_return_code dec_InformationFramework_AttributeValueAssertion_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			InformationFramework_AttributeValueAssertion *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;

	*decLen = 0;

	/* Check leading tag for SEQUENCE */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SEQUENCE)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

/*
** Member unnamedfield1
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the InformationFramework_AttributeType subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = dec_InformationFramework_AttributeType_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->unnamedfield1),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

/*
** Member unnamedfield2
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the InformationFramework_AttributeValue subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = dec_InformationFramework_AttributeValue_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->unnamedfield2),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type InformationFramework_RelativeDistinguishedName =============== */

OM_return_code dec_InformationFramework_RelativeDistinguishedName_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			InformationFramework_RelativeDistinguishedName *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	InformationFramework_RelativeDistinguishedNameNode *item = NULL;


	*decLen = 0;

	/* Check leading tag for SET_OF */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SET_OF)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

	value->head = value->tail = NULL;
	while (result == OM_SUCCESS) {

		if (contLen == 0)
			break;


		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the InformationFramework_AttributeValueAssertion subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS) {
			if ((item = ca_ber_newItem(InformationFramework_RelativeDistinguishedNameNode)) == NULL)
				result = OM_MEMORY_INSUFFICIENT;
			else {
				item->prev = value->tail;
				item->next = NULL;
				if (value->head == NULL)
					value->head = item;
				else
					value->tail->next = item;
				value->tail = item;
			}
		}

		if (result == OM_SUCCESS) {
			result = dec_InformationFramework_AttributeValueAssertion_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(item->elt),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type InformationFramework_RDNSequence =============== */

OM_return_code dec_InformationFramework_RDNSequence_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			InformationFramework_RDNSequence *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	InformationFramework_RDNSequenceNode *item = NULL;


	*decLen = 0;

	/* Check leading tag for SEQUENCE_OF */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SEQUENCE_OF)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SEQUENCE_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

	value->head = value->tail = NULL;
	while (result == OM_SUCCESS) {

		if (contLen == 0)
			break;


		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the InformationFramework_RelativeDistinguishedName subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS) {
			if ((item = ca_ber_newItem(InformationFramework_RDNSequenceNode)) == NULL)
				result = OM_MEMORY_INSUFFICIENT;
			else {
				item->prev = value->tail;
				item->next = NULL;
				if (value->head == NULL)
					value->head = item;
				else
					value->tail->next = item;
				value->tail = item;
			}
		}

		if (result == OM_SUCCESS) {
			result = dec_InformationFramework_RelativeDistinguishedName_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(item->elt),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type CMIP_1_ObjectInstance =============== */

OM_return_code dec_CMIP_1_ObjectInstance_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			CMIP_1_ObjectInstance *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;

	*decLen = 0;

	if (result == OM_SUCCESS) {
		subContLen = contLen;
	}

	/* Try to detect presence of attribute distinguishedName in CMIP_1_ObjectInstance */

	if (result == OM_SUCCESS) {
		result = OM_SUCCESS;

		/* Check leading tag [CTXT 2] */
		if (tagIsExplicit) {
			if (tagValue != 2
			    || tagClass != TAG_CTXT_CLASS)
			  result = OM_TEMPORARY_ERROR;
		}

		if (result == OM_SUCCESS) {
			result = dec_CMIP_1_DistinguishedName_content(contBuffer,
							subContLen,
							0 /* implicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->value.distinguishedName),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS)
			value->choice = CMIP_1_ObjectInstance_distinguishedName;
	}

	/* Try to detect presence of attribute nonSpecificForm in CMIP_1_ObjectInstance */

	if (result == OM_TEMPORARY_ERROR) {
		result = OM_SUCCESS;

		/* Check leading tag [CTXT 3] */
		if (tagIsExplicit) {
			if (tagValue != 3
			    || tagClass != TAG_CTXT_CLASS)
			  result = OM_TEMPORARY_ERROR;
		}

		if (result == OM_SUCCESS) {
			result = ca_ber_dec_OctetString_content(contBuffer,
							subContLen,
							0 /* implicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->value.nonSpecificForm),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS)
			value->choice = CMIP_1_ObjectInstance_nonSpecificForm;
	}

	/* Try to detect presence of attribute localDistinguishedName in CMIP_1_ObjectInstance */

	if (result == OM_TEMPORARY_ERROR) {
		result = OM_SUCCESS;

		/* Check leading tag [CTXT 4] */
		if (tagIsExplicit) {
			if (tagValue != 4
			    || tagClass != TAG_CTXT_CLASS)
			  result = OM_TEMPORARY_ERROR;
		}

		if (result == OM_SUCCESS) {
			result = dec_CMIP_1_RDNSequence_content(contBuffer,
							subContLen,
							0 /* implicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->value.localDistinguishedName),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS)
			value->choice = CMIP_1_ObjectInstance_localDistinguishedName;
	}
	return result;
}




/* =============== Type Attribute_ASN1Module_CorrelatedNotifications_SetOf_SeqT =============== */

OM_return_code dec_Attribute_ASN1Module_CorrelatedNotifications_SetOf_SeqT_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_CorrelatedNotifications_SetOf_SeqT *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	int decodeNextTag;

	*decLen = 0;

	/* Check leading tag for SEQUENCE */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SEQUENCE)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

/*
** Member correlatedNotifications
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the SET_OF subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = dec_Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfT_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->correlatedNotifications),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

/*
** Member sourceObjectInst
*/


	if (result == OM_SUCCESS) {

		/* Test if optional attribute sourceObjectInst is present (end of encoding reached) */
		if (contLen == 0) {
			value->sourceObjectInstIsPresent = OM_FALSE;
		}
		else {

			if (result == OM_SUCCESS) {
				subContLen = contLen;
			}

			/* Decode the leading tag of the Attribute_ASN1Module_ObjectInstance subtype */
			if (result == OM_SUCCESS) {
				result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
				*decLen += tagLen;

				if (result == OM_SUCCESS) {
					contLen -= tagLen;
					subContLen -= tagLen;
					if (contLen < 0)
						result = OM_ENCODING_INVALID;
				}
			}


			if (result == OM_SUCCESS) {
				result = dec_Attribute_ASN1Module_ObjectInstance_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								&(value->sourceObjectInst),
								&subDecLen);
				*decLen += subDecLen;

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Test if optional attribute sourceObjectInst is present (from tag presence) */
			if (result == OM_SUCCESS) {
				decodeNextTag = OM_TRUE;
				value->sourceObjectInstIsPresent = OM_TRUE;
			}
			else if (result == OM_TEMPORARY_ERROR) {
				decodeNextTag = OM_FALSE;
				value->sourceObjectInstIsPresent = OM_FALSE;
				result = OM_SUCCESS;
			}

		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type Attribute_ASN1Module_CorrelatedNotifications =============== */

OM_return_code dec_Attribute_ASN1Module_CorrelatedNotifications_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_CorrelatedNotifications *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	Attribute_ASN1Module_CorrelatedNotificationsNode *item = NULL;


	*decLen = 0;

	/* Check leading tag for SET_OF */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SET_OF)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

	value->head = value->tail = NULL;
	while (result == OM_SUCCESS) {

		if (contLen == 0)
			break;


		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the SEQUENCE subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS) {
			if ((item = ca_ber_newItem(Attribute_ASN1Module_CorrelatedNotificationsNode)) == NULL)
				result = OM_MEMORY_INSUFFICIENT;
			else {
				item->prev = value->tail;
				item->next = NULL;
				if (value->head == NULL)
					value->head = item;
				else
					value->tail->next = item;
				value->tail = item;
			}
		}

		if (result == OM_SUCCESS) {
			result = dec_Attribute_ASN1Module_CorrelatedNotifications_SetOf_SeqT_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(item->elt),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type GraphicString =============== */

OM_return_code dec_GraphicString_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			GraphicString *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;

	*decLen = 0;

	/* Check leading tag [UNIV 25] */
	if (tagIsExplicit) {
		if (tagValue != 25
		    || tagClass != TAG_UNIV_CLASS)
		  result = OM_TEMPORARY_ERROR;
	}

	if (result == OM_SUCCESS) {
		subContLen = contLen;
	}

	if (result == OM_SUCCESS) {
		result = ca_ber_dec_OctetString_content(contBuffer,
						subContLen,
						0 /* implicit form */,
						tagValue,
						tagClass,
						tagForm,
						value,
						&subDecLen);
		*decLen += subDecLen;

		if (result == OM_SUCCESS) {
			contLen -= subDecLen;
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}
	return result;
}




/* =============== Type Attribute_ASN1Module_ManagementExtension =============== */

OM_return_code dec_Attribute_ASN1Module_ManagementExtension_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_ManagementExtension *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;

	*decLen = 0;

	/* Check leading tag for SEQUENCE */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SEQUENCE)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

/*
** Member identifier
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the OBJECT_IDENTIFIER subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = ca_ber_dec_ObjectId_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->identifier),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

/*
** Member significance
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Extract tag [CTXT 1] */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		/* Check tag [CTXT 1] */
		if (result == OM_SUCCESS) {
			if (tagValue != 1
				|| tagClass != TAG_CTXT_CLASS)
			  	result = OM_ENCODING_INVALID;
		}

		if (result == OM_SUCCESS) {
			result = ca_ber_dec_Boolean_content(contBuffer,
							subContLen,
							0 /* implicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->significance),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

/*
** Member information
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Extract tag [CTXT 2] */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		/* Check tag [CTXT 2] */
		if (result == OM_SUCCESS) {
			if (tagValue != 2
				|| tagClass != TAG_CTXT_CLASS
				|| tagForm != TAG_CONS_FORM)
			  	result = OM_ENCODING_INVALID;
		}

		/* Decode the length of the  contents */
		if (result == OM_SUCCESS) {
			result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
			*decLen += lenLen;
		}

		if (result == OM_SUCCESS) {
			contLen -= lenLen;
			if (contLen < 0)
				result = OM_ENCODING_INVALID;
		}

		if (result == OM_SUCCESS) {
			if (subContLen > contLen)
				result = OM_ENCODING_INVALID;
		}


		/* Decode the leading tag of the  subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = ca_ber_dec_Any_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->information),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type Attribute_ASN1Module_AdditionalInformation =============== */

OM_return_code dec_Attribute_ASN1Module_AdditionalInformation_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_AdditionalInformation *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	Attribute_ASN1Module_AdditionalInformationNode *item = NULL;


	*decLen = 0;

	/* Check leading tag for SET_OF */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SET_OF)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

	value->head = value->tail = NULL;
	while (result == OM_SUCCESS) {

		if (contLen == 0)
			break;


		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the Attribute_ASN1Module_ManagementExtension subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS) {
			if ((item = ca_ber_newItem(Attribute_ASN1Module_AdditionalInformationNode)) == NULL)
				result = OM_MEMORY_INSUFFICIENT;
			else {
				item->prev = value->tail;
				item->next = NULL;
				if (value->head == NULL)
					value->head = item;
				else
					value->tail->next = item;
				value->tail = item;
			}
		}

		if (result == OM_SUCCESS) {
			result = dec_Attribute_ASN1Module_ManagementExtension_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(item->elt),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type Notification_ASN1Module_AttributeValueChangeInfo =============== */

OM_return_code dec_Notification_ASN1Module_AttributeValueChangeInfo_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Notification_ASN1Module_AttributeValueChangeInfo *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	int decodeNextTag;

	*decLen = 0;

	/* Check leading tag for SEQUENCE */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SEQUENCE)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

/*
** Member sourceIndicator
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the Notification_ASN1Module_SourceIndicator subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = dec_Notification_ASN1Module_SourceIndicator_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->sourceIndicator),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		/* Test if optional attribute sourceIndicator is present (from tag presence) */
		if (result == OM_SUCCESS) {
			decodeNextTag = OM_TRUE;
			value->sourceIndicatorIsPresent = OM_TRUE;
		}
		else if (result == OM_TEMPORARY_ERROR) {
			decodeNextTag = OM_FALSE;
			value->sourceIndicatorIsPresent = OM_FALSE;
			result = OM_SUCCESS;
		}

	}

/*
** Member attributeIdentifierList
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Extract tag [CTXT 1] */
		if (decodeNextTag) {
			if (result == OM_SUCCESS) {
				result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
				*decLen += tagLen;

				if (result == OM_SUCCESS) {
					contLen -= tagLen;
					subContLen -= tagLen;
					if (contLen < 0)
						result = OM_ENCODING_INVALID;
				}
			}
		}


		/* Check tag [CTXT 1] */
		if (result == OM_SUCCESS) {
			if (tagValue != 1
				|| tagClass != TAG_CTXT_CLASS)
			  	result = OM_TEMPORARY_ERROR;
		}

		if (result == OM_SUCCESS) {
			result = dec_Notification_ASN1Module_AttributeIdentifierList_content(contBuffer,
							subContLen,
							0 /* implicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->attributeIdentifierList),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		/* Test if optional attribute attributeIdentifierList is present (from tag presence) */
		if (result == OM_SUCCESS) {
			decodeNextTag = OM_TRUE;
			value->attributeIdentifierListIsPresent = OM_TRUE;
		}
		else if (result == OM_TEMPORARY_ERROR) {
			decodeNextTag = OM_FALSE;
			value->attributeIdentifierListIsPresent = OM_FALSE;
			result = OM_SUCCESS;
		}

	}

/*
** Member attributeValueChangeDefinition
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the Notification_ASN1Module_AttributeValueChangeDefinition subtype */
		if (decodeNextTag) {
			if (result == OM_SUCCESS) {
				result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
				*decLen += tagLen;

				if (result == OM_SUCCESS) {
					contLen -= tagLen;
					subContLen -= tagLen;
					if (contLen < 0)
						result = OM_ENCODING_INVALID;
				}
			}
		}


		if (result == OM_SUCCESS) {
			result = dec_Notification_ASN1Module_AttributeValueChangeDefinition_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->attributeValueChangeDefinition),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

/*
** Member notificationIdentifier
*/


	if (result == OM_SUCCESS) {

		/* Test if optional attribute notificationIdentifier is present (end of encoding reached) */
		if (contLen == 0) {
			value->notificationIdentifierIsPresent = OM_FALSE;
		}
		else {

			if (result == OM_SUCCESS) {
				subContLen = contLen;
			}

			/* Decode the leading tag of the Notification_ASN1Module_NotificationIdentifier subtype */
			if (result == OM_SUCCESS) {
				result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
				*decLen += tagLen;

				if (result == OM_SUCCESS) {
					contLen -= tagLen;
					subContLen -= tagLen;
					if (contLen < 0)
						result = OM_ENCODING_INVALID;
				}
			}


			if (result == OM_SUCCESS) {
				result = dec_Notification_ASN1Module_NotificationIdentifier_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								&(value->notificationIdentifier),
								&subDecLen);
				*decLen += subDecLen;

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Test if optional attribute notificationIdentifier is present (from tag presence) */
			if (result == OM_SUCCESS) {
				decodeNextTag = OM_TRUE;
				value->notificationIdentifierIsPresent = OM_TRUE;
			}
			else if (result == OM_TEMPORARY_ERROR) {
				decodeNextTag = OM_FALSE;
				value->notificationIdentifierIsPresent = OM_FALSE;
				result = OM_SUCCESS;
			}

		}
	}

/*
** Member correlatedNotifications
*/


	if (result == OM_SUCCESS) {

		/* Test if optional attribute correlatedNotifications is present (end of encoding reached) */
		if (contLen == 0) {
			value->correlatedNotificationsIsPresent = OM_FALSE;
		}
		else {

			if (result == OM_SUCCESS) {
				subContLen = contLen;
			}

			/* Extract tag [CTXT 2] */
			if (decodeNextTag) {
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
					*decLen += tagLen;

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}
			}


			/* Check tag [CTXT 2] */
			if (result == OM_SUCCESS) {
				if (tagValue != 2
					|| tagClass != TAG_CTXT_CLASS)
				  	result = OM_TEMPORARY_ERROR;
			}

			if (result == OM_SUCCESS) {
				result = dec_Notification_ASN1Module_CorrelatedNotifications_content(contBuffer,
								subContLen,
								0 /* implicit form */,
								tagValue,
								tagClass,
								tagForm,
								&(value->correlatedNotifications),
								&subDecLen);
				*decLen += subDecLen;

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Test if optional attribute correlatedNotifications is present (from tag presence) */
			if (result == OM_SUCCESS) {
				decodeNextTag = OM_TRUE;
				value->correlatedNotificationsIsPresent = OM_TRUE;
			}
			else if (result == OM_TEMPORARY_ERROR) {
				decodeNextTag = OM_FALSE;
				value->correlatedNotificationsIsPresent = OM_FALSE;
				result = OM_SUCCESS;
			}

		}
	}

/*
** Member additionalText
*/


	if (result == OM_SUCCESS) {

		/* Test if optional attribute additionalText is present (end of encoding reached) */
		if (contLen == 0) {
			value->additionalTextIsPresent = OM_FALSE;
		}
		else {

			if (result == OM_SUCCESS) {
				subContLen = contLen;
			}

			/* Decode the leading tag of the Notification_ASN1Module_AdditionalText subtype */
			if (decodeNextTag) {
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
					*decLen += tagLen;

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}
			}


			if (result == OM_SUCCESS) {
				result = dec_Notification_ASN1Module_AdditionalText_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								&(value->additionalText),
								&subDecLen);
				*decLen += subDecLen;

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Test if optional attribute additionalText is present (from tag presence) */
			if (result == OM_SUCCESS) {
				decodeNextTag = OM_TRUE;
				value->additionalTextIsPresent = OM_TRUE;
			}
			else if (result == OM_TEMPORARY_ERROR) {
				decodeNextTag = OM_FALSE;
				value->additionalTextIsPresent = OM_FALSE;
				result = OM_SUCCESS;
			}

		}
	}

/*
** Member additionalInformation
*/


	if (result == OM_SUCCESS) {

		/* Test if optional attribute additionalInformation is present (end of encoding reached) */
		if (contLen == 0) {
			value->additionalInformationIsPresent = OM_FALSE;
		}
		else {

			if (result == OM_SUCCESS) {
				subContLen = contLen;
			}

			/* Extract tag [CTXT 3] */
			if (decodeNextTag) {
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
					*decLen += tagLen;

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}
			}


			/* Check tag [CTXT 3] */
			if (result == OM_SUCCESS) {
				if (tagValue != 3
					|| tagClass != TAG_CTXT_CLASS)
				  	result = OM_TEMPORARY_ERROR;
			}

			if (result == OM_SUCCESS) {
				result = dec_Notification_ASN1Module_AdditionalInformation_content(contBuffer,
								subContLen,
								0 /* implicit form */,
								tagValue,
								tagClass,
								tagForm,
								&(value->additionalInformation),
								&subDecLen);
				*decLen += subDecLen;

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Test if optional attribute additionalInformation is present (from tag presence) */
			if (result == OM_SUCCESS) {
				decodeNextTag = OM_TRUE;
				value->additionalInformationIsPresent = OM_TRUE;
			}
			else if (result == OM_TEMPORARY_ERROR) {
				decodeNextTag = OM_FALSE;
				value->additionalInformationIsPresent = OM_FALSE;
				result = OM_SUCCESS;
			}

		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type CFSObject_ASN1module_PermissionBits =============== */

OM_return_code dec_CFSObject_ASN1module_PermissionBits_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			CFSObject_ASN1module_PermissionBits *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;

	*decLen = 0;

	/* Check leading tag for SEQUENCE */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SEQUENCE)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

/*
** Member read
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the BOOLEAN subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = ca_ber_dec_Boolean_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->read),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

/*
** Member write
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the BOOLEAN subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = ca_ber_dec_Boolean_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->write),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

/*
** Member execute
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the BOOLEAN subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = ca_ber_dec_Boolean_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->execute),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type CFSObject_ASN1module_FilePermissions =============== */

OM_return_code dec_CFSObject_ASN1module_FilePermissions_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			CFSObject_ASN1module_FilePermissions *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;

	*decLen = 0;

	/* Check leading tag for SEQUENCE */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SEQUENCE)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

/*
** Member user
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the CFSObject_ASN1module_PermissionBits subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = dec_CFSObject_ASN1module_PermissionBits_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->user),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

/*
** Member group
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the CFSObject_ASN1module_PermissionBits subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = dec_CFSObject_ASN1module_PermissionBits_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->group),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

/*
** Member other
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the CFSObject_ASN1module_PermissionBits subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = dec_CFSObject_ASN1module_PermissionBits_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->other),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type CMIP_1_Attribute =============== */

OM_return_code dec_CMIP_1_Attribute_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			CMIP_1_Attribute *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;

	*decLen = 0;

	/* Check leading tag for SEQUENCE */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SEQUENCE)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

/*
** Member attributeId
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the CMIP_1_AttributeId subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = dec_CMIP_1_AttributeId_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->attributeId),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

/*
** Member attributeValue
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the  subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = ca_ber_dec_Any_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->attributeValue),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type Attribute_ASN1Module_AttributeList =============== */

OM_return_code dec_Attribute_ASN1Module_AttributeList_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_AttributeList *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	Attribute_ASN1Module_AttributeListNode *item = NULL;


	*decLen = 0;

	/* Check leading tag for SET_OF */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SET_OF)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

	value->head = value->tail = NULL;
	while (result == OM_SUCCESS) {

		if (contLen == 0)
			break;


		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the Attribute_ASN1Module_Attribute subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS) {
			if ((item = ca_ber_newItem(Attribute_ASN1Module_AttributeListNode)) == NULL)
				result = OM_MEMORY_INSUFFICIENT;
			else {
				item->prev = value->tail;
				item->next = NULL;
				if (value->head == NULL)
					value->head = item;
				else
					value->tail->next = item;
				value->tail = item;
			}
		}

		if (result == OM_SUCCESS) {
			result = dec_Attribute_ASN1Module_Attribute_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(item->elt),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type Notification_ASN1Module_ObjectInfo =============== */

OM_return_code dec_Notification_ASN1Module_ObjectInfo_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Notification_ASN1Module_ObjectInfo *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	int decodeNextTag;

	*decLen = 0;

	/* Check leading tag for SEQUENCE */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SEQUENCE)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

/*
** Member sourceIndicator
*/


	if (result == OM_SUCCESS) {

		/* Test if optional attribute sourceIndicator is present (end of encoding reached) */
		if (contLen == 0) {
			value->sourceIndicatorIsPresent = OM_FALSE;
		}
		else {

			if (result == OM_SUCCESS) {
				subContLen = contLen;
			}

			/* Decode the leading tag of the Notification_ASN1Module_SourceIndicator subtype */
			if (result == OM_SUCCESS) {
				result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
				*decLen += tagLen;

				if (result == OM_SUCCESS) {
					contLen -= tagLen;
					subContLen -= tagLen;
					if (contLen < 0)
						result = OM_ENCODING_INVALID;
				}
			}


			if (result == OM_SUCCESS) {
				result = dec_Notification_ASN1Module_SourceIndicator_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								&(value->sourceIndicator),
								&subDecLen);
				*decLen += subDecLen;

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Test if optional attribute sourceIndicator is present (from tag presence) */
			if (result == OM_SUCCESS) {
				decodeNextTag = OM_TRUE;
				value->sourceIndicatorIsPresent = OM_TRUE;
			}
			else if (result == OM_TEMPORARY_ERROR) {
				decodeNextTag = OM_FALSE;
				value->sourceIndicatorIsPresent = OM_FALSE;
				result = OM_SUCCESS;
			}

		}
	}

/*
** Member attributeList
*/


	if (result == OM_SUCCESS) {

		/* Test if optional attribute attributeList is present (end of encoding reached) */
		if (contLen == 0) {
			value->attributeListIsPresent = OM_FALSE;
		}
		else {

			if (result == OM_SUCCESS) {
				subContLen = contLen;
			}

			/* Decode the leading tag of the Notification_ASN1Module_AttributeList subtype */
			if (decodeNextTag) {
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
					*decLen += tagLen;

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}
			}


			if (result == OM_SUCCESS) {
				result = dec_Notification_ASN1Module_AttributeList_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								&(value->attributeList),
								&subDecLen);
				*decLen += subDecLen;

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Test if optional attribute attributeList is present (from tag presence) */
			if (result == OM_SUCCESS) {
				decodeNextTag = OM_TRUE;
				value->attributeListIsPresent = OM_TRUE;
			}
			else if (result == OM_TEMPORARY_ERROR) {
				decodeNextTag = OM_FALSE;
				value->attributeListIsPresent = OM_FALSE;
				result = OM_SUCCESS;
			}

		}
	}

/*
** Member notificationIdentifier
*/


	if (result == OM_SUCCESS) {

		/* Test if optional attribute notificationIdentifier is present (end of encoding reached) */
		if (contLen == 0) {
			value->notificationIdentifierIsPresent = OM_FALSE;
		}
		else {

			if (result == OM_SUCCESS) {
				subContLen = contLen;
			}

			/* Decode the leading tag of the Notification_ASN1Module_NotificationIdentifier subtype */
			if (decodeNextTag) {
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
					*decLen += tagLen;

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}
			}


			if (result == OM_SUCCESS) {
				result = dec_Notification_ASN1Module_NotificationIdentifier_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								&(value->notificationIdentifier),
								&subDecLen);
				*decLen += subDecLen;

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Test if optional attribute notificationIdentifier is present (from tag presence) */
			if (result == OM_SUCCESS) {
				decodeNextTag = OM_TRUE;
				value->notificationIdentifierIsPresent = OM_TRUE;
			}
			else if (result == OM_TEMPORARY_ERROR) {
				decodeNextTag = OM_FALSE;
				value->notificationIdentifierIsPresent = OM_FALSE;
				result = OM_SUCCESS;
			}

		}
	}

/*
** Member correlatedNotifications
*/


	if (result == OM_SUCCESS) {

		/* Test if optional attribute correlatedNotifications is present (end of encoding reached) */
		if (contLen == 0) {
			value->correlatedNotificationsIsPresent = OM_FALSE;
		}
		else {

			if (result == OM_SUCCESS) {
				subContLen = contLen;
			}

			/* Extract tag [CTXT 1] */
			if (decodeNextTag) {
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
					*decLen += tagLen;

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}
			}


			/* Check tag [CTXT 1] */
			if (result == OM_SUCCESS) {
				if (tagValue != 1
					|| tagClass != TAG_CTXT_CLASS)
				  	result = OM_TEMPORARY_ERROR;
			}

			if (result == OM_SUCCESS) {
				result = dec_Notification_ASN1Module_CorrelatedNotifications_content(contBuffer,
								subContLen,
								0 /* implicit form */,
								tagValue,
								tagClass,
								tagForm,
								&(value->correlatedNotifications),
								&subDecLen);
				*decLen += subDecLen;

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Test if optional attribute correlatedNotifications is present (from tag presence) */
			if (result == OM_SUCCESS) {
				decodeNextTag = OM_TRUE;
				value->correlatedNotificationsIsPresent = OM_TRUE;
			}
			else if (result == OM_TEMPORARY_ERROR) {
				decodeNextTag = OM_FALSE;
				value->correlatedNotificationsIsPresent = OM_FALSE;
				result = OM_SUCCESS;
			}

		}
	}

/*
** Member additionalText
*/


	if (result == OM_SUCCESS) {

		/* Test if optional attribute additionalText is present (end of encoding reached) */
		if (contLen == 0) {
			value->additionalTextIsPresent = OM_FALSE;
		}
		else {

			if (result == OM_SUCCESS) {
				subContLen = contLen;
			}

			/* Decode the leading tag of the Notification_ASN1Module_AdditionalText subtype */
			if (decodeNextTag) {
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
					*decLen += tagLen;

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}
			}


			if (result == OM_SUCCESS) {
				result = dec_Notification_ASN1Module_AdditionalText_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								&(value->additionalText),
								&subDecLen);
				*decLen += subDecLen;

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Test if optional attribute additionalText is present (from tag presence) */
			if (result == OM_SUCCESS) {
				decodeNextTag = OM_TRUE;
				value->additionalTextIsPresent = OM_TRUE;
			}
			else if (result == OM_TEMPORARY_ERROR) {
				decodeNextTag = OM_FALSE;
				value->additionalTextIsPresent = OM_FALSE;
				result = OM_SUCCESS;
			}

		}
	}

/*
** Member additionalInformation
*/


	if (result == OM_SUCCESS) {

		/* Test if optional attribute additionalInformation is present (end of encoding reached) */
		if (contLen == 0) {
			value->additionalInformationIsPresent = OM_FALSE;
		}
		else {

			if (result == OM_SUCCESS) {
				subContLen = contLen;
			}

			/* Extract tag [CTXT 2] */
			if (decodeNextTag) {
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
					*decLen += tagLen;

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}
			}


			/* Check tag [CTXT 2] */
			if (result == OM_SUCCESS) {
				if (tagValue != 2
					|| tagClass != TAG_CTXT_CLASS)
				  	result = OM_TEMPORARY_ERROR;
			}

			if (result == OM_SUCCESS) {
				result = dec_Notification_ASN1Module_AdditionalInformation_content(contBuffer,
								subContLen,
								0 /* implicit form */,
								tagValue,
								tagClass,
								tagForm,
								&(value->additionalInformation),
								&subDecLen);
				*decLen += subDecLen;

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Test if optional attribute additionalInformation is present (from tag presence) */
			if (result == OM_SUCCESS) {
				decodeNextTag = OM_TRUE;
				value->additionalInformationIsPresent = OM_TRUE;
			}
			else if (result == OM_TEMPORARY_ERROR) {
				decodeNextTag = OM_FALSE;
				value->additionalInformationIsPresent = OM_FALSE;
				result = OM_SUCCESS;
			}

		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type Attribute_ASN1Module_SystemId =============== */

OM_return_code dec_Attribute_ASN1Module_SystemId_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_SystemId *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;

	*decLen = 0;

	if (result == OM_SUCCESS) {
		subContLen = contLen;
	}

	/* Try to detect presence of attribute name in Attribute_ASN1Module_SystemId */

	if (result == OM_SUCCESS) {
		result = dec_GraphicString_content(contBuffer,
						subContLen,
						1 /* explicit form */,
						tagValue,
						tagClass,
						tagForm,
						&(value->value.name),
						&subDecLen);
		*decLen += subDecLen;

		if (result == OM_SUCCESS) {
			contLen -= subDecLen;
		}

		if (result == OM_SUCCESS)
			value->choice = Attribute_ASN1Module_SystemId_name;
	}

	/* Try to detect presence of attribute number in Attribute_ASN1Module_SystemId */

	if (result == OM_TEMPORARY_ERROR) {
		result = ca_ber_dec_Integer_content(contBuffer,
						subContLen,
						1 /* explicit form */,
						tagValue,
						tagClass,
						tagForm,
						&(value->value.number),
						&subDecLen);
		*decLen += subDecLen;

		if (result == OM_SUCCESS) {
			contLen -= subDecLen;
		}

		if (result == OM_SUCCESS)
			value->choice = Attribute_ASN1Module_SystemId_number;
	}

	/* Try to detect presence of attribute nothing in Attribute_ASN1Module_SystemId */

	if (result == OM_TEMPORARY_ERROR) {
		result = ca_ber_dec_Null_content(contBuffer,
						subContLen,
						1 /* explicit form */,
						tagValue,
						tagClass,
						tagForm,
						&subDecLen);
		*decLen += subDecLen;

		if (result == OM_SUCCESS) {
			contLen -= subDecLen;
		}

		if (result == OM_SUCCESS)
			value->choice = Attribute_ASN1Module_SystemId_nothing;
	}
	return result;
}




/* =============== Type Attribute_ASN1Module_SystemTitle =============== */

OM_return_code dec_Attribute_ASN1Module_SystemTitle_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_SystemTitle *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;

	*decLen = 0;

	if (result == OM_SUCCESS) {
		subContLen = contLen;
	}

	/* Try to detect presence of attribute distinguishedName in Attribute_ASN1Module_SystemTitle */

	if (result == OM_SUCCESS) {
		result = dec_Attribute_ASN1Module_DistinguishedName_content(contBuffer,
						subContLen,
						1 /* explicit form */,
						tagValue,
						tagClass,
						tagForm,
						&(value->value.distinguishedName),
						&subDecLen);
		*decLen += subDecLen;

		if (result == OM_SUCCESS) {
			contLen -= subDecLen;
		}

		if (result == OM_SUCCESS)
			value->choice = Attribute_ASN1Module_SystemTitle_distinguishedName;
	}

	/* Try to detect presence of attribute oid in Attribute_ASN1Module_SystemTitle */

	if (result == OM_TEMPORARY_ERROR) {
		result = ca_ber_dec_ObjectId_content(contBuffer,
						subContLen,
						1 /* explicit form */,
						tagValue,
						tagClass,
						tagForm,
						&(value->value.oid),
						&subDecLen);
		*decLen += subDecLen;

		if (result == OM_SUCCESS) {
			contLen -= subDecLen;
		}

		if (result == OM_SUCCESS)
			value->choice = Attribute_ASN1Module_SystemTitle_oid;
	}

	/* Try to detect presence of attribute nothing in Attribute_ASN1Module_SystemTitle */

	if (result == OM_TEMPORARY_ERROR) {
		result = ca_ber_dec_Null_content(contBuffer,
						subContLen,
						1 /* explicit form */,
						tagValue,
						tagClass,
						tagForm,
						&subDecLen);
		*decLen += subDecLen;

		if (result == OM_SUCCESS) {
			contLen -= subDecLen;
		}

		if (result == OM_SUCCESS)
			value->choice = Attribute_ASN1Module_SystemTitle_nothing;
	}
	return result;
}




/* =============== Type Attribute_ASN1Module_SupportedFeatures_SeqT =============== */

OM_return_code dec_Attribute_ASN1Module_SupportedFeatures_SeqT_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_SupportedFeatures_SeqT *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;

	*decLen = 0;

	/* Check leading tag for SEQUENCE */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SEQUENCE)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SEQUENCE contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

/*
** Member featureIdentifier
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the OBJECT_IDENTIFIER subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = ca_ber_dec_ObjectId_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->featureIdentifier),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

/*
** Member featureInfo
*/


	if (result == OM_SUCCESS) {

		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the  subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}


		if (result == OM_SUCCESS) {
			result = ca_ber_dec_Any_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(value->featureInfo),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}
		}

		if (result == OM_TEMPORARY_ERROR) {
			result = OM_ENCODING_INVALID;
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type Attribute_ASN1Module_SupportedFeatures =============== */

OM_return_code dec_Attribute_ASN1Module_SupportedFeatures_content(ca_buff_t * contBuffer, int contLen,
			unsigned char tagIsExplicit,
			int tagValue, int tagClass, int tagForm,
			Attribute_ASN1Module_SupportedFeatures *value, int *decLen)
{
	OM_return_code result = OM_SUCCESS;
	int subContLen;
	int lenLen;
	int subDecLen;
	int tagLen;
	Attribute_ASN1Module_SupportedFeaturesNode *item = NULL;


	*decLen = 0;

	/* Check leading tag for SET_OF */
	if (tagIsExplicit) {
		if ((tagValue | tagClass | tagForm) != TAG_SET_OF)
		    result = OM_TEMPORARY_ERROR;
	}
	else if (tagForm != TAG_CONS_FORM)
		  result = OM_ENCODING_INVALID;


	/* Decode the length of the SET_OF contents */
	if (result == OM_SUCCESS) {
		result = ca_ber_decLength(contBuffer, &subContLen, &lenLen);
		*decLen += lenLen;
	}

	if (result == OM_SUCCESS) {
		contLen -= lenLen;
		if (contLen < 0)
			result = OM_ENCODING_INVALID;
	}

	if (result == OM_SUCCESS) {
		if (subContLen > contLen)
			result = OM_ENCODING_INVALID;
	}


	if (result == OM_SUCCESS) {
		contLen = subContLen;

	}

	value->head = value->tail = NULL;
	while (result == OM_SUCCESS) {

		if (contLen == 0)
			break;


		if (result == OM_SUCCESS) {
			subContLen = contLen;
		}

		/* Decode the leading tag of the SEQUENCE subtype */
		if (result == OM_SUCCESS) {
			result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);
			*decLen += tagLen;

			if (result == OM_SUCCESS) {
				contLen -= tagLen;
				subContLen -= tagLen;
				if (contLen < 0)
					result = OM_ENCODING_INVALID;
			}
		}

		if (result == OM_SUCCESS) {
			if ((item = ca_ber_newItem(Attribute_ASN1Module_SupportedFeaturesNode)) == NULL)
				result = OM_MEMORY_INSUFFICIENT;
			else {
				item->prev = value->tail;
				item->next = NULL;
				if (value->head == NULL)
					value->head = item;
				else
					value->tail->next = item;
				value->tail = item;
			}
		}

		if (result == OM_SUCCESS) {
			result = dec_Attribute_ASN1Module_SupportedFeatures_SeqT_content(contBuffer,
							subContLen,
							1 /* explicit form */,
							tagValue,
							tagClass,
							tagForm,
							&(item->elt),
							&subDecLen);
			*decLen += subDecLen;

			if (result == OM_SUCCESS) {
				contLen -= subDecLen;
			}

			if (result == OM_TEMPORARY_ERROR) {
				result = OM_ENCODING_INVALID;
			}
		}
	}

	if (result == OM_SUCCESS) {
		if (contLen != 0)
			result = OM_ENCODING_INVALID;
	}

	return result;
}




/* =============== Type CFSObject_ASN1module_CfsId =============== */

OM_return_code dec_CFSObject_ASN1module_CfsId(OM_descriptor *encoding,
			CFSObject_ASN1module_CfsId *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_PRINTABLE_STRING : {
			return ca_xom_strcpy(value, &(encoding->value.string));
		}

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the CFSObject_ASN1module_CfsId subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_CFSObject_ASN1module_CfsId_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type CFSObject_ASN1module_CfsDirectory =============== */

OM_return_code dec_CFSObject_ASN1module_CfsDirectory(OM_descriptor *encoding,
			CFSObject_ASN1module_CfsDirectory *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_PRINTABLE_STRING : {
			return ca_xom_strcpy(value, &(encoding->value.string));
		}

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the CFSObject_ASN1module_CfsDirectory subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_CFSObject_ASN1module_CfsDirectory_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type CFSObject_ASN1module_AutoUpdate =============== */

OM_return_code dec_CFSObject_ASN1module_AutoUpdate(OM_descriptor *encoding,
			CFSObject_ASN1module_AutoUpdate *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_BOOLEAN : {
			*value = encoding->value.boolean;
			return OM_SUCCESS;
		}

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the CFSObject_ASN1module_AutoUpdate subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_CFSObject_ASN1module_AutoUpdate_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type CMIP_1_ObjectClass =============== */

OM_return_code dec_CMIP_1_ObjectClass(OM_descriptor *encoding,
			CMIP_1_ObjectClass *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the CMIP_1_ObjectClass subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_CMIP_1_ObjectClass_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type Attribute_ASN1Module_NameBinding =============== */

OM_return_code dec_Attribute_ASN1Module_NameBinding(OM_descriptor *encoding,
			Attribute_ASN1Module_NameBinding *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_OBJECT_IDENTIFIER_STRING : {
			return ca_xom_strcpy(value, &(encoding->value.string));
		}

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the Attribute_ASN1Module_NameBinding subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_Attribute_ASN1Module_NameBinding_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type Attribute_ASN1Module_Packages =============== */

OM_return_code dec_Attribute_ASN1Module_Packages(OM_descriptor *encoding,
			Attribute_ASN1Module_Packages *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the Attribute_ASN1Module_Packages subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_Attribute_ASN1Module_Packages_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type Attribute_ASN1Module_Allomorphs =============== */

OM_return_code dec_Attribute_ASN1Module_Allomorphs(OM_descriptor *encoding,
			Attribute_ASN1Module_Allomorphs *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the Attribute_ASN1Module_Allomorphs subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_Attribute_ASN1Module_Allomorphs_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type Notification_ASN1Module_AttributeValueChangeInfo =============== */

OM_return_code dec_Notification_ASN1Module_AttributeValueChangeInfo(OM_descriptor *encoding,
			Notification_ASN1Module_AttributeValueChangeInfo *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the Notification_ASN1Module_AttributeValueChangeInfo subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_Notification_ASN1Module_AttributeValueChangeInfo_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type CFSObject_ASN1module_FilePermissions =============== */

OM_return_code dec_CFSObject_ASN1module_FilePermissions(OM_descriptor *encoding,
			CFSObject_ASN1module_FilePermissions *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the CFSObject_ASN1module_FilePermissions subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_CFSObject_ASN1module_FilePermissions_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type Notification_ASN1Module_ObjectInfo =============== */

OM_return_code dec_Notification_ASN1Module_ObjectInfo(OM_descriptor *encoding,
			Notification_ASN1Module_ObjectInfo *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the Notification_ASN1Module_ObjectInfo subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_Notification_ASN1Module_ObjectInfo_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type Attribute_ASN1Module_SystemId =============== */

OM_return_code dec_Attribute_ASN1Module_SystemId(OM_descriptor *encoding,
			Attribute_ASN1Module_SystemId *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_GRAPHIC_STRING : {
			value->choice = Attribute_ASN1Module_SystemId_name;
			return ca_xom_strcpy(&(value->value.name), &(encoding->value.string));
		}

		case OM_S_INTEGER : {
			value->choice = Attribute_ASN1Module_SystemId_number;
			value->value.number = encoding->value.integer;
			return OM_SUCCESS;
		}

		case OM_S_NULL : {
			value->choice = Attribute_ASN1Module_SystemId_nothing;
					return OM_SUCCESS;
		}

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the Attribute_ASN1Module_SystemId subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_Attribute_ASN1Module_SystemId_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type Attribute_ASN1Module_SystemTitle =============== */

OM_return_code dec_Attribute_ASN1Module_SystemTitle(OM_descriptor *encoding,
			Attribute_ASN1Module_SystemTitle *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_OBJECT_IDENTIFIER_STRING : {
			value->choice = Attribute_ASN1Module_SystemTitle_oid;
			return ca_xom_strcpy(&(value->value.oid), &(encoding->value.string));
		}

		case OM_S_NULL : {
			value->choice = Attribute_ASN1Module_SystemTitle_nothing;
					return OM_SUCCESS;
		}

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the Attribute_ASN1Module_SystemTitle subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_Attribute_ASN1Module_SystemTitle_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type Attribute_ASN1Module_OperationalState =============== */

OM_return_code dec_Attribute_ASN1Module_OperationalState(OM_descriptor *encoding,
			Attribute_ASN1Module_OperationalState *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_ENUMERATION : {
			*value = encoding->value.enumeration;
			return OM_SUCCESS;
		}

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the Attribute_ASN1Module_OperationalState subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_Attribute_ASN1Module_OperationalState_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type Attribute_ASN1Module_UsageState =============== */

OM_return_code dec_Attribute_ASN1Module_UsageState(OM_descriptor *encoding,
			Attribute_ASN1Module_UsageState *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_ENUMERATION : {
			*value = encoding->value.enumeration;
			return OM_SUCCESS;
		}

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the Attribute_ASN1Module_UsageState subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_Attribute_ASN1Module_UsageState_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type Attribute_ASN1Module_AdministrativeState =============== */

OM_return_code dec_Attribute_ASN1Module_AdministrativeState(OM_descriptor *encoding,
			Attribute_ASN1Module_AdministrativeState *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_ENUMERATION : {
			*value = encoding->value.enumeration;
			return OM_SUCCESS;
		}

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the Attribute_ASN1Module_AdministrativeState subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_Attribute_ASN1Module_AdministrativeState_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type Attribute_ASN1Module_SupportedFeatures =============== */

OM_return_code dec_Attribute_ASN1Module_SupportedFeatures(OM_descriptor *encoding,
			Attribute_ASN1Module_SupportedFeatures *value)
{
	switch(encoding->syntax & OM_S_SYNTAX) {

		case OM_S_ENCODING_STRING : {
			OM_return_code result = OM_SUCCESS;
			ca_buff_t cb, *contBuffer = &cb;
			int subDecLen;
			int contLen;
			int subContLen = -1;
			int tagValue, tagForm, tagClass, tagLen;


			/* Initialize content buffer */
			contLen = ca_buff_fom(contBuffer, encoding);

			if (result == OM_SUCCESS) {

				/* Decode the leading tag of the Attribute_ASN1Module_SupportedFeatures subtype */
				if (result == OM_SUCCESS) {
					result = ca_ber_decTag(contBuffer, &tagValue, &tagForm, &tagClass, &tagLen);

					if (result == OM_SUCCESS) {
						contLen -= tagLen;
						subContLen -= tagLen;
						if (contLen < 0)
							result = OM_ENCODING_INVALID;
					}
				}


				if (result == OM_SUCCESS) {
					subContLen = contLen;
				}
				result = dec_Attribute_ASN1Module_SupportedFeatures_content(contBuffer,
								subContLen,
								1 /* explicit form */,
								tagValue,
								tagClass,
								tagForm,
								value,
								&subDecLen);

				if (result == OM_SUCCESS) {
					contLen -= subDecLen;
				}
			}

			/* Release buffers */
			ca_buff_release(contBuffer);

			if (contLen != 0)
				result = OM_ENCODING_INVALID;

			return result;
		}

		default : {
			return OM_ENCODING_INVALID;
		}
	}
}



/* =============== Type CMIP_1_ObjectClass =============== */

void print_CMIP_1_ObjectClass(FILE *f, CMIP_1_ObjectClass *value, unsigned short indent)
{
	switch(value->choice) {

		case CMIP_1_ObjectClass_globalForm : {
			fprintf(f, "globalForm	");
			ca_ber_print_ObjectId(f, &(value->value.globalForm));
			break;
		}

		case CMIP_1_ObjectClass_localForm : {
			fprintf(f, "localForm	");
			ca_ber_print_Integer(f, &(value->value.localForm));
			break;
		}

	}
}




/* =============== Type Attribute_ASN1Module_Packages =============== */

void print_Attribute_ASN1Module_Packages(FILE *f, Attribute_ASN1Module_Packages *value, unsigned short indent)
{
	Attribute_ASN1Module_PackagesNode *item = NULL;

	fprintf(f, "{\n");
	indent++;
	indent++;
	for (item = value->head; item != NULL; ) {
		ca_ber_Indent(f, indent);
		ca_ber_print_ObjectId(f, &(item->elt));
		if (item = item->next)
			printf(",");
			printf("\n");
	}
	indent--;
	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type Attribute_ASN1Module_Allomorphs =============== */

void print_Attribute_ASN1Module_Allomorphs(FILE *f, Attribute_ASN1Module_Allomorphs *value, unsigned short indent)
{
	Attribute_ASN1Module_AllomorphsNode *item = NULL;

	fprintf(f, "{\n");
	indent++;
	indent++;
	for (item = value->head; item != NULL; ) {
		ca_ber_Indent(f, indent);
		print_Attribute_ASN1Module_ObjectClass(f, &(item->elt), indent);
		if (item = item->next)
			printf(",");
			printf("\n");
	}
	indent--;
	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type CFSObject_ASN1module_PermissionBits =============== */

void print_CFSObject_ASN1module_PermissionBits(FILE *f, CFSObject_ASN1module_PermissionBits *value, unsigned short indent)
{
	fprintf(f, "{\n");
	indent++;
/*
** Member read
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "read	");
	ca_ber_print_Boolean(f, &(value->read));
	fprintf(f, ",");
	fprintf(f, "\n");
/*
** Member write
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "write	");
	ca_ber_print_Boolean(f, &(value->write));
	fprintf(f, ",");
	fprintf(f, "\n");
/*
** Member execute
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "execute	");
	ca_ber_print_Boolean(f, &(value->execute));
	fprintf(f, "\n");

	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type CFSObject_ASN1module_FilePermissions =============== */

void print_CFSObject_ASN1module_FilePermissions(FILE *f, CFSObject_ASN1module_FilePermissions *value, unsigned short indent)
{
	fprintf(f, "{\n");
	indent++;
/*
** Member user
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "user	");
	print_CFSObject_ASN1module_PermissionBits(f, &(value->user), indent);
	fprintf(f, ",");
	fprintf(f, "\n");
/*
** Member group
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "group	");
	print_CFSObject_ASN1module_PermissionBits(f, &(value->group), indent);
	fprintf(f, ",");
	fprintf(f, "\n");
/*
** Member other
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "other	");
	print_CFSObject_ASN1module_PermissionBits(f, &(value->other), indent);
	fprintf(f, "\n");

	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type Attribute_ASN1Module_SystemId =============== */

void print_Attribute_ASN1Module_SystemId(FILE *f, Attribute_ASN1Module_SystemId *value, unsigned short indent)
{
	switch(value->choice) {

		case Attribute_ASN1Module_SystemId_name : {
			fprintf(f, "name	");
			print_GraphicString(f, &(value->value.name), indent);
			break;
		}

		case Attribute_ASN1Module_SystemId_number : {
			fprintf(f, "number	");
			ca_ber_print_Integer(f, &(value->value.number));
			break;
		}

		case Attribute_ASN1Module_SystemId_nothing : {
			fprintf(f, "nothing	");
			ca_ber_print_Null(f);
			break;
		}

	}
}




/* =============== Type InformationFramework_AttributeValueAssertion =============== */

void print_InformationFramework_AttributeValueAssertion(FILE *f, InformationFramework_AttributeValueAssertion *value, unsigned short indent)
{
	fprintf(f, "{\n");
	indent++;
/*
** Member unnamedfield1
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "unnamedfield1	");
	print_InformationFramework_AttributeType(f, &(value->unnamedfield1), indent);
	fprintf(f, ",");
	fprintf(f, "\n");
/*
** Member unnamedfield2
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "unnamedfield2	");
	print_InformationFramework_AttributeValue(f, &(value->unnamedfield2), indent);
	fprintf(f, "\n");

	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type InformationFramework_RelativeDistinguishedName =============== */

void print_InformationFramework_RelativeDistinguishedName(FILE *f, InformationFramework_RelativeDistinguishedName *value, unsigned short indent)
{
	InformationFramework_RelativeDistinguishedNameNode *item = NULL;

	fprintf(f, "{\n");
	indent++;
	indent++;
	for (item = value->head; item != NULL; ) {
		ca_ber_Indent(f, indent);
		print_InformationFramework_AttributeValueAssertion(f, &(item->elt), indent);
		if (item = item->next)
			printf(",");
			printf("\n");
	}
	indent--;
	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type InformationFramework_RDNSequence =============== */

void print_InformationFramework_RDNSequence(FILE *f, InformationFramework_RDNSequence *value, unsigned short indent)
{
	InformationFramework_RDNSequenceNode *item = NULL;

	fprintf(f, "{\n");
	indent++;
	indent++;
	for (item = value->head; item != NULL; ) {
		ca_ber_Indent(f, indent);
		print_InformationFramework_RelativeDistinguishedName(f, &(item->elt), indent);
		if (item = item->next)
			printf(",");
			printf("\n");
	}
	indent--;
	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type Attribute_ASN1Module_SystemTitle =============== */

void print_Attribute_ASN1Module_SystemTitle(FILE *f, Attribute_ASN1Module_SystemTitle *value, unsigned short indent)
{
	switch(value->choice) {

		case Attribute_ASN1Module_SystemTitle_distinguishedName : {
			fprintf(f, "distinguishedName	");
			print_Attribute_ASN1Module_DistinguishedName(f, &(value->value.distinguishedName), indent);
			break;
		}

		case Attribute_ASN1Module_SystemTitle_oid : {
			fprintf(f, "oid	");
			ca_ber_print_ObjectId(f, &(value->value.oid));
			break;
		}

		case Attribute_ASN1Module_SystemTitle_nothing : {
			fprintf(f, "nothing	");
			ca_ber_print_Null(f);
			break;
		}

	}
}




/* =============== Type Attribute_ASN1Module_SupportedFeatures_SeqT =============== */

void print_Attribute_ASN1Module_SupportedFeatures_SeqT(FILE *f, Attribute_ASN1Module_SupportedFeatures_SeqT *value, unsigned short indent)
{
	fprintf(f, "{\n");
	indent++;
/*
** Member featureIdentifier
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "featureIdentifier	");
	ca_ber_print_ObjectId(f, &(value->featureIdentifier));
	fprintf(f, ",");
	fprintf(f, "\n");
/*
** Member featureInfo
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "featureInfo	");
	ca_ber_print_Any(f, &(value->featureInfo));
	fprintf(f, "\n");

	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type Attribute_ASN1Module_SupportedFeatures =============== */

void print_Attribute_ASN1Module_SupportedFeatures(FILE *f, Attribute_ASN1Module_SupportedFeatures *value, unsigned short indent)
{
	Attribute_ASN1Module_SupportedFeaturesNode *item = NULL;

	fprintf(f, "{\n");
	indent++;
	indent++;
	for (item = value->head; item != NULL; ) {
		ca_ber_Indent(f, indent);
		print_Attribute_ASN1Module_SupportedFeatures_SeqT(f, &(item->elt), indent);
		if (item = item->next)
			printf(",");
			printf("\n");
	}
	indent--;
	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type CMIP_1_AttributeId =============== */

void print_CMIP_1_AttributeId(FILE *f, CMIP_1_AttributeId *value, unsigned short indent)
{
	switch(value->choice) {

		case CMIP_1_AttributeId_globalForm : {
			fprintf(f, "globalForm	");
			ca_ber_print_ObjectId(f, &(value->value.globalForm));
			break;
		}

		case CMIP_1_AttributeId_localForm : {
			fprintf(f, "localForm	");
			ca_ber_print_Integer(f, &(value->value.localForm));
			break;
		}

	}
}




/* =============== Type Attribute_ASN1Module_AttributeIdentifierList =============== */

void print_Attribute_ASN1Module_AttributeIdentifierList(FILE *f, Attribute_ASN1Module_AttributeIdentifierList *value, unsigned short indent)
{
	Attribute_ASN1Module_AttributeIdentifierListNode *item = NULL;

	fprintf(f, "{\n");
	indent++;
	indent++;
	for (item = value->head; item != NULL; ) {
		ca_ber_Indent(f, indent);
		print_Attribute_ASN1Module_AttributeId(f, &(item->elt), indent);
		if (item = item->next)
			printf(",");
			printf("\n");
	}
	indent--;
	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type Attribute_ASN1Module_AttributeValueChangeDefinition_SetOf_SeqT =============== */

void print_Attribute_ASN1Module_AttributeValueChangeDefinition_SetOf_SeqT(FILE *f, Attribute_ASN1Module_AttributeValueChangeDefinition_SetOf_SeqT *value, unsigned short indent)
{
	fprintf(f, "{\n");
	indent++;
/*
** Member attributeID
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "attributeID	");
	print_Attribute_ASN1Module_AttributeId(f, &(value->attributeID), indent);
	fprintf(f, ",");
	fprintf(f, "\n");
/*
** Member oldAttributeValue
*/

	if (value->oldAttributeValueIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "oldAttributeValue	");
		ca_ber_print_Any(f, &(value->oldAttributeValue));
		fprintf(f, ",");
		fprintf(f, "\n");
	}
/*
** Member newAttributeValue
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "newAttributeValue	");
	ca_ber_print_Any(f, &(value->newAttributeValue));
	fprintf(f, "\n");

	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type Attribute_ASN1Module_AttributeValueChangeDefinition =============== */

void print_Attribute_ASN1Module_AttributeValueChangeDefinition(FILE *f, Attribute_ASN1Module_AttributeValueChangeDefinition *value, unsigned short indent)
{
	Attribute_ASN1Module_AttributeValueChangeDefinitionNode *item = NULL;

	fprintf(f, "{\n");
	indent++;
	indent++;
	for (item = value->head; item != NULL; ) {
		ca_ber_Indent(f, indent);
		print_Attribute_ASN1Module_AttributeValueChangeDefinition_SetOf_SeqT(f, &(item->elt), indent);
		if (item = item->next)
			printf(",");
			printf("\n");
	}
	indent--;
	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfT =============== */

void print_Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfT(FILE *f, Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfT *value, unsigned short indent)
{
	Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfTNode *item = NULL;

	fprintf(f, "{\n");
	indent++;
	indent++;
	for (item = value->head; item != NULL; ) {
		ca_ber_Indent(f, indent);
		print_Attribute_ASN1Module_NotificationIdentifier(f, &(item->elt), indent);
		if (item = item->next)
			printf(",");
			printf("\n");
	}
	indent--;
	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type CMIP_1_ObjectInstance =============== */

void print_CMIP_1_ObjectInstance(FILE *f, CMIP_1_ObjectInstance *value, unsigned short indent)
{
	switch(value->choice) {

		case CMIP_1_ObjectInstance_distinguishedName : {
			fprintf(f, "distinguishedName	");
			print_CMIP_1_DistinguishedName(f, &(value->value.distinguishedName), indent);
			break;
		}

		case CMIP_1_ObjectInstance_nonSpecificForm : {
			fprintf(f, "nonSpecificForm	");
			ca_ber_print_OctetString(f, &(value->value.nonSpecificForm));
			break;
		}

		case CMIP_1_ObjectInstance_localDistinguishedName : {
			fprintf(f, "localDistinguishedName	");
			print_CMIP_1_RDNSequence(f, &(value->value.localDistinguishedName), indent);
			break;
		}

	}
}




/* =============== Type Attribute_ASN1Module_CorrelatedNotifications_SetOf_SeqT =============== */

void print_Attribute_ASN1Module_CorrelatedNotifications_SetOf_SeqT(FILE *f, Attribute_ASN1Module_CorrelatedNotifications_SetOf_SeqT *value, unsigned short indent)
{
	fprintf(f, "{\n");
	indent++;
/*
** Member correlatedNotifications
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "correlatedNotifications	");
	print_Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfT(f, &(value->correlatedNotifications), indent);
	fprintf(f, ",");
	fprintf(f, "\n");
/*
** Member sourceObjectInst
*/

	if (value->sourceObjectInstIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "sourceObjectInst	");
		print_Attribute_ASN1Module_ObjectInstance(f, &(value->sourceObjectInst), indent);
		fprintf(f, "\n");
	}

	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type Attribute_ASN1Module_CorrelatedNotifications =============== */

void print_Attribute_ASN1Module_CorrelatedNotifications(FILE *f, Attribute_ASN1Module_CorrelatedNotifications *value, unsigned short indent)
{
	Attribute_ASN1Module_CorrelatedNotificationsNode *item = NULL;

	fprintf(f, "{\n");
	indent++;
	indent++;
	for (item = value->head; item != NULL; ) {
		ca_ber_Indent(f, indent);
		print_Attribute_ASN1Module_CorrelatedNotifications_SetOf_SeqT(f, &(item->elt), indent);
		if (item = item->next)
			printf(",");
			printf("\n");
	}
	indent--;
	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type Attribute_ASN1Module_ManagementExtension =============== */

void print_Attribute_ASN1Module_ManagementExtension(FILE *f, Attribute_ASN1Module_ManagementExtension *value, unsigned short indent)
{
	fprintf(f, "{\n");
	indent++;
/*
** Member identifier
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "identifier	");
	ca_ber_print_ObjectId(f, &(value->identifier));
	fprintf(f, ",");
	fprintf(f, "\n");
/*
** Member significance
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "significance	");
	ca_ber_print_Boolean(f, &(value->significance));
	fprintf(f, ",");
	fprintf(f, "\n");
/*
** Member information
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "information	");
	ca_ber_print_Any(f, &(value->information));
	fprintf(f, "\n");

	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type Attribute_ASN1Module_AdditionalInformation =============== */

void print_Attribute_ASN1Module_AdditionalInformation(FILE *f, Attribute_ASN1Module_AdditionalInformation *value, unsigned short indent)
{
	Attribute_ASN1Module_AdditionalInformationNode *item = NULL;

	fprintf(f, "{\n");
	indent++;
	indent++;
	for (item = value->head; item != NULL; ) {
		ca_ber_Indent(f, indent);
		print_Attribute_ASN1Module_ManagementExtension(f, &(item->elt), indent);
		if (item = item->next)
			printf(",");
			printf("\n");
	}
	indent--;
	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type Notification_ASN1Module_AttributeValueChangeInfo =============== */

void print_Notification_ASN1Module_AttributeValueChangeInfo(FILE *f, Notification_ASN1Module_AttributeValueChangeInfo *value, unsigned short indent)
{
	fprintf(f, "{\n");
	indent++;
/*
** Member sourceIndicator
*/

	if (value->sourceIndicatorIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "sourceIndicator	");
		print_Notification_ASN1Module_SourceIndicator(f, &(value->sourceIndicator), indent);
		fprintf(f, ",");
		fprintf(f, "\n");
	}
/*
** Member attributeIdentifierList
*/

	if (value->attributeIdentifierListIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "attributeIdentifierList	");
		print_Notification_ASN1Module_AttributeIdentifierList(f, &(value->attributeIdentifierList), indent);
		fprintf(f, ",");
		fprintf(f, "\n");
	}
/*
** Member attributeValueChangeDefinition
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "attributeValueChangeDefinition	");
	print_Notification_ASN1Module_AttributeValueChangeDefinition(f, &(value->attributeValueChangeDefinition), indent);
	fprintf(f, ",");
	fprintf(f, "\n");
/*
** Member notificationIdentifier
*/

	if (value->notificationIdentifierIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "notificationIdentifier	");
		print_Notification_ASN1Module_NotificationIdentifier(f, &(value->notificationIdentifier), indent);
		fprintf(f, ",");
		fprintf(f, "\n");
	}
/*
** Member correlatedNotifications
*/

	if (value->correlatedNotificationsIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "correlatedNotifications	");
		print_Notification_ASN1Module_CorrelatedNotifications(f, &(value->correlatedNotifications), indent);
		fprintf(f, ",");
		fprintf(f, "\n");
	}
/*
** Member additionalText
*/

	if (value->additionalTextIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "additionalText	");
		print_Notification_ASN1Module_AdditionalText(f, &(value->additionalText), indent);
		fprintf(f, ",");
		fprintf(f, "\n");
	}
/*
** Member additionalInformation
*/

	if (value->additionalInformationIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "additionalInformation	");
		print_Notification_ASN1Module_AdditionalInformation(f, &(value->additionalInformation), indent);
		fprintf(f, "\n");
	}

	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type CMIP_1_Attribute =============== */

void print_CMIP_1_Attribute(FILE *f, CMIP_1_Attribute *value, unsigned short indent)
{
	fprintf(f, "{\n");
	indent++;
/*
** Member attributeId
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "attributeId	");
	print_CMIP_1_AttributeId(f, &(value->attributeId), indent);
	fprintf(f, ",");
	fprintf(f, "\n");
/*
** Member attributeValue
*/

	ca_ber_Indent(f, indent);
	fprintf(f, "attributeValue	");
	ca_ber_print_Any(f, &(value->attributeValue));
	fprintf(f, "\n");

	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type Attribute_ASN1Module_AttributeList =============== */

void print_Attribute_ASN1Module_AttributeList(FILE *f, Attribute_ASN1Module_AttributeList *value, unsigned short indent)
{
	Attribute_ASN1Module_AttributeListNode *item = NULL;

	fprintf(f, "{\n");
	indent++;
	indent++;
	for (item = value->head; item != NULL; ) {
		ca_ber_Indent(f, indent);
		print_Attribute_ASN1Module_Attribute(f, &(item->elt), indent);
		if (item = item->next)
			printf(",");
			printf("\n");
	}
	indent--;
	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}




/* =============== Type Notification_ASN1Module_ObjectInfo =============== */

void print_Notification_ASN1Module_ObjectInfo(FILE *f, Notification_ASN1Module_ObjectInfo *value, unsigned short indent)
{
	fprintf(f, "{\n");
	indent++;
/*
** Member sourceIndicator
*/

	if (value->sourceIndicatorIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "sourceIndicator	");
		print_Notification_ASN1Module_SourceIndicator(f, &(value->sourceIndicator), indent);
		fprintf(f, ",");
		fprintf(f, "\n");
	}
/*
** Member attributeList
*/

	if (value->attributeListIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "attributeList	");
		print_Notification_ASN1Module_AttributeList(f, &(value->attributeList), indent);
		fprintf(f, ",");
		fprintf(f, "\n");
	}
/*
** Member notificationIdentifier
*/

	if (value->notificationIdentifierIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "notificationIdentifier	");
		print_Notification_ASN1Module_NotificationIdentifier(f, &(value->notificationIdentifier), indent);
		fprintf(f, ",");
		fprintf(f, "\n");
	}
/*
** Member correlatedNotifications
*/

	if (value->correlatedNotificationsIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "correlatedNotifications	");
		print_Notification_ASN1Module_CorrelatedNotifications(f, &(value->correlatedNotifications), indent);
		fprintf(f, ",");
		fprintf(f, "\n");
	}
/*
** Member additionalText
*/

	if (value->additionalTextIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "additionalText	");
		print_Notification_ASN1Module_AdditionalText(f, &(value->additionalText), indent);
		fprintf(f, ",");
		fprintf(f, "\n");
	}
/*
** Member additionalInformation
*/

	if (value->additionalInformationIsPresent == OM_TRUE) {
		ca_ber_Indent(f, indent);
		fprintf(f, "additionalInformation	");
		print_Notification_ASN1Module_AdditionalInformation(f, &(value->additionalInformation), indent);
		fprintf(f, "\n");
	}

	indent--;
	ca_ber_Indent(f, indent);
	fprintf(f, "}");
}






/* =============== Type CMIP_1_ObjectClass =============== */

void free_CMIP_1_ObjectClass(CMIP_1_ObjectClass *value)
{
	switch(value->choice) {

		case CMIP_1_ObjectClass_globalForm : {
			ca_ber_free_ObjectId(&(value->value.globalForm));
			break;
		}

		case CMIP_1_ObjectClass_localForm : {
			ca_ber_free_Integer(&(value->value.localForm));
			break;
		}

	}
}




/* =============== Type Attribute_ASN1Module_NameBinding =============== */

void free_Attribute_ASN1Module_NameBinding(Attribute_ASN1Module_NameBinding *value)
{
}




/* =============== Type Attribute_ASN1Module_Packages =============== */

void free_Attribute_ASN1Module_Packages(Attribute_ASN1Module_Packages *value)
{
	Attribute_ASN1Module_PackagesNode *item = NULL;

	for (item = value->head; item != NULL;) {
		Attribute_ASN1Module_PackagesNode *nextItem;
		ca_ber_free_ObjectId(&(item->elt));
		nextItem = item->next;
		ca_ber_freeItem(item);
		item = nextItem;
	}
}




/* =============== Type Attribute_ASN1Module_Allomorphs =============== */

void free_Attribute_ASN1Module_Allomorphs(Attribute_ASN1Module_Allomorphs *value)
{
	Attribute_ASN1Module_AllomorphsNode *item = NULL;

	for (item = value->head; item != NULL;) {
		Attribute_ASN1Module_AllomorphsNode *nextItem;
		free_Attribute_ASN1Module_ObjectClass(&(item->elt));
		nextItem = item->next;
		ca_ber_freeItem(item);
		item = nextItem;
	}
}




/* =============== Type CFSObject_ASN1module_PermissionBits =============== */

void free_CFSObject_ASN1module_PermissionBits(CFSObject_ASN1module_PermissionBits *value)
{
/*
** Member read
*/

	ca_ber_free_Boolean(&(value->read));
/*
** Member write
*/

	ca_ber_free_Boolean(&(value->write));
/*
** Member execute
*/

	ca_ber_free_Boolean(&(value->execute));

}




/* =============== Type CFSObject_ASN1module_FilePermissions =============== */

void free_CFSObject_ASN1module_FilePermissions(CFSObject_ASN1module_FilePermissions *value)
{
/*
** Member user
*/

	free_CFSObject_ASN1module_PermissionBits(&(value->user));
/*
** Member group
*/

	free_CFSObject_ASN1module_PermissionBits(&(value->group));
/*
** Member other
*/

	free_CFSObject_ASN1module_PermissionBits(&(value->other));

}




/* =============== Type Attribute_ASN1Module_SystemId =============== */

void free_Attribute_ASN1Module_SystemId(Attribute_ASN1Module_SystemId *value)
{
	switch(value->choice) {

		case Attribute_ASN1Module_SystemId_name : {
			free_GraphicString(&(value->value.name));
			break;
		}

		case Attribute_ASN1Module_SystemId_number : {
			ca_ber_free_Integer(&(value->value.number));
			break;
		}

		case Attribute_ASN1Module_SystemId_nothing : {
			ca_ber_free_Null();
			break;
		}

	}
}




/* =============== Type InformationFramework_AttributeType =============== */

void free_InformationFramework_AttributeType(InformationFramework_AttributeType *value)
{
}




/* =============== Type InformationFramework_AttributeValueAssertion =============== */

void free_InformationFramework_AttributeValueAssertion(InformationFramework_AttributeValueAssertion *value)
{
/*
** Member unnamedfield1
*/

	free_InformationFramework_AttributeType(&(value->unnamedfield1));
/*
** Member unnamedfield2
*/

	free_InformationFramework_AttributeValue(&(value->unnamedfield2));

}




/* =============== Type InformationFramework_RelativeDistinguishedName =============== */

void free_InformationFramework_RelativeDistinguishedName(InformationFramework_RelativeDistinguishedName *value)
{
	InformationFramework_RelativeDistinguishedNameNode *item = NULL;

	for (item = value->head; item != NULL;) {
		InformationFramework_RelativeDistinguishedNameNode *nextItem;
		free_InformationFramework_AttributeValueAssertion(&(item->elt));
		nextItem = item->next;
		ca_ber_freeItem(item);
		item = nextItem;
	}
}




/* =============== Type InformationFramework_RDNSequence =============== */

void free_InformationFramework_RDNSequence(InformationFramework_RDNSequence *value)
{
	InformationFramework_RDNSequenceNode *item = NULL;

	for (item = value->head; item != NULL;) {
		InformationFramework_RDNSequenceNode *nextItem;
		free_InformationFramework_RelativeDistinguishedName(&(item->elt));
		nextItem = item->next;
		ca_ber_freeItem(item);
		item = nextItem;
	}
}




/* =============== Type Attribute_ASN1Module_SystemTitle =============== */

void free_Attribute_ASN1Module_SystemTitle(Attribute_ASN1Module_SystemTitle *value)
{
	switch(value->choice) {

		case Attribute_ASN1Module_SystemTitle_distinguishedName : {
			free_Attribute_ASN1Module_DistinguishedName(&(value->value.distinguishedName));
			break;
		}

		case Attribute_ASN1Module_SystemTitle_oid : {
			ca_ber_free_ObjectId(&(value->value.oid));
			break;
		}

		case Attribute_ASN1Module_SystemTitle_nothing : {
			ca_ber_free_Null();
			break;
		}

	}
}




/* =============== Type Attribute_ASN1Module_OperationalState =============== */

void free_Attribute_ASN1Module_OperationalState(Attribute_ASN1Module_OperationalState *value)
{
}




/* =============== Type Attribute_ASN1Module_UsageState =============== */

void free_Attribute_ASN1Module_UsageState(Attribute_ASN1Module_UsageState *value)
{
}




/* =============== Type Attribute_ASN1Module_AdministrativeState =============== */

void free_Attribute_ASN1Module_AdministrativeState(Attribute_ASN1Module_AdministrativeState *value)
{
}




/* =============== Type Attribute_ASN1Module_SupportedFeatures_SeqT =============== */

void free_Attribute_ASN1Module_SupportedFeatures_SeqT(Attribute_ASN1Module_SupportedFeatures_SeqT *value)
{
/*
** Member featureIdentifier
*/

	ca_ber_free_ObjectId(&(value->featureIdentifier));
/*
** Member featureInfo
*/

	ca_ber_free_Any(&(value->featureInfo));

}




/* =============== Type Attribute_ASN1Module_SupportedFeatures =============== */

void free_Attribute_ASN1Module_SupportedFeatures(Attribute_ASN1Module_SupportedFeatures *value)
{
	Attribute_ASN1Module_SupportedFeaturesNode *item = NULL;

	for (item = value->head; item != NULL;) {
		Attribute_ASN1Module_SupportedFeaturesNode *nextItem;
		free_Attribute_ASN1Module_SupportedFeatures_SeqT(&(item->elt));
		nextItem = item->next;
		ca_ber_freeItem(item);
		item = nextItem;
	}
}




/* =============== Type Attribute_ASN1Module_SourceIndicator =============== */

void free_Attribute_ASN1Module_SourceIndicator(Attribute_ASN1Module_SourceIndicator *value)
{
}




/* =============== Type CMIP_1_AttributeId =============== */

void free_CMIP_1_AttributeId(CMIP_1_AttributeId *value)
{
	switch(value->choice) {

		case CMIP_1_AttributeId_globalForm : {
			ca_ber_free_ObjectId(&(value->value.globalForm));
			break;
		}

		case CMIP_1_AttributeId_localForm : {
			ca_ber_free_Integer(&(value->value.localForm));
			break;
		}

	}
}




/* =============== Type Attribute_ASN1Module_AttributeIdentifierList =============== */

void free_Attribute_ASN1Module_AttributeIdentifierList(Attribute_ASN1Module_AttributeIdentifierList *value)
{
	Attribute_ASN1Module_AttributeIdentifierListNode *item = NULL;

	for (item = value->head; item != NULL;) {
		Attribute_ASN1Module_AttributeIdentifierListNode *nextItem;
		free_Attribute_ASN1Module_AttributeId(&(item->elt));
		nextItem = item->next;
		ca_ber_freeItem(item);
		item = nextItem;
	}
}




/* =============== Type Attribute_ASN1Module_AttributeValueChangeDefinition_SetOf_SeqT =============== */

void free_Attribute_ASN1Module_AttributeValueChangeDefinition_SetOf_SeqT(Attribute_ASN1Module_AttributeValueChangeDefinition_SetOf_SeqT *value)
{
/*
** Member attributeID
*/

	free_Attribute_ASN1Module_AttributeId(&(value->attributeID));
/*
** Member oldAttributeValue
*/

	if (value->oldAttributeValueIsPresent == OM_TRUE) {
		ca_ber_free_Any(&(value->oldAttributeValue));
	}
/*
** Member newAttributeValue
*/

	ca_ber_free_Any(&(value->newAttributeValue));

}




/* =============== Type Attribute_ASN1Module_AttributeValueChangeDefinition =============== */

void free_Attribute_ASN1Module_AttributeValueChangeDefinition(Attribute_ASN1Module_AttributeValueChangeDefinition *value)
{
	Attribute_ASN1Module_AttributeValueChangeDefinitionNode *item = NULL;

	for (item = value->head; item != NULL;) {
		Attribute_ASN1Module_AttributeValueChangeDefinitionNode *nextItem;
		free_Attribute_ASN1Module_AttributeValueChangeDefinition_SetOf_SeqT(&(item->elt));
		nextItem = item->next;
		ca_ber_freeItem(item);
		item = nextItem;
	}
}




/* =============== Type Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfT =============== */

void free_Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfT(Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfT *value)
{
	Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfTNode *item = NULL;

	for (item = value->head; item != NULL;) {
		Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfTNode *nextItem;
		free_Attribute_ASN1Module_NotificationIdentifier(&(item->elt));
		nextItem = item->next;
		ca_ber_freeItem(item);
		item = nextItem;
	}
}




/* =============== Type CMIP_1_ObjectInstance =============== */

void free_CMIP_1_ObjectInstance(CMIP_1_ObjectInstance *value)
{
	switch(value->choice) {

		case CMIP_1_ObjectInstance_distinguishedName : {
			free_CMIP_1_DistinguishedName(&(value->value.distinguishedName));
			break;
		}

		case CMIP_1_ObjectInstance_nonSpecificForm : {
			ca_ber_free_OctetString(&(value->value.nonSpecificForm));
			break;
		}

		case CMIP_1_ObjectInstance_localDistinguishedName : {
			free_CMIP_1_RDNSequence(&(value->value.localDistinguishedName));
			break;
		}

	}
}




/* =============== Type Attribute_ASN1Module_CorrelatedNotifications_SetOf_SeqT =============== */

void free_Attribute_ASN1Module_CorrelatedNotifications_SetOf_SeqT(Attribute_ASN1Module_CorrelatedNotifications_SetOf_SeqT *value)
{
/*
** Member correlatedNotifications
*/

	free_Attribute_ASN1Module_CorrelatedNotifications_SetOf_Seq_correlatedNotifications_SetOfT(&(value->correlatedNotifications));
/*
** Member sourceObjectInst
*/

	if (value->sourceObjectInstIsPresent == OM_TRUE) {
		free_Attribute_ASN1Module_ObjectInstance(&(value->sourceObjectInst));
	}

}




/* =============== Type Attribute_ASN1Module_CorrelatedNotifications =============== */

void free_Attribute_ASN1Module_CorrelatedNotifications(Attribute_ASN1Module_CorrelatedNotifications *value)
{
	Attribute_ASN1Module_CorrelatedNotificationsNode *item = NULL;

	for (item = value->head; item != NULL;) {
		Attribute_ASN1Module_CorrelatedNotificationsNode *nextItem;
		free_Attribute_ASN1Module_CorrelatedNotifications_SetOf_SeqT(&(item->elt));
		nextItem = item->next;
		ca_ber_freeItem(item);
		item = nextItem;
	}
}




/* =============== Type Attribute_ASN1Module_ManagementExtension =============== */

void free_Attribute_ASN1Module_ManagementExtension(Attribute_ASN1Module_ManagementExtension *value)
{
/*
** Member identifier
*/

	ca_ber_free_ObjectId(&(value->identifier));
/*
** Member significance
*/

	ca_ber_free_Boolean(&(value->significance));
/*
** Member information
*/

	ca_ber_free_Any(&(value->information));

}




/* =============== Type Attribute_ASN1Module_AdditionalInformation =============== */

void free_Attribute_ASN1Module_AdditionalInformation(Attribute_ASN1Module_AdditionalInformation *value)
{
	Attribute_ASN1Module_AdditionalInformationNode *item = NULL;

	for (item = value->head; item != NULL;) {
		Attribute_ASN1Module_AdditionalInformationNode *nextItem;
		free_Attribute_ASN1Module_ManagementExtension(&(item->elt));
		nextItem = item->next;
		ca_ber_freeItem(item);
		item = nextItem;
	}
}




/* =============== Type Notification_ASN1Module_AttributeValueChangeInfo =============== */

void free_Notification_ASN1Module_AttributeValueChangeInfo(Notification_ASN1Module_AttributeValueChangeInfo *value)
{
/*
** Member sourceIndicator
*/

	if (value->sourceIndicatorIsPresent == OM_TRUE) {
		free_Notification_ASN1Module_SourceIndicator(&(value->sourceIndicator));
	}
/*
** Member attributeIdentifierList
*/

	if (value->attributeIdentifierListIsPresent == OM_TRUE) {
		free_Notification_ASN1Module_AttributeIdentifierList(&(value->attributeIdentifierList));
	}
/*
** Member attributeValueChangeDefinition
*/

	free_Notification_ASN1Module_AttributeValueChangeDefinition(&(value->attributeValueChangeDefinition));
/*
** Member notificationIdentifier
*/

	if (value->notificationIdentifierIsPresent == OM_TRUE) {
		free_Notification_ASN1Module_NotificationIdentifier(&(value->notificationIdentifier));
	}
/*
** Member correlatedNotifications
*/

	if (value->correlatedNotificationsIsPresent == OM_TRUE) {
		free_Notification_ASN1Module_CorrelatedNotifications(&(value->correlatedNotifications));
	}
/*
** Member additionalText
*/

	if (value->additionalTextIsPresent == OM_TRUE) {
		free_Notification_ASN1Module_AdditionalText(&(value->additionalText));
	}
/*
** Member additionalInformation
*/

	if (value->additionalInformationIsPresent == OM_TRUE) {
		free_Notification_ASN1Module_AdditionalInformation(&(value->additionalInformation));
	}

}




/* =============== Type CMIP_1_Attribute =============== */

void free_CMIP_1_Attribute(CMIP_1_Attribute *value)
{
/*
** Member attributeId
*/

	free_CMIP_1_AttributeId(&(value->attributeId));
/*
** Member attributeValue
*/

	ca_ber_free_Any(&(value->attributeValue));

}




/* =============== Type Attribute_ASN1Module_AttributeList =============== */

void free_Attribute_ASN1Module_AttributeList(Attribute_ASN1Module_AttributeList *value)
{
	Attribute_ASN1Module_AttributeListNode *item = NULL;

	for (item = value->head; item != NULL;) {
		Attribute_ASN1Module_AttributeListNode *nextItem;
		free_Attribute_ASN1Module_Attribute(&(item->elt));
		nextItem = item->next;
		ca_ber_freeItem(item);
		item = nextItem;
	}
}




/* =============== Type Notification_ASN1Module_ObjectInfo =============== */

void free_Notification_ASN1Module_ObjectInfo(Notification_ASN1Module_ObjectInfo *value)
{
/*
** Member sourceIndicator
*/

	if (value->sourceIndicatorIsPresent == OM_TRUE) {
		free_Notification_ASN1Module_SourceIndicator(&(value->sourceIndicator));
	}
/*
** Member attributeList
*/

	if (value->attributeListIsPresent == OM_TRUE) {
		free_Notification_ASN1Module_AttributeList(&(value->attributeList));
	}
/*
** Member notificationIdentifier
*/

	if (value->notificationIdentifierIsPresent == OM_TRUE) {
		free_Notification_ASN1Module_NotificationIdentifier(&(value->notificationIdentifier));
	}
/*
** Member correlatedNotifications
*/

	if (value->correlatedNotificationsIsPresent == OM_TRUE) {
		free_Notification_ASN1Module_CorrelatedNotifications(&(value->correlatedNotifications));
	}
/*
** Member additionalText
*/

	if (value->additionalTextIsPresent == OM_TRUE) {
		free_Notification_ASN1Module_AdditionalText(&(value->additionalText));
	}
/*
** Member additionalInformation
*/

	if (value->additionalInformationIsPresent == OM_TRUE) {
		free_Notification_ASN1Module_AdditionalInformation(&(value->additionalInformation));
	}

}






/* =============== Type CMIP_1_ObjectClass =============== */

/* Comparing function for type CMIP_1_ObjectClass */

OM_boolean cmp_CMIP_1_ObjectClass(CMIP_1_ObjectClass *value1, CMIP_1_ObjectClass *value2, unsigned int operator)
{
	OM_boolean result = OM_TRUE;

	if (value1->choice != value2->choice)
		return OM_FALSE;

	switch(value1->choice) {

		case CMIP_1_ObjectClass_globalForm : {
			result = ca_ber_cmp_ObjectId(&(value1->value.globalForm), &(value2->value.globalForm), operator);
			break;
		}

		case CMIP_1_ObjectClass_localForm : {
			result = ca_ber_cmp_Integer(&(value1->value.localForm), &(value2->value.localForm), operator);
			break;
		}

	}
	return result;
}




/* =============== Type Attribute_ASN1Module_Packages =============== */

/* Comparing function for type Attribute_ASN1Module_Packages */

OM_boolean cmp_Attribute_ASN1Module_Packages(Attribute_ASN1Module_Packages *value1, Attribute_ASN1Module_Packages *value2, unsigned int operator)
{
	OM_boolean result = OM_TRUE;

	Attribute_ASN1Module_PackagesNode *item1 = NULL;
	Attribute_ASN1Module_PackagesNode *item2 = NULL;

	if (operator == MP_SUPERSET_OF) {
		result = cmp_Attribute_ASN1Module_Packages(value2, value1, MP_SUBSET_OF);
		result = (result == OM_TRUE) ? OM_FALSE : OM_TRUE;
	}
	for (item1 = value1->head, item2 = value2->head;
		item1 != NULL && item2 != NULL;
		item1 = item1->next, item2 = item2->next);

	switch(operator) {
		case MP_EQUALITY:
			if (item1 != NULL || item2 != NULL)
				return OM_FALSE;
			break;

		case MP_SUBSET_OF:
			if (item1 != NULL)
				return OM_FALSE;
			break;

	}

	operator = MP_EQUALITY;
	for (item1 = value1->head; item1 != NULL; item1 = item1->next) {
		for (item2 = value2->head; item2 != NULL; item2 = item2->next) {
			result = ca_ber_cmp_ObjectId(&(item1->elt), &(item2->elt), operator);
			if (result == OM_TRUE)
				break;
		}
		if (result == OM_FALSE)
			break;
	}

	return result;
}




/* =============== Type Attribute_ASN1Module_Allomorphs =============== */

/* Comparing function for type Attribute_ASN1Module_Allomorphs */

OM_boolean cmp_Attribute_ASN1Module_Allomorphs(Attribute_ASN1Module_Allomorphs *value1, Attribute_ASN1Module_Allomorphs *value2, unsigned int operator)
{
	OM_boolean result = OM_TRUE;

	Attribute_ASN1Module_AllomorphsNode *item1 = NULL;
	Attribute_ASN1Module_AllomorphsNode *item2 = NULL;

	if (operator == MP_SUPERSET_OF) {
		result = cmp_Attribute_ASN1Module_Allomorphs(value2, value1, MP_SUBSET_OF);
		result = (result == OM_TRUE) ? OM_FALSE : OM_TRUE;
	}
	for (item1 = value1->head, item2 = value2->head;
		item1 != NULL && item2 != NULL;
		item1 = item1->next, item2 = item2->next);

	switch(operator) {
		case MP_EQUALITY:
			if (item1 != NULL || item2 != NULL)
				return OM_FALSE;
			break;

		case MP_SUBSET_OF:
			if (item1 != NULL)
				return OM_FALSE;
			break;

	}

	operator = MP_EQUALITY;
	for (item1 = value1->head; item1 != NULL; item1 = item1->next) {
		for (item2 = value2->head; item2 != NULL; item2 = item2->next) {
			result = cmp_Attribute_ASN1Module_ObjectClass(&(item1->elt), &(item2->elt), operator);
			if (result == OM_TRUE)
				break;
		}
		if (result == OM_FALSE)
			break;
	}

	return result;
}




/* =============== Type CFSObject_ASN1module_PermissionBits =============== */

/* Comparing function for type CFSObject_ASN1module_PermissionBits */

OM_boolean cmp_CFSObject_ASN1module_PermissionBits(CFSObject_ASN1module_PermissionBits *value1, CFSObject_ASN1module_PermissionBits *value2, unsigned int operator)
{
	OM_boolean result = OM_TRUE;

/*
** Member read
*/


	if (result == OM_TRUE) {
		result = ca_ber_cmp_Boolean(&(value1->read), &(value2->read), operator);
	}
/*
** Member write
*/


	if (result == OM_TRUE) {
		result = ca_ber_cmp_Boolean(&(value1->write), &(value2->write), operator);
	}
/*
** Member execute
*/


	if (result == OM_TRUE) {
		result = ca_ber_cmp_Boolean(&(value1->execute), &(value2->execute), operator);
	}

	return result;
}




/* =============== Type CFSObject_ASN1module_FilePermissions =============== */

/* Comparing function for type CFSObject_ASN1module_FilePermissions */

OM_boolean cmp_CFSObject_ASN1module_FilePermissions(CFSObject_ASN1module_FilePermissions *value1, CFSObject_ASN1module_FilePermissions *value2, unsigned int operator)
{
	OM_boolean result = OM_TRUE;

/*
** Member user
*/


	if (result == OM_TRUE) {
		result = cmp_CFSObject_ASN1module_PermissionBits(&(value1->user), &(value2->user), operator);
	}
/*
** Member group
*/


	if (result == OM_TRUE) {
		result = cmp_CFSObject_ASN1module_PermissionBits(&(value1->group), &(value2->group), operator);
	}
/*
** Member other
*/


	if (result == OM_TRUE) {
		result = cmp_CFSObject_ASN1module_PermissionBits(&(value1->other), &(value2->other), operator);
	}

	return result;
}




/* =============== Type Attribute_ASN1Module_SystemId =============== */

/* Comparing function for type Attribute_ASN1Module_SystemId */

OM_boolean cmp_Attribute_ASN1Module_SystemId(Attribute_ASN1Module_SystemId *value1, Attribute_ASN1Module_SystemId *value2, unsigned int operator)
{
	OM_boolean result = OM_TRUE;

	if (value1->choice != value2->choice)
		return OM_FALSE;

	switch(value1->choice) {

		case Attribute_ASN1Module_SystemId_name : {
			result = cmp_GraphicString(&(value1->value.name), &(value2->value.name), operator);
			break;
		}

		case Attribute_ASN1Module_SystemId_number : {
			result = ca_ber_cmp_Integer(&(value1->value.number), &(value2->value.number), operator);
			break;
		}

		case Attribute_ASN1Module_SystemId_nothing : {
			result = ca_ber_cmp_Null(operator);
			break;
		}

	}
	return result;
}




/* =============== Type InformationFramework_AttributeValueAssertion =============== */

/* Comparing function for type InformationFramework_AttributeValueAssertion */

OM_boolean cmp_InformationFramework_AttributeValueAssertion(InformationFramework_AttributeValueAssertion *value1, InformationFramework_AttributeValueAssertion *value2, unsigned int operator)
{
	OM_boolean result = OM_TRUE;

/*
** Member unnamedfield1
*/


	if (result == OM_TRUE) {
		result = cmp_InformationFramework_AttributeType(&(value1->unnamedfield1), &(value2->unnamedfield1), operator);
	}
/*
** Member unnamedfield2
*/


	if (result == OM_TRUE) {
		result = cmp_InformationFramework_AttributeValue(&(value1->unnamedfield2), &(value2->unnamedfield2), operator);
	}

	return result;
}




/* =============== Type InformationFramework_RelativeDistinguishedName =============== */

/* Comparing function for type InformationFramework_RelativeDistinguishedName */

OM_boolean cmp_InformationFramework_RelativeDistinguishedName(InformationFramework_RelativeDistinguishedName *value1, InformationFramework_RelativeDistinguishedName *value2, unsigned int operator)
{
	OM_boolean result = OM_TRUE;

	InformationFramework_RelativeDistinguishedNameNode *item1 = NULL;
	InformationFramework_RelativeDistinguishedNameNode *item2 = NULL;

	if (operator == MP_SUPERSET_OF) {
		result = cmp_InformationFramework_RelativeDistinguishedName(value2, value1, MP_SUBSET_OF);
		result = (result == OM_TRUE) ? OM_FALSE : OM_TRUE;
	}
	for (item1 = value1->head, item2 = value2->head;
		item1 != NULL && item2 != NULL;
		item1 = item1->next, item2 = item2->next);

	switch(operator) {
		case MP_EQUALITY:
			if (item1 != NULL || item2 != NULL)
				return OM_FALSE;
			break;

		case MP_SUBSET_OF:
			if (item1 != NULL)
				return OM_FALSE;
			break;

	}

	operator = MP_EQUALITY;
	for (item1 = value1->head; item1 != NULL; item1 = item1->next) {
		for (item2 = value2->head; item2 != NULL; item2 = item2->next) {
			result = cmp_InformationFramework_AttributeValueAssertion(&(item1->elt), &(item2->elt), operator);
			if (result == OM_TRUE)
				break;
		}
		if (result == OM_FALSE)
			break;
	}

	return result;
}




/* =============== Type InformationFramework_RDNSequence =============== */

/* Comparing function for type InformationFramework_RDNSequence */

OM_boolean cmp_InformationFramework_RDNSequence(InformationFramework_RDNSequence *value1, InformationFramework_RDNSequence *value2, unsigned int operator)
{
	OM_boolean result = OM_TRUE;

	InformationFramework_RDNSequenceNode *item1 = NULL;
	InformationFramework_RDNSequenceNode *item2 = NULL;

	if (operator == MP_SUPERSET_OF) {
		result = cmp_InformationFramework_RDNSequence(value2, value1, MP_SUBSET_OF);
		result = (result == OM_TRUE) ? OM_FALSE : OM_TRUE;
	}
	for (item1 = value1->head, item2 = value2->head;
		item1 != NULL && item2 != NULL;
		item1 = item1->next, item2 = item2->next);

	switch(operator) {
		case MP_EQUALITY:
			if (item1 != NULL || item2 != NULL)
				return OM_FALSE;
			break;

		case MP_SUBSET_OF:
			if (item1 != NULL)
				return OM_FALSE;
			break;

	}

	operator = MP_EQUALITY;
	for (item1 = value1->head; item1 != NULL; item1 = item1->next) {
		for (item2 = value2->head; item2 != NULL; item2 = item2->next) {
			result = cmp_InformationFramework_RelativeDistinguishedName(&(item1->elt), &(item2->elt), operator);
			if (result == OM_TRUE)
				break;
		}
		if (result == OM_FALSE)
			break;
	}

	return result;
}




/* =============== Type Attribute_ASN1Module_SystemTitle =============== */

/* Comparing function for type Attribute_ASN1Module_SystemTitle */

OM_boolean cmp_Attribute_ASN1Module_SystemTitle(Attribute_ASN1Module_SystemTitle *value1, Attribute_ASN1Module_SystemTitle *value2, unsigned int operator)
{
	OM_boolean result = OM_TRUE;

	if (value1->choice != value2->choice)
		return OM_FALSE;

	switch(value1->choice) {

		case Attribute_ASN1Module_SystemTitle_distinguishedName : {
			result = cmp_Attribute_ASN1Module_DistinguishedName(&(value1->value.distinguishedName), &(value2->value.distinguishedName), operator);
			break;
		}

		case Attribute_ASN1Module_SystemTitle_oid : {
			result = ca_ber_cmp_ObjectId(&(value1->value.oid), &(value2->value.oid), operator);
			break;
		}

		case Attribute_ASN1Module_SystemTitle_nothing : {
			result = ca_ber_cmp_Null(operator);
			break;
		}

	}
	return result;
}




/* =============== Type Attribute_ASN1Module_SupportedFeatures_SeqT =============== */

/* Comparing function for type Attribute_ASN1Module_SupportedFeatures_SeqT */

OM_boolean cmp_Attribute_ASN1Module_SupportedFeatures_SeqT(Attribute_ASN1Module_SupportedFeatures_SeqT *value1, Attribute_ASN1Module_SupportedFeatures_SeqT *value2, unsigned int operator)
{
	OM_boolean result = OM_TRUE;

/*
** Member featureIdentifier
*/


	if (result == OM_TRUE) {
		result = ca_ber_cmp_ObjectId(&(value1->featureIdentifier), &(value2->featureIdentifier), operator);
	}
/*
** Member featureInfo
*/


	if (result == OM_TRUE) {
		result = ca_ber_cmp_Any(&(value1->featureInfo), &(value2->featureInfo), operator);
	}

	return result;
}




/* =============== Type Attribute_ASN1Module_SupportedFeatures =============== */

/* Comparing function for type Attribute_ASN1Module_SupportedFeatures */

OM_boolean cmp_Attribute_ASN1Module_SupportedFeatures(Attribute_ASN1Module_SupportedFeatures *value1, Attribute_ASN1Module_SupportedFeatures *value2, unsigned int operator)
{
	OM_boolean result = OM_TRUE;

	Attribute_ASN1Module_SupportedFeaturesNode *item1 = NULL;
	Attribute_ASN1Module_SupportedFeaturesNode *item2 = NULL;

	if (operator == MP_SUPERSET_OF) {
		result = cmp_Attribute_ASN1Module_SupportedFeatures(value2, value1, MP_SUBSET_OF);
		result = (result == OM_TRUE) ? OM_FALSE : OM_TRUE;
	}
	for (item1 = value1->head, item2 = value2->head;
		item1 != NULL && item2 != NULL;
		item1 = item1->next, item2 = item2->next);

	switch(operator) {
		case MP_EQUALITY:
			if (item1 != NULL || item2 != NULL)
				return OM_FALSE;
			break;

		case MP_SUBSET_OF:
			if (item1 != NULL)
				return OM_FALSE;
			break;

	}

	operator = MP_EQUALITY;
	for (item1 = value1->head; item1 != NULL; item1 = item1->next) {
		for (item2 = value2->head; item2 != NULL; item2 = item2->next) {
			result = cmp_Attribute_ASN1Module_SupportedFeatures_SeqT(&(item1->elt), &(item2->elt), operator);
			if (result == OM_TRUE)
				break;
		}
		if (result == OM_FALSE)
			break;
	}

	return result;
}



